//
// Copyright 1998-2008 Sun Microsystems, Inc.  All Rights Reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
// CA 95054 USA or visit www.sun.com if you need additional information or
// have any questions.
//
//

// Machine Generated File.  Do Not Edit!


#include "incls/_precompiled.incl"
#include "incls/_ad_sparc.cpp.incl"


//SourceForm

#define __ _masm.

// tertiary op of a LoadP or StoreP encoding
#define REGP_OP true

static FloatRegister reg_to_SingleFloatRegister_object(int register_encoding);
static FloatRegister reg_to_DoubleFloatRegister_object(int register_encoding);
static Register reg_to_register_object(int register_encoding);

// Used by the DFA in dfa_sparc.cpp.
// Check for being able to use a V9 branch-on-register.  Requires a
// compare-vs-zero, equal/not-equal, of a value which was zero- or sign-
// extended.  Doesn't work following an integer ADD, for example, because of
// overflow (-1 incremented yields 0 plus a carry in the high-order word).  On
// 32-bit V9 systems, interrupts currently blow away the high-order 32 bits and
// replace them with zero, which could become sign-extension in a different OS
// release.  There's no obvious reason why an interrupt will ever fill these
// bits with non-zero junk (the registers are reloaded with standard LD
// instructions which either zero-fill or sign-fill).
bool can_branch_register( Node *bol, Node *cmp ) {
  if( !BranchOnRegister ) return false;
#ifdef _LP64
  if( cmp->Opcode() == Op_CmpP )
    return true;  // No problems with pointer compares
#endif
  if( cmp->Opcode() == Op_CmpL )
    return true;  // No problems with long compares

  if( !SparcV9RegsHiBitsZero ) return false;
  if( bol->as_Bool()->_test._test != BoolTest::ne &&
      bol->as_Bool()->_test._test != BoolTest::eq )
     return false;

  // Check for comparing against a 'safe' value.  Any operation which
  // clears out the high word is safe.  Thus, loads and certain shifts
  // are safe, as are non-negative constants.  Any operation which
  // preserves zero bits in the high word is safe as long as each of its
  // inputs are safe.  Thus, phis and bitwise booleans are safe if their
  // inputs are safe.  At present, the only important case to recognize
  // seems to be loads.  Constants should fold away, and shifts &
  // logicals can use the 'cc' forms.
  Node *x = cmp->in(1);
  if( x->is_Load() ) return true;
  if( x->is_Phi() ) {
    for( uint i = 1; i < x->req(); i++ )
      if( !x->in(i)->is_Load() )
        return false;
    return true;
  }
  return false;
}

// ****************************************************************************

// REQUIRED FUNCTIONALITY

// !!!!! Special hack to get all type of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.
//       The "return address" is the address of the call instruction, plus 8.

int MachCallStaticJavaNode::ret_addr_offset() {
  return NativeCall::instruction_size;  // call; delay slot
}

int MachCallDynamicJavaNode::ret_addr_offset() {
  int vtable_index = this->_vtable_index;
  if (vtable_index < 0) {
    // must be invalid_vtable_index, not nonvirtual_vtable_index
    assert(vtable_index == methodOopDesc::invalid_vtable_index, "correct sentinel value");
    return (NativeMovConstReg::instruction_size +
           NativeCall::instruction_size);  // sethi; setlo; call; delay slot
  } else {
    assert(!UseInlineCaches, "expect vtable calls only if not using ICs");
    int entry_offset = instanceKlass::vtable_start_offset() + vtable_index*vtableEntry::size();
    int v_off = entry_offset*wordSize + vtableEntry::method_offset_in_bytes();
    int klass_load_size;
    if (UseCompressedOops) {
      klass_load_size = 3*BytesPerInstWord; // see MacroAssembler::load_klass()
    } else {
      klass_load_size = 1*BytesPerInstWord;
    }
    if( Assembler::is_simm13(v_off) ) {
      return klass_load_size +
             (2*BytesPerInstWord +           // ld_ptr, ld_ptr
             NativeCall::instruction_size);  // call; delay slot
    } else {
      return klass_load_size +
             (4*BytesPerInstWord +           // set_hi, set, ld_ptr, ld_ptr
             NativeCall::instruction_size);  // call; delay slot
    }
  }
}

int MachCallRuntimeNode::ret_addr_offset() {
#ifdef _LP64
  return NativeFarCall::instruction_size;  // farcall; delay slot
#else
  return NativeCall::instruction_size;  // call; delay slot
#endif
}

// Indicate if the safepoint node needs the polling page as an input.
// Since Sparc does not have absolute addressing, it does.
bool SafePointNode::needs_polling_address_input() {
  return true;
}

// emit an interrupt that is caught by the debugger (for debugging compiler)
void emit_break(CodeBuffer &cbuf) {
  MacroAssembler _masm(&cbuf);
  __ breakpoint_trap();
}

#ifndef PRODUCT
void MachBreakpointNode::format( PhaseRegAlloc *, outputStream *st ) const {
  st->print("TA");
}
#endif

void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  emit_break(cbuf);
}

uint MachBreakpointNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

// Traceable jump
void  emit_jmpl(CodeBuffer &cbuf, int jump_target) {
  MacroAssembler _masm(&cbuf);
  Register rdest = reg_to_register_object(jump_target);
  __ JMP(rdest, 0);
  __ delayed()->nop();
}

// Traceable jump and set exception pc
void  emit_jmpl_set_exception_pc(CodeBuffer &cbuf, int jump_target) {
  MacroAssembler _masm(&cbuf);
  Register rdest = reg_to_register_object(jump_target);
  __ JMP(rdest, 0);
  __ delayed()->add(O7, frame::pc_return_offset, Oissuing_pc );
}

void emit_nop(CodeBuffer &cbuf) {
  MacroAssembler _masm(&cbuf);
  __ nop();
}

void emit_illtrap(CodeBuffer &cbuf) {
  MacroAssembler _masm(&cbuf);
  __ illtrap(0);
}


intptr_t get_offset_from_base(const MachNode* n, const TypePtr* atype, int disp32) {
  assert(n->rule() != loadUB_rule, "");

  intptr_t offset = 0;
  const TypePtr *adr_type = TYPE_PTR_SENTINAL;  // Check for base==RegI, disp==immP
  const Node* addr = n->get_base_and_disp(offset, adr_type);
  assert(adr_type == (const TypePtr*)-1, "VerifyOops: no support for sparc operands with base==RegI, disp==immP");
  assert(addr != NULL && addr != (Node*)-1, "invalid addr");
  assert(addr->bottom_type()->isa_oopptr() == atype, "");
  atype = atype->add_offset(offset);
  assert(disp32 == offset, "wrong disp32");
  return atype->_offset;
}


intptr_t get_offset_from_base_2(const MachNode* n, const TypePtr* atype, int disp32) {
  assert(n->rule() != loadUB_rule, "");

  intptr_t offset = 0;
  Node* addr = n->in(2);
  assert(addr->bottom_type()->isa_oopptr() == atype, "");
  if (addr->is_Mach() && addr->as_Mach()->ideal_Opcode() == Op_AddP) {
    Node* a = addr->in(2/*AddPNode::Address*/);
    Node* o = addr->in(3/*AddPNode::Offset*/);
    offset = o->is_Con() ? o->bottom_type()->is_intptr_t()->get_con() : Type::OffsetBot;
    atype = a->bottom_type()->is_ptr()->add_offset(offset);
    assert(atype->isa_oop_ptr(), "still an oop");
  }
  offset = atype->is_ptr()->_offset;
  if (offset != Type::OffsetBot)  offset += disp32;
  return offset;
}

// Standard Sparc opcode form2 field breakdown
static inline void emit2_19(CodeBuffer &cbuf, int f30, int f29, int f25, int f22, int f20, int f19, int f0 ) {
  f0 &= (1<<19)-1;     // Mask displacement to 19 bits
  int op = (f30 << 30) |
           (f29 << 29) |
           (f25 << 25) |
           (f22 << 22) |
           (f20 << 20) |
           (f19 << 19) |
           (f0  <<  0);
  *((int*)(cbuf.code_end())) = op;
  cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
}

// Standard Sparc opcode form2 field breakdown
static inline void emit2_22(CodeBuffer &cbuf, int f30, int f25, int f22, int f0 ) {
  f0 >>= 10;           // Drop 10 bits
  f0 &= (1<<22)-1;     // Mask displacement to 22 bits
  int op = (f30 << 30) |
           (f25 << 25) |
           (f22 << 22) |
           (f0  <<  0);
  *((int*)(cbuf.code_end())) = op;
  cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
}

// Standard Sparc opcode form3 field breakdown
static inline void emit3(CodeBuffer &cbuf, int f30, int f25, int f19, int f14, int f5, int f0 ) {
  int op = (f30 << 30) |
           (f25 << 25) |
           (f19 << 19) |
           (f14 << 14) |
           (f5  <<  5) |
           (f0  <<  0);
  *((int*)(cbuf.code_end())) = op;
  cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
}

// Standard Sparc opcode form3 field breakdown
static inline void emit3_simm13(CodeBuffer &cbuf, int f30, int f25, int f19, int f14, int simm13 ) {
  simm13 &= (1<<13)-1; // Mask to 13 bits
  int op = (f30 << 30) |
           (f25 << 25) |
           (f19 << 19) |
           (f14 << 14) |
           (1   << 13) | // bit to indicate immediate-mode
           (simm13<<0);
  *((int*)(cbuf.code_end())) = op;
  cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
}

static inline void emit3_simm10(CodeBuffer &cbuf, int f30, int f25, int f19, int f14, int simm10 ) {
  simm10 &= (1<<10)-1; // Mask to 10 bits
  emit3_simm13(cbuf,f30,f25,f19,f14,simm10);
}

#ifdef ASSERT
// Helper function for VerifyOops in emit_form3_mem_reg
void verify_oops_warning(const MachNode *n, int ideal_op, int mem_op) {
  warning("VerifyOops encountered unexpected instruction:");
  n->dump(2);
  warning("Instruction has ideal_Opcode==Op_%s and op_ld==Op_%s \n", NodeClassNames[ideal_op], NodeClassNames[mem_op]);
}
#endif


void emit_form3_mem_reg(CodeBuffer &cbuf, const MachNode* n, int primary, int tertiary,
                        int src1_enc, int disp32, int src2_enc, int dst_enc) {

#ifdef ASSERT
  // The following code implements the +VerifyOops feature.
  // It verifies oop values which are loaded into or stored out of
  // the current method activation.  +VerifyOops complements techniques
  // like ScavengeALot, because it eagerly inspects oops in transit,
  // as they enter or leave the stack, as opposed to ScavengeALot,
  // which inspects oops "at rest", in the stack or heap, at safepoints.
  // For this reason, +VerifyOops can sometimes detect bugs very close
  // to their point of creation.  It can also serve as a cross-check
  // on the validity of oop maps, when used toegether with ScavengeALot.

  // It would be good to verify oops at other points, especially
  // when an oop is used as a base pointer for a load or store.
  // This is presently difficult, because it is hard to know when
  // a base address is biased or not.  (If we had such information,
  // it would be easy and useful to make a two-argument version of
  // verify_oop which unbiases the base, and performs verification.)

  assert((uint)tertiary == 0xFFFFFFFF || tertiary == REGP_OP, "valid tertiary");
  bool is_verified_oop_base  = false;
  bool is_verified_oop_load  = false;
  bool is_verified_oop_store = false;
  int tmp_enc = -1;
  if (VerifyOops && src1_enc != R_SP_enc) {
    // classify the op, mainly for an assert check
    int st_op = 0, ld_op = 0;
    switch (primary) {
    case Assembler::stb_op3:  st_op = Op_StoreB; break;
    case Assembler::sth_op3:  st_op = Op_StoreC; break;
    case Assembler::stx_op3:  // may become StoreP or stay StoreI or StoreD0
    case Assembler::stw_op3:  st_op = Op_StoreI; break;
    case Assembler::std_op3:  st_op = Op_StoreL; break;
    case Assembler::stf_op3:  st_op = Op_StoreF; break;
    case Assembler::stdf_op3: st_op = Op_StoreD; break;

    case Assembler::ldsb_op3: ld_op = Op_LoadB; break;
    case Assembler::lduh_op3: ld_op = Op_LoadC; break;
    case Assembler::ldsh_op3: ld_op = Op_LoadS; break;
    case Assembler::ldx_op3:  // may become LoadP or stay LoadI
    case Assembler::ldsw_op3: // may become LoadP or stay LoadI
    case Assembler::lduw_op3: ld_op = Op_LoadI; break;
    case Assembler::ldd_op3:  ld_op = Op_LoadL; break;
    case Assembler::ldf_op3:  ld_op = Op_LoadF; break;
    case Assembler::lddf_op3: ld_op = Op_LoadD; break;
    case Assembler::ldub_op3: ld_op = Op_LoadB; break;
    case Assembler::prefetch_op3: ld_op = Op_LoadI; break;

    default: ShouldNotReachHere();
    }
    if (tertiary == REGP_OP) {
      if      (st_op == Op_StoreI)  st_op = Op_StoreP;
      else if (ld_op == Op_LoadI)   ld_op = Op_LoadP;
      else                          ShouldNotReachHere();
      if (st_op) {
        // a store
        // inputs are (0:control, 1:memory, 2:address, 3:value)
        Node* n2 = n->in(3);
        if (n2 != NULL) {
          const Type* t = n2->bottom_type();
          is_verified_oop_store = t->isa_oop_ptr() ? (t->is_ptr()->_offset==0) : false;
        }
      } else {
        // a load
        const Type* t = n->bottom_type();
        is_verified_oop_load = t->isa_oop_ptr() ? (t->is_ptr()->_offset==0) : false;
      }
    }

    if (ld_op) {
      // a Load
      // inputs are (0:control, 1:memory, 2:address)
      if (!(n->ideal_Opcode()==ld_op)       && // Following are special cases
          !(n->ideal_Opcode()==Op_LoadLLocked && ld_op==Op_LoadI) &&
          !(n->ideal_Opcode()==Op_LoadPLocked && ld_op==Op_LoadP) &&
          !(n->ideal_Opcode()==Op_LoadI     && ld_op==Op_LoadF) &&
          !(n->ideal_Opcode()==Op_LoadF     && ld_op==Op_LoadI) &&
          !(n->ideal_Opcode()==Op_LoadRange && ld_op==Op_LoadI) &&
          !(n->ideal_Opcode()==Op_LoadKlass && ld_op==Op_LoadP) &&
          !(n->ideal_Opcode()==Op_LoadL     && ld_op==Op_LoadI) &&
          !(n->ideal_Opcode()==Op_LoadL_unaligned && ld_op==Op_LoadI) &&
          !(n->ideal_Opcode()==Op_LoadD_unaligned && ld_op==Op_LoadF) &&
          !(n->ideal_Opcode()==Op_ConvI2F   && ld_op==Op_LoadF) &&
          !(n->ideal_Opcode()==Op_ConvI2D   && ld_op==Op_LoadF) &&
          !(n->ideal_Opcode()==Op_PrefetchRead  && ld_op==Op_LoadI) &&
          !(n->ideal_Opcode()==Op_PrefetchWrite && ld_op==Op_LoadI) &&
          !(n->rule() == loadUB_rule)) {
        verify_oops_warning(n, n->ideal_Opcode(), ld_op);
      }
    } else if (st_op) {
      // a Store
      // inputs are (0:control, 1:memory, 2:address, 3:value)
      if (!(n->ideal_Opcode()==st_op)    && // Following are special cases
          !(n->ideal_Opcode()==Op_StoreCM && st_op==Op_StoreB) &&
          !(n->ideal_Opcode()==Op_StoreI && st_op==Op_StoreF) &&
          !(n->ideal_Opcode()==Op_StoreF && st_op==Op_StoreI) &&
          !(n->ideal_Opcode()==Op_StoreL && st_op==Op_StoreI) &&
          !(n->ideal_Opcode()==Op_StoreD && st_op==Op_StoreI && n->rule() == storeD0_rule)) {
        verify_oops_warning(n, n->ideal_Opcode(), st_op);
      }
    }

    if (src2_enc == R_G0_enc && n->rule() != loadUB_rule && n->ideal_Opcode() != Op_StoreCM ) {
      Node* addr = n->in(2);
      if (!(addr->is_Mach() && addr->as_Mach()->ideal_Opcode() == Op_AddP)) {
        const TypeOopPtr* atype = addr->bottom_type()->isa_instptr();  // %%% oopptr?
        if (atype != NULL) {
          intptr_t offset = get_offset_from_base(n, atype, disp32);
          intptr_t offset_2 = get_offset_from_base_2(n, atype, disp32);
          if (offset != offset_2) {
            get_offset_from_base(n, atype, disp32);
            get_offset_from_base_2(n, atype, disp32);
          }
          assert(offset == offset_2, "different offsets");
          if (offset == disp32) {
            // we now know that src1 is a true oop pointer
            is_verified_oop_base = true;
            if (ld_op && src1_enc == dst_enc && ld_op != Op_LoadF && ld_op != Op_LoadD) {
              if( primary == Assembler::ldd_op3 ) {
                is_verified_oop_base = false; // Cannot 'ldd' into O7
              } else {
                tmp_enc = dst_enc;
                dst_enc = R_O7_enc; // Load into O7; preserve source oop
                assert(src1_enc != dst_enc, "");
              }
            }
          }
          if (st_op && (( offset == oopDesc::klass_offset_in_bytes())
                       || offset == oopDesc::mark_offset_in_bytes())) {
                      // loading the mark should not be allowed either, but
                      // we don't check this since it conflicts with InlineObjectHash
                      // usage of LoadINode to get the mark. We could keep the
                      // check if we create a new LoadMarkNode
            // but do not verify the object before its header is initialized
            ShouldNotReachHere();
          }
        }
      }
    }
  }
#endif

  uint instr;
  instr = (Assembler::ldst_op << 30)
        | (dst_enc        << 25)
        | (primary        << 19)
        | (src1_enc       << 14);

  uint index = src2_enc;
  int disp = disp32;

  if (src1_enc == R_SP_enc || src1_enc == R_FP_enc)
    disp += STACK_BIAS;

  // We should have a compiler bailout here rather than a guarantee.
  // Better yet would be some mechanism to handle variable-size matches correctly.
  guarantee(Assembler::is_simm13(disp), "Do not match large constant offsets" );

  if( disp == 0 ) {
    // use reg-reg form
    // bit 13 is already zero
    instr |= index;
  } else {
    // use reg-imm form
    instr |= 0x00002000;          // set bit 13 to one
    instr |= disp & 0x1FFF;
  }

  uint *code = (uint*)cbuf.code_end();
  *code = instr;
  cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);

#ifdef ASSERT
  {
    MacroAssembler _masm(&cbuf);
    if (is_verified_oop_base) {
      __ verify_oop(reg_to_register_object(src1_enc));
    }
    if (is_verified_oop_store) {
      __ verify_oop(reg_to_register_object(dst_enc));
    }
    if (tmp_enc != -1) {
      __ mov(O7, reg_to_register_object(tmp_enc));
    }
    if (is_verified_oop_load) {
      __ verify_oop(reg_to_register_object(dst_enc));
    }
  }
#endif
}

void emit_form3_mem_reg_asi(CodeBuffer &cbuf, const MachNode* n, int primary, int tertiary,
                        int src1_enc, int disp32, int src2_enc, int dst_enc, int asi) {

  uint instr;
  instr = (Assembler::ldst_op << 30)
        | (dst_enc        << 25)
        | (primary        << 19)
        | (src1_enc       << 14);

  int disp = disp32;
  int index    = src2_enc;

  if (src1_enc == R_SP_enc || src1_enc == R_FP_enc)
    disp += STACK_BIAS;

  // We should have a compiler bailout here rather than a guarantee.
  // Better yet would be some mechanism to handle variable-size matches correctly.
  guarantee(Assembler::is_simm13(disp), "Do not match large constant offsets" );

  if( disp != 0 ) {
    // use reg-reg form
    // set src2=R_O7 contains offset
    index = R_O7_enc;
    emit3_simm13( cbuf, Assembler::arith_op, index, Assembler::or_op3, 0, disp);
  }
  instr |= (asi << 5);
  instr |= index;
  uint *code = (uint*)cbuf.code_end();
  *code = instr;
  cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
}

void emit_call_reloc(CodeBuffer &cbuf, intptr_t entry_point, relocInfo::relocType rtype, bool preserve_g2 = false, bool force_far_call = false) {
  // The method which records debug information at every safepoint
  // expects the call to be the first instruction in the snippet as
  // it creates a PcDesc structure which tracks the offset of a call
  // from the start of the codeBlob. This offset is computed as
  // code_end() - code_begin() of the code which has been emitted
  // so far.
  // In this particular case we have skirted around the problem by
  // putting the "mov" instruction in the delay slot but the problem
  // may bite us again at some other point and a cleaner/generic
  // solution using relocations would be needed.
  MacroAssembler _masm(&cbuf);
  __ set_inst_mark();

  // We flush the current window just so that there is a valid stack copy
  // the fact that the current window becomes active again instantly is
  // not a problem there is nothing live in it.

#ifdef ASSERT
  int startpos = __ offset();
#endif /* ASSERT */

#ifdef _LP64
  // Calls to the runtime or native may not be reachable from compiled code,
  // so we generate the far call sequence on 64 bit sparc.
  // This code sequence is relocatable to any address, even on LP64.
  if ( force_far_call ) {
    __ relocate(rtype);
    Address dest(O7, (address)entry_point);
    __ jumpl_to(dest, O7);
  }
  else
#endif
  {
     __ call((address)entry_point, rtype);
  }

  if (preserve_g2)   __ delayed()->mov(G2, L7);
  else __ delayed()->nop();

  if (preserve_g2)   __ mov(L7, G2);

#ifdef ASSERT
  if (preserve_g2 && (VerifyCompiledCode || VerifyOops)) {
#ifdef _LP64
    // Trash argument dump slots.
    __ set(0xb0b8ac0db0b8ac0d, G1);
    __ mov(G1, G5);
    __ stx(G1, SP, STACK_BIAS + 0x80);
    __ stx(G1, SP, STACK_BIAS + 0x88);
    __ stx(G1, SP, STACK_BIAS + 0x90);
    __ stx(G1, SP, STACK_BIAS + 0x98);
    __ stx(G1, SP, STACK_BIAS + 0xA0);
    __ stx(G1, SP, STACK_BIAS + 0xA8);
#else // _LP64
    // this is also a native call, so smash the first 7 stack locations,
    // and the various registers

    // Note:  [SP+0x40] is sp[callee_aggregate_return_pointer_sp_offset],
    // while [SP+0x44..0x58] are the argument dump slots.
    __ set((intptr_t)0xbaadf00d, G1);
    __ mov(G1, G5);
    __ sllx(G1, 32, G1);
    __ or3(G1, G5, G1);
    __ mov(G1, G5);
    __ stx(G1, SP, 0x40);
    __ stx(G1, SP, 0x48);
    __ stx(G1, SP, 0x50);
    __ stw(G1, SP, 0x58); // Do not trash [SP+0x5C] which is a usable spill slot
#endif // _LP64
  }
#endif /*ASSERT*/
}

//=============================================================================
// REQUIRED FUNCTIONALITY for encoding
void emit_lo(CodeBuffer &cbuf, int val) {  }
void emit_hi(CodeBuffer &cbuf, int val) {  }

void emit_ptr(CodeBuffer &cbuf, intptr_t val, Register reg, bool ForceRelocatable) {
  MacroAssembler _masm(&cbuf);
  if (ForceRelocatable) {
    Address addr(reg, (address)val);
    __ sethi(addr, ForceRelocatable);
    __ add(addr, reg);
  } else {
    __ set(val, reg);
  }
}


//=============================================================================

#ifndef PRODUCT
void MachPrologNode::format( PhaseRegAlloc *ra_, outputStream *st ) const {
  Compile* C = ra_->C;

  for (int i = 0; i < OptoPrologueNops; i++) {
    st->print_cr("NOP"); st->print("\t");
  }

  if( VerifyThread ) {
    st->print_cr("Verify_Thread"); st->print("\t");
  }

  size_t framesize = C->frame_slots() << LogBytesPerInt;

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them.  But be careful, because
  // some VM calls (such as call site linkage) can use several kilobytes of
  // stack.  But the stack safety zone should account for that.
  // See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize)) {
    st->print_cr("! stack bang"); st->print("\t");
  }

  if (Assembler::is_simm13(-framesize)) {
    st->print   ("SAVE   R_SP,-%d,R_SP",framesize);
  } else {
    st->print_cr("SETHI  R_SP,hi%%(-%d),R_G3",framesize); st->print("\t");
    st->print_cr("ADD    R_G3,lo%%(-%d),R_G3",framesize); st->print("\t");
    st->print   ("SAVE   R_SP,R_G3,R_SP");
  }

}
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);

  for (int i = 0; i < OptoPrologueNops; i++) {
    __ nop();
  }

  __ verify_thread();

  size_t framesize = C->frame_slots() << LogBytesPerInt;
  assert(framesize >= 16*wordSize, "must have room for reg. save area");
  assert(framesize%(2*wordSize) == 0, "must preserve 2*wordSize alignment");

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them.  But be careful, because
  // some VM calls (such as call site linkage) can use several kilobytes of
  // stack.  But the stack safety zone should account for that.
  // See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize)) {
    __ generate_stack_overflow_check(framesize);
  }

  if (Assembler::is_simm13(-framesize)) {
    __ save(SP, -framesize, SP);
  } else {
    __ sethi(-framesize & ~0x3ff, G3);
    __ add(G3, -framesize & 0x3ff, G3);
    __ save(SP, G3, SP);
  }
  C->set_frame_complete( __ offset() );
}

uint MachPrologNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

int MachPrologNode::reloc() const {
  return 10; // a large enough number
}

//=============================================================================
#ifndef PRODUCT
void MachEpilogNode::format( PhaseRegAlloc *ra_, outputStream *st ) const {
  Compile* C = ra_->C;

  if( do_polling() && ra_->C->is_method_compilation() ) {
    st->print("SETHI  #PollAddr,L0\t! Load Polling address\n\t");
#ifdef _LP64
    st->print("LDX    [L0],G0\t!Poll for Safepointing\n\t");
#else
    st->print("LDUW   [L0],G0\t!Poll for Safepointing\n\t");
#endif
  }

  if( do_polling() )
    st->print("RET\n\t");

  st->print("RESTORE");
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler _masm(&cbuf);
  Compile* C = ra_->C;

  __ verify_thread();

  // If this does safepoint polling, then do it here
  if( do_polling() && ra_->C->is_method_compilation() ) {
    Address polling_page(L0, (address)os::get_polling_page());
    __ sethi(polling_page, false);
    __ relocate(relocInfo::poll_return_type);
    __ ld_ptr( L0, 0, G0 );
  }

  // If this is a return, then stuff the restore in the delay slot
  if( do_polling() ) {
    __ ret();
    __ delayed()->restore();
  } else {
    __ restore();
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

int MachEpilogNode::reloc() const {
  return 16; // a large enough number
}

const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

int MachEpilogNode::safepoint_offset() const {
  assert( do_polling(), "no return for this epilog node");
  return MacroAssembler::size_of_sethi(os::get_polling_page());
}

//=============================================================================

// Figure out which register class each belongs in: rc_int, rc_float, rc_stack
enum RC { rc_bad, rc_int, rc_float, rc_stack };
static enum RC rc_class( OptoReg::Name reg ) {
  if( !OptoReg::is_valid(reg)  ) return rc_bad;
  if (OptoReg::is_stack(reg)) return rc_stack;
  VMReg r = OptoReg::as_VMReg(reg);
  if (r->is_Register()) return rc_int;
  assert(r->is_FloatRegister(), "must be");
  return rc_float;
}

static int impl_helper( const MachNode *mach, CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, bool is_load, int offset, int reg, int opcode, const char *op_str, int size, outputStream* st ) {
  if( cbuf ) {
    // Better yet would be some mechanism to handle variable-size matches correctly
    if (!Assembler::is_simm13(offset + STACK_BIAS)) {
      ra_->C->record_method_not_compilable("unable to handle large constant offsets");
    } else {
      emit_form3_mem_reg(*cbuf, mach, opcode, -1, R_SP_enc, offset, 0, Matcher::_regEncode[reg]);
    }
  }
#ifndef PRODUCT
  else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    if( is_load ) st->print("%s   [R_SP + #%d],R_%s\t! spill",op_str,offset,OptoReg::regname(reg));
    else          st->print("%s   R_%s,[R_SP + #%d]\t! spill",op_str,OptoReg::regname(reg),offset);
  }
#endif
  return size+4;
}

static int impl_mov_helper( CodeBuffer *cbuf, bool do_size, int src, int dst, int op1, int op2, const char *op_str, int size, outputStream* st ) {
  if( cbuf ) emit3( *cbuf, Assembler::arith_op, Matcher::_regEncode[dst], op1, 0, op2, Matcher::_regEncode[src] );
#ifndef PRODUCT
  else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    st->print("%s  R_%s,R_%s\t! spill",op_str,OptoReg::regname(src),OptoReg::regname(dst));
  }
#endif
  return size+4;
}

uint MachSpillCopyNode::implementation( CodeBuffer *cbuf,
                                        PhaseRegAlloc *ra_,
                                        bool do_size,
                                        outputStream* st ) const {
  // Get registers to move
  OptoReg::Name src_second = ra_->get_reg_second(in(1));
  OptoReg::Name src_first = ra_->get_reg_first(in(1));
  OptoReg::Name dst_second = ra_->get_reg_second(this );
  OptoReg::Name dst_first = ra_->get_reg_first(this );

  enum RC src_second_rc = rc_class(src_second);
  enum RC src_first_rc = rc_class(src_first);
  enum RC dst_second_rc = rc_class(dst_second);
  enum RC dst_first_rc = rc_class(dst_first);

  assert( OptoReg::is_valid(src_first) && OptoReg::is_valid(dst_first), "must move at least 1 register" );

  // Generate spill code!
  int size = 0;

  if( src_first == dst_first && src_second == dst_second )
    return size;            // Self copy, no move

  // --------------------------------------
  // Check for mem-mem move.  Load into unused float registers and fall into
  // the float-store case.
  if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(src_first);
    // Further check for aligned-adjacent pair, so we can use a double load
    if( (src_first&1)==0 && src_first+1 == src_second ) {
      src_second    = OptoReg::Name(R_F31_num);
      src_second_rc = rc_float;
      size = impl_helper(this,cbuf,ra_,do_size,true,offset,R_F30_num,Assembler::lddf_op3,"LDDF",size, st);
    } else {
      size = impl_helper(this,cbuf,ra_,do_size,true,offset,R_F30_num,Assembler::ldf_op3 ,"LDF ",size, st);
    }
    src_first    = OptoReg::Name(R_F30_num);
    src_first_rc = rc_float;
  }

  if( src_second_rc == rc_stack && dst_second_rc == rc_stack ) {
    int offset = ra_->reg2offset(src_second);
    size = impl_helper(this,cbuf,ra_,do_size,true,offset,R_F31_num,Assembler::ldf_op3,"LDF ",size, st);
    src_second    = OptoReg::Name(R_F31_num);
    src_second_rc = rc_float;
  }

  // --------------------------------------
  // Check for float->int copy; requires a trip through memory
  if( src_first_rc == rc_float && dst_first_rc == rc_int ) {
    int offset = frame::register_save_words*wordSize;
    if( cbuf ) {
      emit3_simm13( *cbuf, Assembler::arith_op, R_SP_enc, Assembler::sub_op3, R_SP_enc, 16 );
      impl_helper(this,cbuf,ra_,do_size,false,offset,src_first,Assembler::stf_op3 ,"STF ",size, st);
      impl_helper(this,cbuf,ra_,do_size,true ,offset,dst_first,Assembler::lduw_op3,"LDUW",size, st);
      emit3_simm13( *cbuf, Assembler::arith_op, R_SP_enc, Assembler::add_op3, R_SP_enc, 16 );
    }
#ifndef PRODUCT
    else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print(  "SUB    R_SP,16,R_SP\n");
      impl_helper(this,cbuf,ra_,do_size,false,offset,src_first,Assembler::stf_op3 ,"STF ",size, st);
      impl_helper(this,cbuf,ra_,do_size,true ,offset,dst_first,Assembler::lduw_op3,"LDUW",size, st);
      st->print("\tADD    R_SP,16,R_SP\n");
    }
#endif
    size += 16;
  }

  // --------------------------------------
  // In the 32-bit 1-reg-longs build ONLY, I see mis-aligned long destinations.
  // In such cases, I have to do the big-endian swap.  For aligned targets, the
  // hardware does the flop for me.  Doubles are always aligned, so no problem
  // there.  Misaligned sources only come from native-long-returns (handled
  // special below).
#ifndef _LP64
  if( src_first_rc == rc_int &&     // source is already big-endian
      src_second_rc != rc_bad &&    // 64-bit move
      ((dst_first&1)!=0 || dst_second != dst_first+1) ) { // misaligned dst
    assert( (src_first&1)==0 && src_second == src_first+1, "source must be aligned" );
    // Do the big-endian flop.
    OptoReg::Name tmp    = dst_first   ; dst_first    = dst_second   ; dst_second    = tmp   ;
    enum RC       tmp_rc = dst_first_rc; dst_first_rc = dst_second_rc; dst_second_rc = tmp_rc;
  }
#endif

  // --------------------------------------
  // Check for integer reg-reg copy
  if( src_first_rc == rc_int && dst_first_rc == rc_int ) {
#ifndef _LP64
    if( src_first == R_O0_num && src_second == R_O1_num ) {  // Check for the evil O0/O1 native long-return case
      // Note: The _first and _second suffixes refer to the addresses of the the 2 halves of the 64-bit value
      //       as stored in memory.  On a big-endian machine like SPARC, this means that the _second
      //       operand contains the least significant word of the 64-bit value and vice versa.
      OptoReg::Name tmp = OptoReg::Name(R_O7_num);
      assert( (dst_first&1)==0 && dst_second == dst_first+1, "return a native O0/O1 long to an aligned-adjacent 64-bit reg" );
      // Shift O0 left in-place, zero-extend O1, then OR them into the dst
      if( cbuf ) {
        emit3_simm13( *cbuf, Assembler::arith_op, Matcher::_regEncode[tmp], Assembler::sllx_op3, Matcher::_regEncode[src_first], 0x1020 );
        emit3_simm13( *cbuf, Assembler::arith_op, Matcher::_regEncode[src_second], Assembler::srl_op3, Matcher::_regEncode[src_second], 0x0000 );
        emit3       ( *cbuf, Assembler::arith_op, Matcher::_regEncode[dst_first], Assembler:: or_op3, Matcher::_regEncode[tmp], 0, Matcher::_regEncode[src_second] );
#ifndef PRODUCT
      } else if( !do_size ) {
        if( size != 0 ) st->print("\n\t");
        st->print("SLLX   R_%s,32,R_%s\t! Move O0-first to O7-high\n\t", OptoReg::regname(src_first), OptoReg::regname(tmp));
        st->print("SRL    R_%s, 0,R_%s\t! Zero-extend O1\n\t", OptoReg::regname(src_second), OptoReg::regname(src_second));
        st->print("OR     R_%s,R_%s,R_%s\t! spill",OptoReg::regname(tmp), OptoReg::regname(src_second), OptoReg::regname(dst_first));
#endif
      }
      return size+12;
    }
    else if( dst_first == R_I0_num && dst_second == R_I1_num ) {
      // returning a long value in I0/I1
      // a SpillCopy must be able to target a return instruction's reg_class
      // Note: The _first and _second suffixes refer to the addresses of the the 2 halves of the 64-bit value
      //       as stored in memory.  On a big-endian machine like SPARC, this means that the _second
      //       operand contains the least significant word of the 64-bit value and vice versa.
      OptoReg::Name tdest = dst_first;

      if (src_first == dst_first) {
        tdest = OptoReg::Name(R_O7_num);
        size += 4;
      }

      if( cbuf ) {
        assert( (src_first&1) == 0 && (src_first+1) == src_second, "return value was in an aligned-adjacent 64-bit reg");
        // Shift value in upper 32-bits of src to lower 32-bits of I0; move lower 32-bits to I1
        // ShrL_reg_imm6
        emit3_simm13( *cbuf, Assembler::arith_op, Matcher::_regEncode[tdest], Assembler::srlx_op3, Matcher::_regEncode[src_second], 32 | 0x1000 );
        // ShrR_reg_imm6  src, 0, dst
        emit3_simm13( *cbuf, Assembler::arith_op, Matcher::_regEncode[dst_second], Assembler::srl_op3, Matcher::_regEncode[src_first], 0x0000 );
        if (tdest != dst_first) {
          emit3     ( *cbuf, Assembler::arith_op, Matcher::_regEncode[dst_first], Assembler::or_op3, 0/*G0*/, 0/*op2*/, Matcher::_regEncode[tdest] );
        }
      }
#ifndef PRODUCT
      else if( !do_size ) {
        if( size != 0 ) st->print("\n\t");  // %%%%% !!!!!
        st->print("SRLX   R_%s,32,R_%s\t! Extract MSW\n\t",OptoReg::regname(src_second),OptoReg::regname(tdest));
        st->print("SRL    R_%s, 0,R_%s\t! Extract LSW\n\t",OptoReg::regname(src_first),OptoReg::regname(dst_second));
        if (tdest != dst_first) {
          st->print("MOV    R_%s,R_%s\t! spill\n\t", OptoReg::regname(tdest), OptoReg::regname(dst_first));
        }
      }
#endif // PRODUCT
      return size+8;
    }
#endif // !_LP64
    // Else normal reg-reg copy
    assert( src_second != dst_first, "smashed second before evacuating it" );
    size = impl_mov_helper(cbuf,do_size,src_first,dst_first,Assembler::or_op3,0,"MOV  ",size, st);
    assert( (src_first&1) == 0 && (dst_first&1) == 0, "never move second-halves of int registers" );
    // This moves an aligned adjacent pair.
    // See if we are done.
    if( src_first+1 == src_second && dst_first+1 == dst_second )
      return size;
  }

  // Check for integer store
  if( src_first_rc == rc_int && dst_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(dst_first);
    // Further check for aligned-adjacent pair, so we can use a double store
    if( (src_first&1)==0 && src_first+1 == src_second && (dst_first&1)==0 && dst_first+1 == dst_second )
      return impl_helper(this,cbuf,ra_,do_size,false,offset,src_first,Assembler::stx_op3,"STX ",size, st);
    size  =  impl_helper(this,cbuf,ra_,do_size,false,offset,src_first,Assembler::stw_op3,"STW ",size, st);
  }

  // Check for integer load
  if( dst_first_rc == rc_int && src_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(src_first);
    // Further check for aligned-adjacent pair, so we can use a double load
    if( (src_first&1)==0 && src_first+1 == src_second && (dst_first&1)==0 && dst_first+1 == dst_second )
      return impl_helper(this,cbuf,ra_,do_size,true,offset,dst_first,Assembler::ldx_op3 ,"LDX ",size, st);
    size  =  impl_helper(this,cbuf,ra_,do_size,true,offset,dst_first,Assembler::lduw_op3,"LDUW",size, st);
  }

  // Check for float reg-reg copy
  if( src_first_rc == rc_float && dst_first_rc == rc_float ) {
    // Further check for aligned-adjacent pair, so we can use a double move
    if( (src_first&1)==0 && src_first+1 == src_second && (dst_first&1)==0 && dst_first+1 == dst_second )
      return impl_mov_helper(cbuf,do_size,src_first,dst_first,Assembler::fpop1_op3,Assembler::fmovd_opf,"FMOVD",size, st);
    size  =  impl_mov_helper(cbuf,do_size,src_first,dst_first,Assembler::fpop1_op3,Assembler::fmovs_opf,"FMOVS",size, st);
  }

  // Check for float store
  if( src_first_rc == rc_float && dst_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(dst_first);
    // Further check for aligned-adjacent pair, so we can use a double store
    if( (src_first&1)==0 && src_first+1 == src_second && (dst_first&1)==0 && dst_first+1 == dst_second )
      return impl_helper(this,cbuf,ra_,do_size,false,offset,src_first,Assembler::stdf_op3,"STDF",size, st);
    size  =  impl_helper(this,cbuf,ra_,do_size,false,offset,src_first,Assembler::stf_op3 ,"STF ",size, st);
  }

  // Check for float load
  if( dst_first_rc == rc_float && src_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(src_first);
    // Further check for aligned-adjacent pair, so we can use a double load
    if( (src_first&1)==0 && src_first+1 == src_second && (dst_first&1)==0 && dst_first+1 == dst_second )
      return impl_helper(this,cbuf,ra_,do_size,true,offset,dst_first,Assembler::lddf_op3,"LDDF",size, st);
    size  =  impl_helper(this,cbuf,ra_,do_size,true,offset,dst_first,Assembler::ldf_op3 ,"LDF ",size, st);
  }

  // --------------------------------------------------------------------
  // Check for hi bits still needing moving.  Only happens for misaligned
  // arguments to native calls.
  if( src_second == dst_second )
    return size;               // Self copy; no move
  assert( src_second_rc != rc_bad && dst_second_rc != rc_bad, "src_second & dst_second cannot be Bad" );

#ifndef _LP64
  // In the LP64 build, all registers can be moved as aligned/adjacent
  // pairs, so there's never any need to move the high bits seperately.
  // The 32-bit builds have to deal with the 32-bit ABI which can force
  // all sorts of silly alignment problems.

  // Check for integer reg-reg copy.  Hi bits are stuck up in the top
  // 32-bits of a 64-bit register, but are needed in low bits of another
  // register (else it's a hi-bits-to-hi-bits copy which should have
  // happened already as part of a 64-bit move)
  if( src_second_rc == rc_int && dst_second_rc == rc_int ) {
    assert( (src_second&1)==1, "its the evil O0/O1 native return case" );
    assert( (dst_second&1)==0, "should have moved with 1 64-bit move" );
    // Shift src_second down to dst_second's low bits.
    if( cbuf ) {
      emit3_simm13( *cbuf, Assembler::arith_op, Matcher::_regEncode[dst_second], Assembler::srlx_op3, Matcher::_regEncode[src_second-1], 0x1020 );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("SRLX   R_%s,32,R_%s\t! spill: Move high bits down low",OptoReg::regname(src_second-1),OptoReg::regname(dst_second));
#endif
    }
    return size+4;
  }

  // Check for high word integer store.  Must down-shift the hi bits
  // into a temp register, then fall into the case of storing int bits.
  if( src_second_rc == rc_int && dst_second_rc == rc_stack && (src_second&1)==1 ) {
    // Shift src_second down to dst_second's low bits.
    if( cbuf ) {
      emit3_simm13( *cbuf, Assembler::arith_op, Matcher::_regEncode[R_O7_num], Assembler::srlx_op3, Matcher::_regEncode[src_second-1], 0x1020 );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("SRLX   R_%s,32,R_%s\t! spill: Move high bits down low",OptoReg::regname(src_second-1),OptoReg::regname(R_O7_num));
#endif
    }
    size+=4;
    src_second = OptoReg::Name(R_O7_num); // Not R_O7H_num!
  }

  // Check for high word integer load
  if( dst_second_rc == rc_int && src_second_rc == rc_stack )
    return impl_helper(this,cbuf,ra_,do_size,true ,ra_->reg2offset(src_second),dst_second,Assembler::lduw_op3,"LDUW",size, st);

  // Check for high word integer store
  if( src_second_rc == rc_int && dst_second_rc == rc_stack )
    return impl_helper(this,cbuf,ra_,do_size,false,ra_->reg2offset(dst_second),src_second,Assembler::stw_op3 ,"STW ",size, st);

  // Check for high word float store
  if( src_second_rc == rc_float && dst_second_rc == rc_stack )
    return impl_helper(this,cbuf,ra_,do_size,false,ra_->reg2offset(dst_second),src_second,Assembler::stf_op3 ,"STF ",size, st);

#endif // !_LP64

  Unimplemented();
}

#ifndef PRODUCT
void MachSpillCopyNode::format( PhaseRegAlloc *ra_, outputStream *st ) const {
  implementation( NULL, ra_, false, st );
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation( &cbuf, ra_, false, NULL );
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return implementation( NULL, ra_, true, NULL );
}

//=============================================================================
#ifndef PRODUCT
void MachNopNode::format( PhaseRegAlloc *, outputStream *st ) const {
  st->print("NOP \t# %d bytes pad for loops and calls", 4 * _count);
}
#endif

void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc * ) const {
  MacroAssembler _masm(&cbuf);
  for(int i = 0; i < _count; i += 1) {
    __ nop();
  }
}

uint MachNopNode::size(PhaseRegAlloc *ra_) const {
  return 4 * _count;
}


//=============================================================================
#ifndef PRODUCT
void BoxLockNode::format( PhaseRegAlloc *ra_, outputStream *st ) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("LEA    [R_SP+#%d+BIAS],%s",offset,Matcher::regName[reg]);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler _masm(&cbuf);
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem()) + STACK_BIAS;
  int reg = ra_->get_encode(this);

  if (Assembler::is_simm13(offset)) {
     __ add(SP, offset, reg_to_register_object(reg));
  } else {
     __ set(offset, O7);
     __ add(SP, O7, reg_to_register_object(reg));
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  // BoxLockNode is not a MachNode, so we can't just call MachNode::size(ra_)
  assert(ra_ == ra_->C->regalloc(), "sanity");
  return ra_->C->scratch_emit_size(this);
}

//=============================================================================

// emit call stub, compiled java to interpretor
void emit_java_to_interp(CodeBuffer &cbuf ) {

  // Stub is fixed up when the corresponding call is converted from calling
  // compiled code to calling interpreted code.
  // set (empty), G5
  // jmp -1

  address mark = cbuf.inst_mark();  // get mark within main instrs section

  MacroAssembler _masm(&cbuf);

  address base =
  __ start_a_stub(Compile::MAX_stubs_size);
  if (base == NULL)  return;  // CodeBuffer::expand failed

  // static stub relocation stores the instruction address of the call
  __ relocate(static_stub_Relocation::spec(mark));

  __ set_oop(NULL, reg_to_register_object(Matcher::inline_cache_reg_encode()));

  __ set_inst_mark();
  Address a(G3, (address)-1);
  __ JUMP(a, 0);

  __ delayed()->nop();

  // Update current stubs pointer and restore code_end.
  __ end_a_stub();
}

// size of call stub, compiled java to interpretor
uint size_java_to_interp() {
  // This doesn't need to be accurate but it must be larger or equal to
  // the real size of the stub.
  return (NativeMovConstReg::instruction_size +  // sethi/setlo;
          NativeJump::instruction_size + // sethi; jmp; nop
          (TraceJumps ? 20 * BytesPerInstWord : 0) );
}
// relocation entries for call stub, compiled java to interpretor
uint reloc_java_to_interp() {
  return 10;  // 4 in emit_java_to_interp + 1 in Java_Static_Call
}


//=============================================================================
#ifndef PRODUCT
void MachUEPNode::format( PhaseRegAlloc *ra_, outputStream *st ) const {
  st->print_cr("\nUEP:");
#ifdef    _LP64
  if (UseCompressedOops) {
    st->print_cr("\tLDUW   [R_O0 + oopDesc::klass_offset_in_bytes],R_G5\t! Inline cache check - compressed klass");
    st->print_cr("\tSLL    R_G5,3,R_G5");
    st->print_cr("\tADD    R_G5,R_G6_heap_base,R_G5");
  } else {
    st->print_cr("\tLDX    [R_O0 + oopDesc::klass_offset_in_bytes],R_G5\t! Inline cache check");
  }
  st->print_cr("\tCMP    R_G5,R_G3" );
  st->print   ("\tTne    xcc,R_G0+ST_RESERVED_FOR_USER_0+2");
#else  // _LP64
  st->print_cr("\tLDUW   [R_O0 + oopDesc::klass_offset_in_bytes],R_G5\t! Inline cache check");
  st->print_cr("\tCMP    R_G5,R_G3" );
  st->print   ("\tTne    icc,R_G0+ST_RESERVED_FOR_USER_0+2");
#endif // _LP64
}
#endif

void MachUEPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler _masm(&cbuf);
  Label L;
  Register G5_ic_reg  = reg_to_register_object(Matcher::inline_cache_reg_encode());
  Register temp_reg   = G3;
  assert( G5_ic_reg != temp_reg, "conflicting registers" );

  // Load klass from reciever
  __ load_klass(O0, temp_reg);
  // Compare against expected klass
  __ cmp(temp_reg, G5_ic_reg);
  // Branch to miss code, checks xcc or icc depending
  __ trap(Assembler::notEqual, Assembler::ptr_cc, G0, ST_RESERVED_FOR_USER_0+2);
}

uint MachUEPNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}


//=============================================================================

uint size_exception_handler() {
  if (TraceJumps) {
    return (400); // just a guess
  }
  return ( NativeJump::instruction_size ); // sethi;jmp;nop
}

uint size_deopt_handler() {
  if (TraceJumps) {
    return (400); // just a guess
  }
  return ( 4+  NativeJump::instruction_size ); // save;sethi;jmp;restore
}

// Emit exception handler code.
int emit_exception_handler(CodeBuffer& cbuf) {
  Register temp_reg = G3;
  Address exception_blob(temp_reg, OptoRuntime::exception_blob()->instructions_begin());
  MacroAssembler _masm(&cbuf);

  address base =
  __ start_a_stub(size_exception_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed

  int offset = __ offset();

  __ JUMP(exception_blob, 0); // sethi;jmp
  __ delayed()->nop();

  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");

  __ end_a_stub();

  return offset;
}

int emit_deopt_handler(CodeBuffer& cbuf) {
  // Can't use any of the current frame's registers as we may have deopted
  // at a poll and everything (including G3) can be live.
  Register temp_reg = L0;
  Address deopt_blob(temp_reg, SharedRuntime::deopt_blob()->unpack());
  MacroAssembler _masm(&cbuf);

  address base =
  __ start_a_stub(size_deopt_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed

  int offset = __ offset();
  __ save_frame(0);
  __ JUMP(deopt_blob, 0); // sethi;jmp
  __ delayed()->restore();

  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");

  __ end_a_stub();
  return offset;

}

// Given a register encoding, produce a Integer Register object
static Register reg_to_register_object(int register_encoding) {
  assert(L5->encoding() == R_L5_enc && G1->encoding() == R_G1_enc, "right coding");
  return as_Register(register_encoding);
}

// Given a register encoding, produce a single-precision Float Register object
static FloatRegister reg_to_SingleFloatRegister_object(int register_encoding) {
  assert(F5->encoding(FloatRegisterImpl::S) == R_F5_enc && F12->encoding(FloatRegisterImpl::S) == R_F12_enc, "right coding");
  return as_SingleFloatRegister(register_encoding);
}

// Given a register encoding, produce a double-precision Float Register object
static FloatRegister reg_to_DoubleFloatRegister_object(int register_encoding) {
  assert(F4->encoding(FloatRegisterImpl::D) == R_F4_enc, "right coding");
  assert(F32->encoding(FloatRegisterImpl::D) == R_D32_enc, "right coding");
  return as_DoubleFloatRegister(register_encoding);
}

int Matcher::regnum_to_fpu_offset(int regnum) {
  return regnum - 32; // The FP registers are in the second chunk
}

#ifdef ASSERT
address last_rethrow = NULL;  // debugging aid for Rethrow encoding
#endif

// Vector width in bytes
const uint Matcher::vector_width_in_bytes(void) {
  return 8;
}

// Vector ideal reg
const uint Matcher::vector_ideal_reg(void) {
  return Op_RegD;
}

// USII supports fxtof through the whole range of number, USIII doesn't
const bool Matcher::convL2FSupported(void) {
  return VM_Version::has_fast_fxtof();
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int offset) {
  return false;
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  // Depends on optimizations in MacroAssembler::setx.
  int hi = (int)(value >> 32);
  int lo = (int)(value & ~0);
  return (hi == 0) || (hi == -1) || (lo == 0);
}

// No scaling for the parameter the ClearArray node.
const bool Matcher::init_array_count_is_in_bytes = true;

// Threshold size for cleararray.
const int Matcher::init_array_short_size = 8 * BytesPerLong;

// Should the Matcher clone shifts on addressing modes, expecting them to
// be subsumed into complex addressing expressions or compute them into
// registers?  True for Intel but false for most RISCs
const bool Matcher::clone_shift_expressions = false;

// Is it better to copy float constants, or load them directly from memory?
// Intel can load a float constant from a direct address, requiring no
// extra registers.  Most RISCs will have to materialize an address into a
// register first, so they would do better to copy the constant from stack.
const bool Matcher::rematerialize_float_constants = false;

// If CPU can load and store mis-aligned doubles directly then no fixup is
// needed.  Else we split the double into 2 integer pieces and move it
// piece-by-piece.  Only happens when passing doubles into C code as the
// Java calling convention forces doubles to be aligned.
#ifdef _LP64
const bool Matcher::misaligned_doubles_ok = true;
#else
const bool Matcher::misaligned_doubles_ok = false;
#endif

// No-op on SPARC.
void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {
}

// Advertise here if the CPU requires explicit rounding operations
// to implement the UseStrictFP mode.
const bool Matcher::strict_fp_requires_explicit_rounding = false;

// Do floats take an entire double register or just half?
const bool Matcher::float_in_double = false;

// Do ints take an entire long register or just half?
// Note that we if-def off of _LP64.
// The relevant question is how the int is callee-saved.  In _LP64
// the whole long is written but de-opt'ing will have to extract
// the relevant 32 bits, in not-_LP64 only the low 32 bits is written.
#ifdef _LP64
const bool Matcher::int_in_long = true;
#else
const bool Matcher::int_in_long = false;
#endif

// Return whether or not this register is ever used as an argument.  This
// function is used on startup to build the trampoline stubs in generateOptoStub.
// Registers not mentioned will be killed by the VM call in the trampoline, and
// arguments in those registers not be available to the callee.
bool Matcher::can_be_java_arg( int reg ) {
  // Standard sparc 6 args in registers
  if( reg == R_I0_num ||
      reg == R_I1_num ||
      reg == R_I2_num ||
      reg == R_I3_num ||
      reg == R_I4_num ||
      reg == R_I5_num ) return true;
#ifdef _LP64
  // 64-bit builds can pass 64-bit pointers and longs in
  // the high I registers
  if( reg == R_I0H_num ||
      reg == R_I1H_num ||
      reg == R_I2H_num ||
      reg == R_I3H_num ||
      reg == R_I4H_num ||
      reg == R_I5H_num ) return true;

  if ((UseCompressedOops) && (reg == R_G6_num || reg == R_G6H_num)) {
    return true;
  }

#else
  // 32-bit builds with longs-in-one-entry pass longs in G1 & G4.
  // Longs cannot be passed in O regs, because O regs become I regs
  // after a 'save' and I regs get their high bits chopped off on
  // interrupt.
  if( reg == R_G1H_num || reg == R_G1_num ) return true;
  if( reg == R_G4H_num || reg == R_G4_num ) return true;
#endif
  // A few float args in registers
  if( reg >= R_F0_num && reg <= R_F7_num ) return true;

  return false;
}

bool Matcher::is_spillable_arg( int reg ) {
  return can_be_java_arg(reg);
}

// Register for DIVI projection of divmodI
RegMask Matcher::divI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODI projection of divmodI
RegMask Matcher::modI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for DIVL projection of divmodL
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}



#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
  assert( DEFAULT_COST == 100, "Expect (100) to equal 100");
  assert( HUGE_COST == 1000000, "Expect (1000000) to equal 1000000");
  assert( MEMORY_REF_COST == 200, "Expect (DEFAULT_COST * 2) to equal 200");
  assert( BRANCH_COST == 300, "Expect (DEFAULT_COST * 3) to equal 300");
  assert( CALL_COST == 300, "Expect (DEFAULT_COST * 3) to equal 300");
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'C',
  'C',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'N',
  'N',
  'N',
  'N',
  'C',
  'C',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C' // no trailing comma
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'C',
  'C',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'N',
  'N',
  'N',
  'N',
  'C',
  'C',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C' // no trailing comma
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegD,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags // no trailing comma
};


const        int   reduceOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  immI_rule, // 8
  immI13_rule, // 9
  immU13_rule, // 10
  immU6_rule, // 11
  immI11_rule, // 12
  immI0_rule, // 13
  immI10_rule, // 14
  immU5_rule, // 15
  immI_1_31_rule, // 16
  immI_32_63_rule, // 17
  immI_255_rule, // 18
  immL_FF_rule, // 19
  immL_FFFF_rule, // 20
  immP_rule, // 21
  immP13_rule, // 22
  immP0_rule, // 23
  immP_poll_rule, // 24
  immN_rule, // 25
  immN0_rule, // 26
  immL_rule, // 27
  immL0_rule, // 28
  immL13_rule, // 29
  immL_32bits_rule, // 30
  immD_rule, // 31
  immD0_rule, // 32
  immF_rule, // 33
  immF0_rule, // 34
  iRegI_rule, // 35
  notemp_iRegI_rule, // 36
  o0RegI_rule, // 37
  iRegP_rule, // 38
  sp_ptr_RegP_rule, // 39
  lock_ptr_RegP_rule, // 40
  g1RegP_rule, // 41
  g2RegP_rule, // 42
  g3RegP_rule, // 43
  g1RegI_rule, // 44
  g3RegI_rule, // 45
  g4RegI_rule, // 46
  g4RegP_rule, // 47
  i0RegP_rule, // 48
  o0RegP_rule, // 49
  o1RegP_rule, // 50
  o2RegP_rule, // 51
  o7RegP_rule, // 52
  l7RegP_rule, // 53
  o7RegI_rule, // 54
  iRegN_rule, // 55
  iRegL_rule, // 56
  o2RegL_rule, // 57
  o7RegL_rule, // 58
  g1RegL_rule, // 59
  g3RegL_rule, // 60
  iRegIsafe_rule, // 61
  flagsReg_rule, // 62
  flagsRegU_rule, // 63
  flagsRegP_rule, // 64
  flagsRegL_rule, // 65
  flagsRegF_rule, // 66
  flagsRegF0_rule, // 67
  flagsReg_long_LTGE_rule, // 68
  flagsReg_long_EQNE_rule, // 69
  flagsReg_long_LEGT_rule, // 70
  regD_rule, // 71
  regF_rule, // 72
  regD_low_rule, // 73
  inline_cache_regP_rule, // 74
  interpreter_method_oop_regP_rule, // 75
  indirect_rule, // 76
  indOffset13_rule, // 77
  indIndex_rule, // 78
  stackSlotI_rule, // 79
  stackSlotP_rule, // 80
  stackSlotF_rule, // 81
  stackSlotD_rule, // 82
  stackSlotL_rule, // 83
  cmpOp_rule, // 84
  cmpOpU_rule, // 85
  cmpOpP_rule, // 86
  cmpOp_reg_rule, // 87
  cmpOpF_rule, // 88
  cmpOp_commute_rule, // 89
  // last operand
  memory_rule, // 90
  // last operand class
  _LoadB_memory__rule, // 91
  _ConvI2L__LoadB_memory___rule, // 92
  _LoadC_memory__rule, // 93
  _ConvI2L__LoadC_memory___rule, // 94
  _Replicate4C_immI0__rule, // 95
  _Binary_cmpOpP_flagsRegP_rule, // 96
  _Binary_iRegI_iRegI_rule, // 97
  _Binary_iRegI_immI11_rule, // 98
  _Binary_cmpOp_flagsReg_rule, // 99
  _Binary_cmpOpU_flagsRegU_rule, // 100
  _Binary_cmpOpF_flagsRegF_rule, // 101
  _Binary_iRegN_iRegN_rule, // 102
  _Binary_iRegP_iRegP_rule, // 103
  _Binary_iRegP_immP0_rule, // 104
  _Binary_regF_regF_rule, // 105
  _Binary_regD_regD_rule, // 106
  _Binary_iRegL_iRegL_rule, // 107
  _Binary_iRegL_immI11_rule, // 108
  _Binary_iRegP_g3RegP_rule, // 109
  _Binary_iRegI_g3RegI_rule, // 110
  _Binary_iRegL_g3RegL_rule, // 111
  _CastP2X_iRegP__rule, // 112
  _ConvF2D_regF__rule, // 113
  _SqrtD__ConvF2D_regF___rule, // 114
  _CastP2X_sp_ptr_RegP__rule, // 115
  _CmpLTMask_iRegI_iRegI_rule, // 116
  _AndI__CmpLTMask_iRegI_iRegI_iRegI_rule, // 117
  _SubI_iRegI_iRegI_rule, // 118
  _AndI_iRegI__CmpLTMask_iRegI_iRegI_rule, // 119
  _LoadI_memory__rule, // 120
  _ConvI2L_iRegI__rule, // 121
  _RShiftL_iRegL_immI_32_63_rule, // 122
  _AndI_iRegI_iRegI_rule, // 123
  _AndI_iRegI_immI13_rule, // 124
  _AndL_iRegL_iRegL_rule, // 125
  _AndL_iRegL_immL13_rule, // 126
  _CmpI_iRegI_immI0_rule, // 127
  _CmpP_iRegP_immP0_rule, // 128
  _CmpL_iRegL_immL0_rule, // 129
  _Binary_cmpOp_flagsRegL_rule, // 130
  _Binary_iRegL_immL0_rule, // 131
  _PartialSubtypeCheck_o1RegP_o2RegP_rule, // 132
  _LoadL_memory__rule, // 133
  _ReverseBytesI_iRegI__rule, // 134
  _ReverseBytesL_iRegL__rule, // 135
  // last internally defined operand
  stackSlotI_rule, // 136
  stackSlotL_rule, // 137
  stackSlotP_rule, // 138
  stackSlotD_rule, // 139
  stackSlotF_rule, // 140
  iRegI_rule, // 141
  iRegL_rule, // 142
  iRegP_rule, // 143
  iRegI_rule, // 144
  iRegI_rule, // 145
  iRegP_rule, // 146
  iRegP_rule, // 147
  iRegP_rule, // 148
  iRegN_rule, // 149
  iRegN_rule, // 150
  iRegL_rule, // 151
  iRegL_rule, // 152
  iRegL_rule, // 153
  regF_rule, // 154
  regD_rule, // 155
  regD_rule, // 156
  0, // 157
  0, // 158
  regD_rule, // 159
  regF_rule, // 160
  iRegP_rule, // 161
  iRegL_rule, // 162
  g2RegP_rule, // 163
  iRegP_rule, // 164
  iRegP_rule, // 165
  iRegI_rule, // 166
  flagsRegP_rule, // 167
  flagsReg_rule, // 168
  flagsRegL_rule, // 169
  0, // 170
  0, // 171
  regD_rule, // 172
  regF_rule, // 173
  regF_rule, // 174
  regD_rule, // 175
  iRegI_rule, // 176
  0, // 177
  0, // 178
  iRegI_rule, // 179
  regF_rule, // 180
  iRegL_rule, // 181
  regD_rule, // 182
  0, // 183
  0, // 184
  0, // 185
  0, // 186
  0, // 187
  0, // 188
  0, // 189
  flagsReg_rule, // 190
  flagsRegU_rule, // 191
  flagsReg_rule, // 192
  flagsReg_rule, // 193
  flagsReg_rule, // 194
  flagsRegL_rule, // 195
  flagsRegL_rule, // 196
  flagsRegL_rule, // 197
  flagsRegL_rule, // 198
  flagsRegU_rule, // 199
  flagsRegP_rule, // 200
  flagsRegP_rule, // 201
  flagsReg_rule, // 202
  flagsReg_rule, // 203
  flagsRegF_rule, // 204
  flagsRegF_rule, // 205
  flagsRegP_rule, // 206
  flagsRegP_rule, // 207
  flagsRegP_rule, // 208
  0, // 209
  0, // 210
  0, // 211
  0, // 212
  0, // 213
  0, // 214
  0, // 215
  0, // 216
  0, // 217
  iRegI_rule, // 218
  iRegI_rule, // 219
  iRegL_rule, // 220
  iRegL_rule, // 221
  iRegI_rule, // 222
  iRegI_rule, // 223
  iRegL_rule, // 224
  iRegL_rule, // 225
  regD_rule, // 226
  regD_rule, // 227
  regD_rule, // 228
  regD_rule, // 229
  iRegI_rule, // 230
  regF_rule, // 231
  iRegP_rule, // 232
  iRegN_rule, // 233
  iRegP_rule, // 234
  iRegN_rule, // 235
  iRegI_rule, // 236
  regD_rule, // 237
  regD_low_rule, // 238
  regF_rule, // 239
  Universe_rule, // 240
  Universe_rule, // 241
  Universe_rule, // 242
  Universe_rule, // 243
  Universe_rule, // 244
  Universe_rule, // 245
  Universe_rule, // 246
  Universe_rule, // 247
  Universe_rule, // 248
  Universe_rule, // 249
  Universe_rule, // 250
  Universe_rule, // 251
  Universe_rule, // 252
  Universe_rule, // 253
  Universe_rule, // 254
  Universe_rule, // 255
  Universe_rule, // 256
  Universe_rule, // 257
  Universe_rule, // 258
  Universe_rule, // 259
  Universe_rule, // 260
  iRegN_rule, // 261
  iRegN_rule, // 262
  iRegP_rule, // 263
  iRegP_rule, // 264
  Universe_rule, // 265
  Universe_rule, // 266
  Universe_rule, // 267
  Universe_rule, // 268
  Universe_rule, // 269
  Universe_rule, // 270
  Universe_rule, // 271
  Universe_rule, // 272
  Universe_rule, // 273
  Universe_rule, // 274
  Universe_rule, // 275
  iRegI_rule, // 276
  iRegI_rule, // 277
  iRegI_rule, // 278
  iRegI_rule, // 279
  iRegI_rule, // 280
  iRegI_rule, // 281
  iRegI_rule, // 282
  iRegI_rule, // 283
  iRegN_rule, // 284
  iRegN_rule, // 285
  iRegN_rule, // 286
  iRegP_rule, // 287
  iRegP_rule, // 288
  iRegP_rule, // 289
  iRegP_rule, // 290
  iRegP_rule, // 291
  iRegP_rule, // 292
  regF_rule, // 293
  regF_rule, // 294
  regF_rule, // 295
  regD_rule, // 296
  regD_rule, // 297
  regD_rule, // 298
  iRegL_rule, // 299
  iRegL_rule, // 300
  iRegL_rule, // 301
  iRegL_rule, // 302
  iRegI_rule, // 303
  iRegI_rule, // 304
  iRegP_rule, // 305
  iRegP_rule, // 306
  iRegL_rule, // 307
  iRegL_rule, // 308
  iRegP_rule, // 309
  iRegL_rule, // 310
  iRegI_rule, // 311
  iRegI_rule, // 312
  iRegI_rule, // 313
  iRegI_rule, // 314
  iRegI_rule, // 315
  iRegI_rule, // 316
  iRegI_rule, // 317
  iRegL_rule, // 318
  iRegL_rule, // 319
  iRegL_rule, // 320
  iRegI_rule, // 321
  iRegI_rule, // 322
  iRegL_rule, // 323
  iRegL_rule, // 324
  iRegI_rule, // 325
  iRegI_rule, // 326
  0, // 327
  0, // 328
  iRegI_rule, // 329
  iRegL_rule, // 330
  iRegL_rule, // 331
  iRegI_rule, // 332
  iRegI_rule, // 333
  0, // 334
  0, // 335
  0, // 336
  0, // 337
  0, // 338
  0, // 339
  iRegL_rule, // 340
  iRegL_rule, // 341
  iRegI_rule, // 342
  iRegI_rule, // 343
  iRegL_rule, // 344
  iRegL_rule, // 345
  iRegI_rule, // 346
  iRegI_rule, // 347
  iRegL_rule, // 348
  iRegL_rule, // 349
  iRegI_rule, // 350
  iRegI_rule, // 351
  iRegL_rule, // 352
  iRegL_rule, // 353
  iRegL_rule, // 354
  regF_rule, // 355
  regD_rule, // 356
  regF_rule, // 357
  regD_rule, // 358
  regF_rule, // 359
  regD_rule, // 360
  regF_rule, // 361
  regD_rule, // 362
  regF_rule, // 363
  regD_rule, // 364
  iRegI_rule, // 365
  iRegI_rule, // 366
  iRegL_rule, // 367
  iRegL_rule, // 368
  iRegI_rule, // 369
  iRegI_rule, // 370
  iRegL_rule, // 371
  iRegL_rule, // 372
  iRegL_rule, // 373
  iRegL_rule, // 374
  iRegI_rule, // 375
  iRegI_rule, // 376
  iRegL_rule, // 377
  iRegL_rule, // 378
  iRegI_rule, // 379
  iRegI_rule, // 380
  iRegI_rule, // 381
  iRegI_rule, // 382
  iRegI_rule, // 383
  iRegI_rule, // 384
  iRegI_rule, // 385
  iRegI_rule, // 386
  iRegI_rule, // 387
  iRegI_rule, // 388
  regF_rule, // 389
  0, // 390
  stackSlotI_rule, // 391
  0, // 392
  stackSlotL_rule, // 393
  regD_rule, // 394
  0, // 395
  stackSlotI_rule, // 396
  0, // 397
  stackSlotL_rule, // 398
  regD_rule, // 399
  regD_low_rule, // 400
  regF_rule, // 401
  regF_rule, // 402
  iRegL_rule, // 403
  iRegL_rule, // 404
  iRegL_rule, // 405
  stackSlotI_rule, // 406
  stackSlotF_rule, // 407
  stackSlotL_rule, // 408
  stackSlotD_rule, // 409
  0, // 410
  0, // 411
  0, // 412
  0, // 413
  0, // 414
  0, // 415
  regD_rule, // 416
  regD_rule, // 417
  regF_rule, // 418
  iRegI_rule, // 419
  iRegI_rule, // 420
  stackSlotD_rule, // 421
  regD_rule, // 422
  stackSlotD_rule, // 423
  regD_rule, // 424
  stackSlotD_rule, // 425
  regD_rule, // 426
  stackSlotD_rule, // 427
  regD_rule, // 428
  0, // 429
  iRegI_rule, // 430
  0, // 431
  iRegI_rule, // 432
  iRegI_rule, // 433
  iRegI_rule, // 434
  Universe_rule, // 435
  Universe_rule, // 436
  Universe_rule, // 437
  Universe_rule, // 438
  Universe_rule, // 439
  Universe_rule, // 440
  Universe_rule, // 441
  Universe_rule, // 442
  Universe_rule, // 443
  Universe_rule, // 444
  Universe_rule, // 445
  Universe_rule, // 446
  iRegI_rule, // 447
  iRegL_rule, // 448
  iRegL_rule, // 449
  iRegI_rule, // 450
  iRegI_rule, // 451
  iRegN_rule, // 452
  iRegP_rule, // 453
  iRegP_rule, // 454
  regF_rule, // 455
  regD_rule, // 456
  Universe_rule, // 457
  Universe_rule, // 458
  Universe_rule, // 459
  Universe_rule, // 460
  Universe_rule, // 461
  Universe_rule, // 462
  Universe_rule, // 463
  Universe_rule, // 464
  Universe_rule, // 465
  o0RegP_rule, // 466
  Universe_rule, // 467
  Universe_rule, // 468
  o0RegP_rule, // 469
  Universe_rule, // 470
  notemp_iRegI_rule, // 471
  iRegI_rule, // 472
  iRegL_rule, // 473
  iRegI_rule, // 474
  iRegL_rule, // 475
  Universe_rule, // 476
  Universe_rule, // 477
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  0, // 8
  0, // 9
  0, // 10
  0, // 11
  0, // 12
  0, // 13
  0, // 14
  0, // 15
  0, // 16
  0, // 17
  0, // 18
  0, // 19
  0, // 20
  0, // 21
  0, // 22
  0, // 23
  0, // 24
  0, // 25
  0, // 26
  0, // 27
  0, // 28
  0, // 29
  0, // 30
  0, // 31
  0, // 32
  0, // 33
  0, // 34
  0, // 35
  0, // 36
  0, // 37
  0, // 38
  0, // 39
  0, // 40
  0, // 41
  0, // 42
  0, // 43
  0, // 44
  0, // 45
  0, // 46
  0, // 47
  0, // 48
  0, // 49
  0, // 50
  0, // 51
  0, // 52
  0, // 53
  0, // 54
  0, // 55
  0, // 56
  0, // 57
  0, // 58
  0, // 59
  0, // 60
  0, // 61
  0, // 62
  0, // 63
  0, // 64
  0, // 65
  0, // 66
  0, // 67
  0, // 68
  0, // 69
  0, // 70
  0, // 71
  0, // 72
  0, // 73
  0, // 74
  0, // 75
  0, // 76
  sp_ptr_RegP_rule, // 77
  iRegP_rule, // 78
  0, // 79
  0, // 80
  0, // 81
  0, // 82
  0, // 83
  0, // 84
  0, // 85
  0, // 86
  0, // 87
  0, // 88
  0, // 89
  // last operand
  0, // 90
  // last operand class
  memory_rule, // 91
  _LoadB_memory__rule, // 92
  memory_rule, // 93
  _LoadC_memory__rule, // 94
  immI0_rule, // 95
  cmpOpP_rule, // 96
  iRegI_rule, // 97
  iRegI_rule, // 98
  cmpOp_rule, // 99
  cmpOpU_rule, // 100
  cmpOpF_rule, // 101
  iRegN_rule, // 102
  iRegP_rule, // 103
  iRegP_rule, // 104
  regF_rule, // 105
  regD_rule, // 106
  iRegL_rule, // 107
  iRegL_rule, // 108
  iRegP_rule, // 109
  iRegI_rule, // 110
  iRegL_rule, // 111
  iRegP_rule, // 112
  regF_rule, // 113
  _ConvF2D_regF__rule, // 114
  sp_ptr_RegP_rule, // 115
  iRegI_rule, // 116
  _CmpLTMask_iRegI_iRegI_rule, // 117
  iRegI_rule, // 118
  iRegI_rule, // 119
  memory_rule, // 120
  iRegI_rule, // 121
  iRegL_rule, // 122
  iRegI_rule, // 123
  iRegI_rule, // 124
  iRegL_rule, // 125
  iRegL_rule, // 126
  iRegI_rule, // 127
  iRegP_rule, // 128
  iRegL_rule, // 129
  cmpOp_rule, // 130
  iRegL_rule, // 131
  o1RegP_rule, // 132
  memory_rule, // 133
  iRegI_rule, // 134
  iRegL_rule, // 135
  // last internally defined operand
  iRegI_rule, // 136
  iRegL_rule, // 137
  iRegP_rule, // 138
  regD_rule, // 139
  regF_rule, // 140
  stackSlotI_rule, // 141
  stackSlotL_rule, // 142
  stackSlotP_rule, // 143
  immI_rule, // 144
  immI13_rule, // 145
  immP_rule, // 146
  immP0_rule, // 147
  immP_poll_rule, // 148
  immN0_rule, // 149
  immN_rule, // 150
  immL_rule, // 151
  immL0_rule, // 152
  immL13_rule, // 153
  immF_rule, // 154
  immD_rule, // 155
  stackSlotD_rule, // 156
  0, // 157
  0, // 158
  regD_rule, // 159
  regF_rule, // 160
  iRegL_rule, // 161
  iRegP_rule, // 162
  0, // 163
  iRegP_rule, // 164
  iRegP_rule, // 165
  iRegI_rule, // 166
  iRegP_rule, // 167
  iRegP_rule, // 168
  iRegP_rule, // 169
  0, // 170
  0, // 171
  regD_rule, // 172
  regF_rule, // 173
  regF_rule, // 174
  regD_rule, // 175
  iRegP_rule, // 176
  0, // 177
  0, // 178
  stackSlotF_rule, // 179
  stackSlotI_rule, // 180
  stackSlotD_rule, // 181
  stackSlotL_rule, // 182
  0, // 183
  0, // 184
  0, // 185
  0, // 186
  0, // 187
  0, // 188
  0, // 189
  iRegI_rule, // 190
  iRegI_rule, // 191
  iRegI_rule, // 192
  _AndI_iRegI_iRegI_rule, // 193
  _AndI_iRegI_immI13_rule, // 194
  iRegL_rule, // 195
  iRegL_rule, // 196
  _AndL_iRegL_iRegL_rule, // 197
  _AndL_iRegL_immL13_rule, // 198
  iRegI_rule, // 199
  iRegP_rule, // 200
  iRegP_rule, // 201
  iRegN_rule, // 202
  iRegN_rule, // 203
  regF_rule, // 204
  regD_rule, // 205
  _PartialSubtypeCheck_o1RegP_o2RegP_rule, // 206
  iRegP_rule, // 207
  iRegP_rule, // 208
  0, // 209
  0, // 210
  0, // 211
  0, // 212
  0, // 213
  0, // 214
  0, // 215
  0, // 216
  0, // 217
  memory_rule, // 218
  _LoadB_memory__rule, // 219
  _ConvI2L__LoadB_memory___rule, // 220
  _ConvI2L__LoadC_memory___rule, // 221
  memory_rule, // 222
  memory_rule, // 223
  memory_rule, // 224
  memory_rule, // 225
  memory_rule, // 226
  memory_rule, // 227
  memory_rule, // 228
  memory_rule, // 229
  memory_rule, // 230
  memory_rule, // 231
  memory_rule, // 232
  memory_rule, // 233
  memory_rule, // 234
  memory_rule, // 235
  memory_rule, // 236
  memory_rule, // 237
  memory_rule, // 238
  memory_rule, // 239
  memory_rule, // 240
  memory_rule, // 241
  memory_rule, // 242
  memory_rule, // 243
  memory_rule, // 244
  memory_rule, // 245
  memory_rule, // 246
  memory_rule, // 247
  memory_rule, // 248
  memory_rule, // 249
  memory_rule, // 250
  memory_rule, // 251
  memory_rule, // 252
  memory_rule, // 253
  memory_rule, // 254
  memory_rule, // 255
  memory_rule, // 256
  memory_rule, // 257
  memory_rule, // 258
  memory_rule, // 259
  memory_rule, // 260
  iRegP_rule, // 261
  iRegP_rule, // 262
  iRegN_rule, // 263
  iRegN_rule, // 264
  memory_rule, // 265
  memory_rule, // 266
  memory_rule, // 267
  memory_rule, // 268
  memory_rule, // 269
  0, // 270
  0, // 271
  0, // 272
  0, // 273
  0, // 274
  0, // 275
  _Binary_cmpOpP_flagsRegP_rule, // 276
  _Binary_cmpOpP_flagsRegP_rule, // 277
  _Binary_cmpOp_flagsReg_rule, // 278
  _Binary_cmpOp_flagsReg_rule, // 279
  _Binary_cmpOpU_flagsRegU_rule, // 280
  _Binary_cmpOpU_flagsRegU_rule, // 281
  _Binary_cmpOpF_flagsRegF_rule, // 282
  _Binary_cmpOpF_flagsRegF_rule, // 283
  _Binary_cmpOpP_flagsRegP_rule, // 284
  _Binary_cmpOp_flagsReg_rule, // 285
  _Binary_cmpOpF_flagsRegF_rule, // 286
  _Binary_cmpOpP_flagsRegP_rule, // 287
  _Binary_cmpOpP_flagsRegP_rule, // 288
  _Binary_cmpOp_flagsReg_rule, // 289
  _Binary_cmpOp_flagsReg_rule, // 290
  _Binary_cmpOpF_flagsRegF_rule, // 291
  _Binary_cmpOpF_flagsRegF_rule, // 292
  _Binary_cmpOpP_flagsRegP_rule, // 293
  _Binary_cmpOp_flagsReg_rule, // 294
  _Binary_cmpOpF_flagsRegF_rule, // 295
  _Binary_cmpOpP_flagsRegP_rule, // 296
  _Binary_cmpOp_flagsReg_rule, // 297
  _Binary_cmpOpF_flagsRegF_rule, // 298
  _Binary_cmpOpP_flagsRegP_rule, // 299
  _Binary_cmpOpP_flagsRegP_rule, // 300
  _Binary_cmpOp_flagsReg_rule, // 301
  _Binary_cmpOpF_flagsRegF_rule, // 302
  iRegI_rule, // 303
  iRegI_rule, // 304
  iRegP_rule, // 305
  iRegP_rule, // 306
  iRegL_rule, // 307
  iRegL_rule, // 308
  memory_rule, // 309
  memory_rule, // 310
  iRegP_rule, // 311
  iRegP_rule, // 312
  iRegP_rule, // 313
  iRegP_rule, // 314
  iRegI_rule, // 315
  iRegI_rule, // 316
  immI0_rule, // 317
  iRegL_rule, // 318
  iRegL_rule, // 319
  immL0_rule, // 320
  iRegI_rule, // 321
  iRegI_rule, // 322
  iRegL_rule, // 323
  iRegL_rule, // 324
  iRegIsafe_rule, // 325
  iRegIsafe_rule, // 326
  0, // 327
  0, // 328
  iRegIsafe_rule, // 329
  iRegL_rule, // 330
  iRegL_rule, // 331
  iRegIsafe_rule, // 332
  iRegIsafe_rule, // 333
  0, // 334
  0, // 335
  0, // 336
  0, // 337
  0, // 338
  0, // 339
  iRegL_rule, // 340
  iRegL_rule, // 341
  iRegI_rule, // 342
  iRegI_rule, // 343
  iRegL_rule, // 344
  iRegL_rule, // 345
  iRegI_rule, // 346
  iRegI_rule, // 347
  iRegL_rule, // 348
  iRegL_rule, // 349
  iRegI_rule, // 350
  iRegI_rule, // 351
  iRegL_rule, // 352
  iRegL_rule, // 353
  _CastP2X_iRegP__rule, // 354
  regF_rule, // 355
  regD_rule, // 356
  regF_rule, // 357
  regD_rule, // 358
  regF_rule, // 359
  regD_rule, // 360
  regF_rule, // 361
  regD_rule, // 362
  _SqrtD__ConvF2D_regF___rule, // 363
  regD_rule, // 364
  iRegI_rule, // 365
  iRegI_rule, // 366
  iRegL_rule, // 367
  iRegL_rule, // 368
  iRegI_rule, // 369
  iRegI_rule, // 370
  iRegL_rule, // 371
  iRegL_rule, // 372
  iRegL_rule, // 373
  _CastP2X_sp_ptr_RegP__rule, // 374
  iRegI_rule, // 375
  iRegI_rule, // 376
  iRegL_rule, // 377
  iRegL_rule, // 378
  iRegI_rule, // 379
  iRegI_rule, // 380
  _AndI__CmpLTMask_iRegI_iRegI_iRegI_rule, // 381
  _SubI_iRegI_iRegI_rule, // 382
  _AndI_iRegI__CmpLTMask_iRegI_iRegI_rule, // 383
  _SubI_iRegI_iRegI_rule, // 384
  _SubI_iRegI_iRegI_rule, // 385
  _AndI__CmpLTMask_iRegI_iRegI_iRegI_rule, // 386
  _SubI_iRegI_iRegI_rule, // 387
  _AndI_iRegI__CmpLTMask_iRegI_iRegI_rule, // 388
  regD_rule, // 389
  0, // 390
  regD_rule, // 391
  0, // 392
  regD_rule, // 393
  regF_rule, // 394
  0, // 395
  regF_rule, // 396
  0, // 397
  regF_rule, // 398
  stackSlotI_rule, // 399
  _LoadI_memory__rule, // 400
  stackSlotI_rule, // 401
  _LoadI_memory__rule, // 402
  iRegI_rule, // 403
  _ConvI2L_iRegI__rule, // 404
  iRegL_rule, // 405
  regF_rule, // 406
  iRegI_rule, // 407
  regD_rule, // 408
  iRegL_rule, // 409
  0, // 410
  0, // 411
  0, // 412
  0, // 413
  0, // 414
  0, // 415
  stackSlotL_rule, // 416
  stackSlotL_rule, // 417
  stackSlotL_rule, // 418
  iRegL_rule, // 419
  _RShiftL_iRegL_immI_32_63_rule, // 420
  iRegI_rule, // 421
  immI13_rule, // 422
  iRegI_rule, // 423
  immI_rule, // 424
  iRegI_rule, // 425
  immI_rule, // 426
  iRegI_rule, // 427
  immI_rule, // 428
  0, // 429
  iRegI_rule, // 430
  0, // 431
  iRegI_rule, // 432
  regF_rule, // 433
  regD_rule, // 434
  iRegL_rule, // 435
  0, // 436
  cmpOp_rule, // 437
  cmpOp_reg_rule, // 438
  cmpOp_reg_rule, // 439
  cmpOp_reg_rule, // 440
  cmpOpU_rule, // 441
  cmpOpP_rule, // 442
  cmpOpF_rule, // 443
  cmpOp_rule, // 444
  cmpOpU_rule, // 445
  cmpOp_rule, // 446
  iRegL_rule, // 447
  _Binary_cmpOp_flagsRegL_rule, // 448
  _Binary_cmpOp_flagsRegL_rule, // 449
  _Binary_cmpOp_flagsRegL_rule, // 450
  _Binary_cmpOp_flagsRegL_rule, // 451
  _Binary_cmpOp_flagsRegL_rule, // 452
  _Binary_cmpOp_flagsRegL_rule, // 453
  _Binary_cmpOp_flagsRegL_rule, // 454
  _Binary_cmpOp_flagsRegL_rule, // 455
  _Binary_cmpOp_flagsRegL_rule, // 456
  iRegP_rule, // 457
  0, // 458
  0, // 459
  0, // 460
  0, // 461
  0, // 462
  g3RegP_rule, // 463
  0, // 464
  g1RegP_rule, // 465
  0, // 466
  0, // 467
  0, // 468
  o1RegP_rule, // 469
  iRegL_rule, // 470
  o0RegP_rule, // 471
  stackSlotI_rule, // 472
  stackSlotL_rule, // 473
  _LoadI_memory__rule, // 474
  _LoadL_memory__rule, // 475
  memory_rule, // 476
  memory_rule, // 477
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  0, // 8
  0, // 9
  0, // 10
  0, // 11
  0, // 12
  0, // 13
  0, // 14
  0, // 15
  0, // 16
  0, // 17
  0, // 18
  0, // 19
  0, // 20
  0, // 21
  0, // 22
  0, // 23
  0, // 24
  0, // 25
  0, // 26
  0, // 27
  0, // 28
  0, // 29
  0, // 30
  0, // 31
  0, // 32
  0, // 33
  0, // 34
  0, // 35
  0, // 36
  0, // 37
  0, // 38
  0, // 39
  0, // 40
  0, // 41
  0, // 42
  0, // 43
  0, // 44
  0, // 45
  0, // 46
  0, // 47
  0, // 48
  0, // 49
  0, // 50
  0, // 51
  0, // 52
  0, // 53
  0, // 54
  0, // 55
  0, // 56
  0, // 57
  0, // 58
  0, // 59
  0, // 60
  0, // 61
  0, // 62
  0, // 63
  0, // 64
  0, // 65
  0, // 66
  0, // 67
  0, // 68
  0, // 69
  0, // 70
  0, // 71
  0, // 72
  0, // 73
  0, // 74
  0, // 75
  0, // 76
  immL13_rule, // 77
  iRegL_rule, // 78
  0, // 79
  0, // 80
  0, // 81
  0, // 82
  0, // 83
  0, // 84
  0, // 85
  0, // 86
  0, // 87
  0, // 88
  0, // 89
  // last operand
  0, // 90
  // last operand class
  0, // 91
  0, // 92
  0, // 93
  0, // 94
  0, // 95
  flagsRegP_rule, // 96
  iRegI_rule, // 97
  immI11_rule, // 98
  flagsReg_rule, // 99
  flagsRegU_rule, // 100
  flagsRegF_rule, // 101
  iRegN_rule, // 102
  iRegP_rule, // 103
  immP0_rule, // 104
  regF_rule, // 105
  regD_rule, // 106
  iRegL_rule, // 107
  immI11_rule, // 108
  g3RegP_rule, // 109
  g3RegI_rule, // 110
  g3RegL_rule, // 111
  0, // 112
  0, // 113
  0, // 114
  0, // 115
  iRegI_rule, // 116
  iRegI_rule, // 117
  iRegI_rule, // 118
  _CmpLTMask_iRegI_iRegI_rule, // 119
  0, // 120
  0, // 121
  immI_32_63_rule, // 122
  iRegI_rule, // 123
  immI13_rule, // 124
  iRegL_rule, // 125
  immL13_rule, // 126
  immI0_rule, // 127
  immP0_rule, // 128
  immL0_rule, // 129
  flagsRegL_rule, // 130
  immL0_rule, // 131
  o2RegP_rule, // 132
  0, // 133
  0, // 134
  0, // 135
  // last internally defined operand
  0, // 136
  0, // 137
  0, // 138
  0, // 139
  0, // 140
  0, // 141
  0, // 142
  0, // 143
  0, // 144
  0, // 145
  0, // 146
  0, // 147
  0, // 148
  0, // 149
  0, // 150
  0, // 151
  0, // 152
  0, // 153
  0, // 154
  0, // 155
  0, // 156
  0, // 157
  0, // 158
  0, // 159
  0, // 160
  0, // 161
  0, // 162
  0, // 163
  0, // 164
  0, // 165
  0, // 166
  _Binary_iRegP_g3RegP_rule, // 167
  _Binary_iRegI_g3RegI_rule, // 168
  _Binary_iRegL_g3RegL_rule, // 169
  0, // 170
  0, // 171
  0, // 172
  0, // 173
  0, // 174
  0, // 175
  0, // 176
  0, // 177
  0, // 178
  0, // 179
  0, // 180
  0, // 181
  0, // 182
  0, // 183
  0, // 184
  0, // 185
  0, // 186
  0, // 187
  0, // 188
  0, // 189
  iRegI_rule, // 190
  iRegI_rule, // 191
  immI13_rule, // 192
  immI0_rule, // 193
  immI0_rule, // 194
  iRegL_rule, // 195
  immL13_rule, // 196
  immL0_rule, // 197
  immL0_rule, // 198
  immU13_rule, // 199
  iRegP_rule, // 200
  immP13_rule, // 201
  iRegN_rule, // 202
  immN0_rule, // 203
  regF_rule, // 204
  regD_rule, // 205
  immP0_rule, // 206
  iRegP_rule, // 207
  iRegP_rule, // 208
  0, // 209
  0, // 210
  0, // 211
  0, // 212
  0, // 213
  0, // 214
  0, // 215
  0, // 216
  0, // 217
  0, // 218
  immI_255_rule, // 219
  immL_FF_rule, // 220
  immL_FFFF_rule, // 221
  0, // 222
  0, // 223
  0, // 224
  0, // 225
  0, // 226
  0, // 227
  0, // 228
  0, // 229
  0, // 230
  0, // 231
  0, // 232
  0, // 233
  0, // 234
  0, // 235
  0, // 236
  0, // 237
  0, // 238
  0, // 239
  0, // 240
  0, // 241
  iRegI_rule, // 242
  immI0_rule, // 243
  immI0_rule, // 244
  iRegI_rule, // 245
  immI0_rule, // 246
  iRegI_rule, // 247
  iRegL_rule, // 248
  immI0_rule, // 249
  immL0_rule, // 250
  regF_rule, // 251
  sp_ptr_RegP_rule, // 252
  immP0_rule, // 253
  iRegN_rule, // 254
  immN0_rule, // 255
  regD_rule, // 256
  immD0_rule, // 257
  regF_rule, // 258
  immF0_rule, // 259
  regD_rule, // 260
  0, // 261
  0, // 262
  0, // 263
  0, // 264
  immI0_rule, // 265
  regD_rule, // 266
  _Replicate4C_immI0__rule, // 267
  regD_rule, // 268
  immI0_rule, // 269
  0, // 270
  0, // 271
  0, // 272
  0, // 273
  0, // 274
  0, // 275
  _Binary_iRegI_iRegI_rule, // 276
  _Binary_iRegI_immI11_rule, // 277
  _Binary_iRegI_iRegI_rule, // 278
  _Binary_iRegI_immI11_rule, // 279
  _Binary_iRegI_iRegI_rule, // 280
  _Binary_iRegI_immI11_rule, // 281
  _Binary_iRegI_iRegI_rule, // 282
  _Binary_iRegI_immI11_rule, // 283
  _Binary_iRegN_iRegN_rule, // 284
  _Binary_iRegN_iRegN_rule, // 285
  _Binary_iRegN_iRegN_rule, // 286
  _Binary_iRegP_iRegP_rule, // 287
  _Binary_iRegP_immP0_rule, // 288
  _Binary_iRegP_iRegP_rule, // 289
  _Binary_iRegP_immP0_rule, // 290
  _Binary_iRegP_iRegP_rule, // 291
  _Binary_iRegP_immP0_rule, // 292
  _Binary_regF_regF_rule, // 293
  _Binary_regF_regF_rule, // 294
  _Binary_regF_regF_rule, // 295
  _Binary_regD_regD_rule, // 296
  _Binary_regD_regD_rule, // 297
  _Binary_regD_regD_rule, // 298
  _Binary_iRegL_iRegL_rule, // 299
  _Binary_iRegL_immI11_rule, // 300
  _Binary_iRegL_iRegL_rule, // 301
  _Binary_iRegL_iRegL_rule, // 302
  iRegI_rule, // 303
  immI13_rule, // 304
  iRegL_rule, // 305
  immL13_rule, // 306
  iRegL_rule, // 307
  immL13_rule, // 308
  0, // 309
  0, // 310
  _Binary_iRegL_iRegL_rule, // 311
  _Binary_iRegI_iRegI_rule, // 312
  _Binary_iRegP_iRegP_rule, // 313
  _Binary_iRegN_iRegN_rule, // 314
  iRegI_rule, // 315
  immI13_rule, // 316
  iRegI_rule, // 317
  iRegL_rule, // 318
  immL13_rule, // 319
  iRegL_rule, // 320
  iRegI_rule, // 321
  immI13_rule, // 322
  iRegL_rule, // 323
  immL13_rule, // 324
  iRegIsafe_rule, // 325
  immI13_rule, // 326
  0, // 327
  0, // 328
  immI10_rule, // 329
  iRegL_rule, // 330
  immL13_rule, // 331
  iRegIsafe_rule, // 332
  immI13_rule, // 333
  0, // 334
  0, // 335
  0, // 336
  0, // 337
  0, // 338
  0, // 339
  iRegL_rule, // 340
  immL13_rule, // 341
  iRegI_rule, // 342
  immU5_rule, // 343
  iRegI_rule, // 344
  immU6_rule, // 345
  iRegI_rule, // 346
  immU5_rule, // 347
  iRegI_rule, // 348
  immU6_rule, // 349
  iRegI_rule, // 350
  immU5_rule, // 351
  iRegI_rule, // 352
  immU6_rule, // 353
  immU6_rule, // 354
  regF_rule, // 355
  regD_rule, // 356
  regF_rule, // 357
  regD_rule, // 358
  regF_rule, // 359
  regD_rule, // 360
  regF_rule, // 361
  regD_rule, // 362
  0, // 363
  0, // 364
  iRegI_rule, // 365
  immI13_rule, // 366
  iRegL_rule, // 367
  immL13_rule, // 368
  iRegI_rule, // 369
  immI13_rule, // 370
  iRegL_rule, // 371
  immL13_rule, // 372
  _CastP2X_sp_ptr_RegP__rule, // 373
  iRegL_rule, // 374
  iRegI_rule, // 375
  immI13_rule, // 376
  iRegL_rule, // 377
  immL13_rule, // 378
  0, // 379
  iRegI_rule, // 380
  _SubI_iRegI_iRegI_rule, // 381
  _AndI__CmpLTMask_iRegI_iRegI_iRegI_rule, // 382
  _SubI_iRegI_iRegI_rule, // 383
  _AndI_iRegI__CmpLTMask_iRegI_iRegI_rule, // 384
  _AndI__CmpLTMask_iRegI_iRegI_iRegI_rule, // 385
  _SubI_iRegI_iRegI_rule, // 386
  _AndI_iRegI__CmpLTMask_iRegI_iRegI_rule, // 387
  _SubI_iRegI_iRegI_rule, // 388
  0, // 389
  0, // 390
  0, // 391
  0, // 392
  0, // 393
  0, // 394
  0, // 395
  0, // 396
  0, // 397
  0, // 398
  0, // 399
  0, // 400
  0, // 401
  0, // 402
  0, // 403
  immL_32bits_rule, // 404
  immL_32bits_rule, // 405
  0, // 406
  0, // 407
  0, // 408
  0, // 409
  0, // 410
  0, // 411
  0, // 412
  0, // 413
  0, // 414
  0, // 415
  0, // 416
  0, // 417
  0, // 418
  0, // 419
  0, // 420
  0, // 421
  0, // 422
  0, // 423
  0, // 424
  0, // 425
  0, // 426
  0, // 427
  0, // 428
  0, // 429
  iRegI_rule, // 430
  0, // 431
  iRegI_rule, // 432
  regF_rule, // 433
  regD_rule, // 434
  0, // 435
  0, // 436
  flagsReg_rule, // 437
  _CmpI_iRegI_immI0_rule, // 438
  _CmpP_iRegP_immP0_rule, // 439
  _CmpL_iRegL_immL0_rule, // 440
  flagsRegU_rule, // 441
  flagsRegP_rule, // 442
  flagsRegF_rule, // 443
  flagsReg_rule, // 444
  flagsRegU_rule, // 445
  flagsRegL_rule, // 446
  iRegL_rule, // 447
  _Binary_iRegL_iRegL_rule, // 448
  _Binary_iRegL_immL0_rule, // 449
  _Binary_iRegI_iRegI_rule, // 450
  _Binary_iRegI_immI11_rule, // 451
  _Binary_iRegN_iRegN_rule, // 452
  _Binary_iRegP_iRegP_rule, // 453
  _Binary_iRegP_immP0_rule, // 454
  _Binary_regF_regF_rule, // 455
  _Binary_regD_regD_rule, // 456
  0, // 457
  0, // 458
  0, // 459
  0, // 460
  0, // 461
  0, // 462
  inline_cache_regP_rule, // 463
  0, // 464
  i0RegP_rule, // 465
  0, // 466
  0, // 467
  0, // 468
  o2RegP_rule, // 469
  iRegP_rule, // 470
  o1RegP_rule, // 471
  0, // 472
  0, // 473
  0, // 474
  0, // 475
  _ReverseBytesI_iRegI__rule, // 476
  _ReverseBytesL_iRegL__rule, // 477
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  "UNIVERSE", // 0
  "LABEL", // 1
  "SREGI", // 2
  "SREGP", // 3
  "SREGF", // 4
  "SREGD", // 5
  "SREGL", // 6
  "METHOD", // 7
  "IMMI", // 8
  "IMMI13", // 9
  "IMMU13", // 10
  "IMMU6", // 11
  "IMMI11", // 12
  "IMMI0", // 13
  "IMMI10", // 14
  "IMMU5", // 15
  "IMMI_1_31", // 16
  "IMMI_32_63", // 17
  "IMMI_255", // 18
  "IMML_FF", // 19
  "IMML_FFFF", // 20
  "IMMP", // 21
  "IMMP13", // 22
  "IMMP0", // 23
  "IMMP_POLL", // 24
  "IMMN", // 25
  "IMMN0", // 26
  "IMML", // 27
  "IMML0", // 28
  "IMML13", // 29
  "IMML_32BITS", // 30
  "IMMD", // 31
  "IMMD0", // 32
  "IMMF", // 33
  "IMMF0", // 34
  "IREGI", // 35
  "NOTEMP_IREGI", // 36
  "O0REGI", // 37
  "IREGP", // 38
  "SP_PTR_REGP", // 39
  "LOCK_PTR_REGP", // 40
  "G1REGP", // 41
  "G2REGP", // 42
  "G3REGP", // 43
  "G1REGI", // 44
  "G3REGI", // 45
  "G4REGI", // 46
  "G4REGP", // 47
  "I0REGP", // 48
  "O0REGP", // 49
  "O1REGP", // 50
  "O2REGP", // 51
  "O7REGP", // 52
  "L7REGP", // 53
  "O7REGI", // 54
  "IREGN", // 55
  "IREGL", // 56
  "O2REGL", // 57
  "O7REGL", // 58
  "G1REGL", // 59
  "G3REGL", // 60
  "IREGISAFE", // 61
  "FLAGSREG", // 62
  "FLAGSREGU", // 63
  "FLAGSREGP", // 64
  "FLAGSREGL", // 65
  "FLAGSREGF", // 66
  "FLAGSREGF0", // 67
  "FLAGSREG_LONG_LTGE", // 68
  "FLAGSREG_LONG_EQNE", // 69
  "FLAGSREG_LONG_LEGT", // 70
  "REGD", // 71
  "REGF", // 72
  "REGD_LOW", // 73
  "INLINE_CACHE_REGP", // 74
  "INTERPRETER_METHOD_OOP_REGP", // 75
  "INDIRECT", // 76
  "INDOFFSET13", // 77
  "INDINDEX", // 78
  "STACKSLOTI", // 79
  "STACKSLOTP", // 80
  "STACKSLOTF", // 81
  "STACKSLOTD", // 82
  "STACKSLOTL", // 83
  "CMPOP", // 84
  "CMPOPU", // 85
  "CMPOPP", // 86
  "CMPOP_REG", // 87
  "CMPOPF", // 88
  "CMPOP_COMMUTE", // 89
  // last operand
  "MEMORY", // 90
  // last operand class
  "_LoadB_memory_", // 91
  "_ConvI2L__LoadB_memory__", // 92
  "_LoadC_memory_", // 93
  "_ConvI2L__LoadC_memory__", // 94
  "_Replicate4C_immI0_", // 95
  "_Binary_cmpOpP_flagsRegP", // 96
  "_Binary_iRegI_iRegI", // 97
  "_Binary_iRegI_immI11", // 98
  "_Binary_cmpOp_flagsReg", // 99
  "_Binary_cmpOpU_flagsRegU", // 100
  "_Binary_cmpOpF_flagsRegF", // 101
  "_Binary_iRegN_iRegN", // 102
  "_Binary_iRegP_iRegP", // 103
  "_Binary_iRegP_immP0", // 104
  "_Binary_regF_regF", // 105
  "_Binary_regD_regD", // 106
  "_Binary_iRegL_iRegL", // 107
  "_Binary_iRegL_immI11", // 108
  "_Binary_iRegP_g3RegP", // 109
  "_Binary_iRegI_g3RegI", // 110
  "_Binary_iRegL_g3RegL", // 111
  "_CastP2X_iRegP_", // 112
  "_ConvF2D_regF_", // 113
  "_SqrtD__ConvF2D_regF__", // 114
  "_CastP2X_sp_ptr_RegP_", // 115
  "_CmpLTMask_iRegI_iRegI", // 116
  "_AndI__CmpLTMask_iRegI_iRegI_iRegI", // 117
  "_SubI_iRegI_iRegI", // 118
  "_AndI_iRegI__CmpLTMask_iRegI_iRegI", // 119
  "_LoadI_memory_", // 120
  "_ConvI2L_iRegI_", // 121
  "_RShiftL_iRegL_immI_32_63", // 122
  "_AndI_iRegI_iRegI", // 123
  "_AndI_iRegI_immI13", // 124
  "_AndL_iRegL_iRegL", // 125
  "_AndL_iRegL_immL13", // 126
  "_CmpI_iRegI_immI0", // 127
  "_CmpP_iRegP_immP0", // 128
  "_CmpL_iRegL_immL0", // 129
  "_Binary_cmpOp_flagsRegL", // 130
  "_Binary_iRegL_immL0", // 131
  "_PartialSubtypeCheck_o1RegP_o2RegP", // 132
  "_LoadL_memory_", // 133
  "_ReverseBytesI_iRegI_", // 134
  "_ReverseBytesL_iRegL_", // 135
  // last internally defined operand
  "regI_to_stkI", // 136
  "regL_to_stkL", // 137
  "regP_to_stkP", // 138
  "stfSSD", // 139
  "stfSSF", // 140
  "stkI_to_regI", // 141
  "stkL_to_regL", // 142
  "stkP_to_regP", // 143
  "loadConI", // 144
  "loadConI13", // 145
  "loadConP", // 146
  "loadConP0", // 147
  "loadConP_poll", // 148
  "loadConN0", // 149
  "loadConN", // 150
  "loadConL", // 151
  "loadConL0", // 152
  "loadConL13", // 153
  "loadConF", // 154
  "loadConD", // 155
  "ldfSSD", // 156
  "stkI_to_regF", // 157
  "stkL_to_regD", // 158
  "roundDouble_nop", // 159
  "roundFloat_nop", // 160
  "castX2P", // 161
  "castP2X", // 162
  "tlsLoadP", // 163
  "checkCastPP", // 164
  "castPP", // 165
  "castII", // 166
  "storePConditional", // 167
  "storeIConditional", // 168
  "storeLConditional", // 169
  "sra_31", // 170
  "sra_reg_2", // 171
  "absD_reg", // 172
  "absF_reg", // 173
  "negF_reg", // 174
  "negD_reg", // 175
  "convP2B", // 176
  "convI2D_helper", // 177
  "convI2F_helper", // 178
  "MoveF2I_stack_reg", // 179
  "MoveI2F_stack_reg", // 180
  "MoveD2L_stack_reg", // 181
  "MoveL2D_stack_reg", // 182
  "convI2D_regDHi_regD", // 183
  "convL2D_helper", // 184
  "convL2F_helper", // 185
  "Repl8B_reg_helper", // 186
  "Repl4C_reg_helper", // 187
  "Repl4S_reg_helper", // 188
  "Repl2I_reg_helper", // 189
  "compI_iReg", // 190
  "compU_iReg", // 191
  "compI_iReg_imm13", // 192
  "testI_reg_reg", // 193
  "testI_reg_imm", // 194
  "compL_reg_reg", // 195
  "compL_reg_con", // 196
  "testL_reg_reg", // 197
  "testL_reg_con", // 198
  "compU_iReg_imm13", // 199
  "compP_iRegP", // 200
  "compP_iRegP_imm13", // 201
  "compN_iRegN", // 202
  "compN_iRegN_immN0", // 203
  "cmpF_cc", // 204
  "cmpD_cc", // 205
  "partialSubtypeCheck_vs_zero", // 206
  "cmpFastLock", // 207
  "cmpFastUnlock", // 208
  "regF_to_stkI", // 209
  "regD_to_stkL", // 210
  "regI_to_stkLHi", // 211
  "regL_to_stkD", // 212
  "Nop_A0", // 213
  "Nop_A1", // 214
  "Nop_MS", // 215
  "Nop_FA", // 216
  "Nop_BR", // 217
  "loadB", // 218
  "loadUB", // 219
  "loadUBL", // 220
  "loadUCL", // 221
  "loadC", // 222
  "loadI", // 223
  "loadL", // 224
  "loadL_unaligned", // 225
  "loadA8B", // 226
  "loadA4C", // 227
  "loadA4S", // 228
  "loadA2I", // 229
  "loadRange", // 230
  "loadI_freg", // 231
  "loadP", // 232
  "loadN", // 233
  "loadKlass", // 234
  "loadNKlass", // 235
  "loadS", // 236
  "loadD", // 237
  "loadD_unaligned", // 238
  "loadF", // 239
  "prefetchr", // 240
  "prefetchw", // 241
  "storeB", // 242
  "storeB0", // 243
  "storeCM0", // 244
  "storeC", // 245
  "storeC0", // 246
  "storeI", // 247
  "storeL", // 248
  "storeI0", // 249
  "storeL0", // 250
  "storeI_Freg", // 251
  "storeP", // 252
  "storeP0", // 253
  "storeN", // 254
  "storeN0", // 255
  "storeD", // 256
  "storeD0", // 257
  "storeF", // 258
  "storeF0", // 259
  "storeA8B", // 260
  "encodeHeapOop", // 261
  "encodeHeapOop_not_null", // 262
  "decodeHeapOop", // 263
  "decodeHeapOop_not_null", // 264
  "storeA8B0", // 265
  "storeA4C", // 266
  "storeA4C0", // 267
  "storeA2I", // 268
  "storeA2I0", // 269
  "membar_acquire", // 270
  "membar_acquire_lock", // 271
  "membar_release", // 272
  "membar_release_lock", // 273
  "membar_volatile", // 274
  "unnecessary_membar_volatile", // 275
  "cmovIP_reg", // 276
  "cmovIP_imm", // 277
  "cmovII_reg", // 278
  "cmovII_imm", // 279
  "cmovII_U_reg", // 280
  "cmovII_U_imm", // 281
  "cmovIF_reg", // 282
  "cmovIF_imm", // 283
  "cmovNP_reg", // 284
  "cmovNI_reg", // 285
  "cmovNF_reg", // 286
  "cmovPP_reg", // 287
  "cmovPP_imm", // 288
  "cmovPI_reg", // 289
  "cmovPI_imm", // 290
  "cmovPF_reg", // 291
  "cmovPF_imm", // 292
  "cmovFP_reg", // 293
  "cmovFI_reg", // 294
  "cmovFF_reg", // 295
  "cmovDP_reg", // 296
  "cmovDI_reg", // 297
  "cmovDF_reg", // 298
  "cmovLP_reg", // 299
  "cmovLP_imm", // 300
  "cmovLI_reg", // 301
  "cmovLF_reg", // 302
  "addI_reg_reg", // 303
  "addI_reg_imm13", // 304
  "addP_reg_reg", // 305
  "addP_reg_imm13", // 306
  "addL_reg_reg", // 307
  "addL_reg_imm13", // 308
  "loadPLocked", // 309
  "loadLLocked", // 310
  "compareAndSwapL_bool", // 311
  "compareAndSwapI_bool", // 312
  "compareAndSwapP_bool", // 313
  "compareAndSwapN_bool", // 314
  "subI_reg_reg", // 315
  "subI_reg_imm13", // 316
  "subI_zero_reg", // 317
  "subL_reg_reg", // 318
  "subL_reg_imm13", // 319
  "negL_reg_reg", // 320
  "mulI_reg_reg", // 321
  "mulI_reg_imm13", // 322
  "mulL_reg_reg", // 323
  "mulL_reg_imm13", // 324
  "divI_reg_reg", // 325
  "divI_reg_imm13", // 326
  "mul_hi", // 327
  "loadConI_x66666667", // 328
  "divI_10", // 329
  "divL_reg_reg", // 330
  "divL_reg_imm13", // 331
  "modI_reg_reg", // 332
  "modI_reg_imm13", // 333
  "divL_reg_reg_1", // 334
  "divL_reg_imm13_1", // 335
  "mulL_reg_reg_1", // 336
  "mulL_reg_imm13_1", // 337
  "subL_reg_reg_1", // 338
  "subL_reg_reg_2", // 339
  "modL_reg_reg", // 340
  "modL_reg_imm13", // 341
  "shlI_reg_reg", // 342
  "shlI_reg_imm5", // 343
  "shlL_reg_reg", // 344
  "shlL_reg_imm6", // 345
  "sarI_reg_reg", // 346
  "sarI_reg_imm5", // 347
  "sarL_reg_reg", // 348
  "sarL_reg_imm6", // 349
  "shrI_reg_reg", // 350
  "shrI_reg_imm5", // 351
  "shrL_reg_reg", // 352
  "shrL_reg_imm6", // 353
  "shrP_reg_imm6", // 354
  "addF_reg_reg", // 355
  "addD_reg_reg", // 356
  "subF_reg_reg", // 357
  "subD_reg_reg", // 358
  "mulF_reg_reg", // 359
  "mulD_reg_reg", // 360
  "divF_reg_reg", // 361
  "divD_reg_reg", // 362
  "sqrtF_reg_reg", // 363
  "sqrtD_reg_reg", // 364
  "andI_reg_reg", // 365
  "andI_reg_imm13", // 366
  "andL_reg_reg", // 367
  "andL_reg_imm13", // 368
  "orI_reg_reg", // 369
  "orI_reg_imm13", // 370
  "orL_reg_reg", // 371
  "orL_reg_imm13", // 372
  "orL_reg_castP2X", // 373
  "orL_reg_castP2X_0", // 374
  "xorI_reg_reg", // 375
  "xorI_reg_imm13", // 376
  "xorL_reg_reg", // 377
  "xorL_reg_imm13", // 378
  "convI2B", // 379
  "cmpLTMask_reg_reg", // 380
  "cadd_cmpLTMask", // 381
  "cadd_cmpLTMask_1", // 382
  "cadd_cmpLTMask_0", // 383
  "cadd_cmpLTMask_2", // 384
  "cadd_cmpLTMask2", // 385
  "cadd_cmpLTMask2_1", // 386
  "cadd_cmpLTMask2_0", // 387
  "cadd_cmpLTMask2_2", // 388
  "convD2F_reg", // 389
  "convD2I_helper", // 390
  "convD2I_reg", // 391
  "convD2L_helper", // 392
  "convD2L_reg", // 393
  "convF2D_reg", // 394
  "convF2I_helper", // 395
  "convF2I_reg", // 396
  "convF2L_helper", // 397
  "convF2L_reg", // 398
  "convI2D_reg", // 399
  "convI2D_mem", // 400
  "convI2F_reg", // 401
  "convI2F_mem", // 402
  "convI2L_reg", // 403
  "convI2L_reg_zex", // 404
  "zerox_long", // 405
  "MoveF2I_reg_stack", // 406
  "MoveI2F_reg_stack", // 407
  "MoveD2L_reg_stack", // 408
  "MoveL2D_reg_stack", // 409
  "loadConI_x43300000", // 410
  "loadConI_x41f00000", // 411
  "regDHi_regDLo_to_regD", // 412
  "addD_regD_regD", // 413
  "subD_regD_regD", // 414
  "mulD_regD_regD", // 415
  "convL2D_reg_slow_fxtof", // 416
  "convL2D_reg_fast_fxtof", // 417
  "convL2F_reg_fast_fxtof", // 418
  "convL2I_reg", // 419
  "shrL_reg_imm6_L2I", // 420
  "Repl8B_reg", // 421
  "Repl8B_immI", // 422
  "Repl4C_reg", // 423
  "Repl4C_immI", // 424
  "Repl4S_reg", // 425
  "Repl4S_immI", // 426
  "Repl2I_reg", // 427
  "Repl2I_immI", // 428
  "cmovI_reg_lt", // 429
  "minI_eReg", // 430
  "cmovI_reg_gt", // 431
  "maxI_eReg", // 432
  "cmpF_reg", // 433
  "cmpD_reg", // 434
  "jumpXtnd", // 435
  "branch", // 436
  "branchCon", // 437
  "branchCon_regI", // 438
  "branchCon_regP", // 439
  "branchCon_regL", // 440
  "branchConU", // 441
  "branchConP", // 442
  "branchConF", // 443
  "branchLoopEnd", // 444
  "branchLoopEndU", // 445
  "branchCon_long", // 446
  "cmpL3_reg_reg", // 447
  "cmovLL_reg", // 448
  "cmovLL_imm", // 449
  "cmovIL_reg", // 450
  "cmovIL_imm", // 451
  "cmovNL_reg", // 452
  "cmovPL_reg", // 453
  "cmovPL_imm", // 454
  "cmovFL_reg", // 455
  "cmovDL_reg", // 456
  "safePoint_poll", // 457
  "CallStaticJavaDirect", // 458
  "CallDynamicJavaDirect", // 459
  "CallRuntimeDirect", // 460
  "CallLeafDirect", // 461
  "CallLeafNoFPDirect", // 462
  "TailCalljmpInd", // 463
  "Ret", // 464
  "tailjmpInd", // 465
  "CreateException", // 466
  "RethrowException", // 467
  "ShouldNotReachHere", // 468
  "partialSubtypeCheck", // 469
  "clear_array", // 470
  "string_compare", // 471
  "bytes_reverse_int", // 472
  "bytes_reverse_long", // 473
  "loadI_reversed", // 474
  "loadL_reversed", // 475
  "storeI_reversed", // 476
  "storeL_reversed", // 477
  // last instruction
  "no trailing comma"
};

const        bool  swallowed[] = {
  false, // 0
  false, // 1
  false, // 2
  false, // 3
  false, // 4
  false, // 5
  false, // 6
  false, // 7
  true, // 8
  true, // 9
  true, // 10
  true, // 11
  true, // 12
  true, // 13
  true, // 14
  true, // 15
  true, // 16
  true, // 17
  true, // 18
  true, // 19
  true, // 20
  true, // 21
  true, // 22
  true, // 23
  true, // 24
  true, // 25
  true, // 26
  true, // 27
  true, // 28
  true, // 29
  true, // 30
  true, // 31
  true, // 32
  true, // 33
  true, // 34
  false, // 35
  false, // 36
  false, // 37
  false, // 38
  false, // 39
  false, // 40
  false, // 41
  false, // 42
  false, // 43
  false, // 44
  false, // 45
  false, // 46
  false, // 47
  false, // 48
  false, // 49
  false, // 50
  false, // 51
  false, // 52
  false, // 53
  false, // 54
  false, // 55
  false, // 56
  false, // 57
  false, // 58
  false, // 59
  false, // 60
  false, // 61
  false, // 62
  false, // 63
  false, // 64
  false, // 65
  false, // 66
  false, // 67
  false, // 68
  false, // 69
  false, // 70
  false, // 71
  false, // 72
  false, // 73
  false, // 74
  false, // 75
  false, // 76
  false, // 77
  false, // 78
  false, // 79
  false, // 80
  false, // 81
  false, // 82
  false, // 83
  true, // 84
  true, // 85
  true, // 86
  true, // 87
  true, // 88
  true, // 89
  // last operand
  false, // 90
  // last operand class
  false, // 91
  false, // 92
  false, // 93
  false, // 94
  false, // 95
  false, // 96
  false, // 97
  false, // 98
  false, // 99
  false, // 100
  false, // 101
  false, // 102
  false, // 103
  false, // 104
  false, // 105
  false, // 106
  false, // 107
  false, // 108
  false, // 109
  false, // 110
  false, // 111
  false, // 112
  false, // 113
  false, // 114
  false, // 115
  false, // 116
  false, // 117
  false, // 118
  false, // 119
  false, // 120
  false, // 121
  false, // 122
  false, // 123
  false, // 124
  false, // 125
  false, // 126
  false, // 127
  false, // 128
  false, // 129
  false, // 130
  false, // 131
  false, // 132
  false, // 133
  false, // 134
  false, // 135
  // last internally defined operand
  false, // 136
  false, // 137
  false, // 138
  false, // 139
  false, // 140
  false, // 141
  false, // 142
  false, // 143
  false, // 144
  false, // 145
  false, // 146
  false, // 147
  false, // 148
  false, // 149
  false, // 150
  false, // 151
  false, // 152
  false, // 153
  false, // 154
  false, // 155
  false, // 156
  false, // 157
  false, // 158
  false, // 159
  false, // 160
  false, // 161
  false, // 162
  false, // 163
  false, // 164
  false, // 165
  false, // 166
  false, // 167
  false, // 168
  false, // 169
  false, // 170
  false, // 171
  false, // 172
  false, // 173
  false, // 174
  false, // 175
  false, // 176
  false, // 177
  false, // 178
  false, // 179
  false, // 180
  false, // 181
  false, // 182
  false, // 183
  false, // 184
  false, // 185
  false, // 186
  false, // 187
  false, // 188
  false, // 189
  false, // 190
  false, // 191
  false, // 192
  false, // 193
  false, // 194
  false, // 195
  false, // 196
  false, // 197
  false, // 198
  false, // 199
  false, // 200
  false, // 201
  false, // 202
  false, // 203
  false, // 204
  false, // 205
  false, // 206
  false, // 207
  false, // 208
  false, // 209
  false, // 210
  false, // 211
  false, // 212
  false, // 213
  false, // 214
  false, // 215
  false, // 216
  false, // 217
  false, // 218
  false, // 219
  false, // 220
  false, // 221
  false, // 222
  false, // 223
  false, // 224
  false, // 225
  false, // 226
  false, // 227
  false, // 228
  false, // 229
  false, // 230
  false, // 231
  false, // 232
  false, // 233
  false, // 234
  false, // 235
  false, // 236
  false, // 237
  false, // 238
  false, // 239
  false, // 240
  false, // 241
  false, // 242
  false, // 243
  false, // 244
  false, // 245
  false, // 246
  false, // 247
  false, // 248
  false, // 249
  false, // 250
  false, // 251
  false, // 252
  false, // 253
  false, // 254
  false, // 255
  false, // 256
  false, // 257
  false, // 258
  false, // 259
  false, // 260
  false, // 261
  false, // 262
  false, // 263
  false, // 264
  false, // 265
  false, // 266
  false, // 267
  false, // 268
  false, // 269
  false, // 270
  false, // 271
  false, // 272
  false, // 273
  false, // 274
  false, // 275
  false, // 276
  false, // 277
  false, // 278
  false, // 279
  false, // 280
  false, // 281
  false, // 282
  false, // 283
  false, // 284
  false, // 285
  false, // 286
  false, // 287
  false, // 288
  false, // 289
  false, // 290
  false, // 291
  false, // 292
  false, // 293
  false, // 294
  false, // 295
  false, // 296
  false, // 297
  false, // 298
  false, // 299
  false, // 300
  false, // 301
  false, // 302
  false, // 303
  false, // 304
  false, // 305
  false, // 306
  false, // 307
  false, // 308
  false, // 309
  false, // 310
  false, // 311
  false, // 312
  false, // 313
  false, // 314
  false, // 315
  false, // 316
  false, // 317
  false, // 318
  false, // 319
  false, // 320
  false, // 321
  false, // 322
  false, // 323
  false, // 324
  false, // 325
  false, // 326
  false, // 327
  false, // 328
  false, // 329
  false, // 330
  false, // 331
  false, // 332
  false, // 333
  false, // 334
  false, // 335
  false, // 336
  false, // 337
  false, // 338
  false, // 339
  false, // 340
  false, // 341
  false, // 342
  false, // 343
  false, // 344
  false, // 345
  false, // 346
  false, // 347
  false, // 348
  false, // 349
  false, // 350
  false, // 351
  false, // 352
  false, // 353
  false, // 354
  false, // 355
  false, // 356
  false, // 357
  false, // 358
  false, // 359
  false, // 360
  false, // 361
  false, // 362
  false, // 363
  false, // 364
  false, // 365
  false, // 366
  false, // 367
  false, // 368
  false, // 369
  false, // 370
  false, // 371
  false, // 372
  false, // 373
  false, // 374
  false, // 375
  false, // 376
  false, // 377
  false, // 378
  false, // 379
  false, // 380
  false, // 381
  false, // 382
  false, // 383
  false, // 384
  false, // 385
  false, // 386
  false, // 387
  false, // 388
  false, // 389
  false, // 390
  false, // 391
  false, // 392
  false, // 393
  false, // 394
  false, // 395
  false, // 396
  false, // 397
  false, // 398
  false, // 399
  false, // 400
  false, // 401
  false, // 402
  false, // 403
  false, // 404
  false, // 405
  false, // 406
  false, // 407
  false, // 408
  false, // 409
  false, // 410
  false, // 411
  false, // 412
  false, // 413
  false, // 414
  false, // 415
  false, // 416
  false, // 417
  false, // 418
  false, // 419
  false, // 420
  false, // 421
  false, // 422
  false, // 423
  false, // 424
  false, // 425
  false, // 426
  false, // 427
  false, // 428
  false, // 429
  false, // 430
  false, // 431
  false, // 432
  false, // 433
  false, // 434
  false, // 435
  false, // 436
  false, // 437
  false, // 438
  false, // 439
  false, // 440
  false, // 441
  false, // 442
  false, // 443
  false, // 444
  false, // 445
  false, // 446
  false, // 447
  false, // 448
  false, // 449
  false, // 450
  false, // 451
  false, // 452
  false, // 453
  false, // 454
  false, // 455
  false, // 456
  false, // 457
  false, // 458
  false, // 459
  false, // 460
  false, // 461
  false, // 462
  false, // 463
  false, // 464
  false, // 465
  false, // 466
  false, // 467
  false, // 468
  false, // 469
  false, // 470
  false, // 471
  false, // 472
  false, // 473
  false, // 474
  false, // 475
  false, // 476
  false, // 477
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // _last_machine_leaf: 9
  0, // AbsD: 10
  0, // AbsF: 11
  0, // AbsI: 12
  0, // AddD: 13
  0, // AddF: 14
  0, // AddI: 15
  0, // AddL: 16
  0, // AddP: 17
  0, // Allocate: 18
  0, // AllocateArray: 19
  0, // AndI: 20
  0, // AndL: 21
  0, // AryEq: 22
  0, // AtanD: 23
  1, // Binary: 24
  1, // Bool: 25
  0, // BoxLock: 26
  0, // ReverseBytesI: 27
  0, // ReverseBytesL: 28
  0, // CProj: 29
  0, // CallDynamicJava: 30
  0, // CallJava: 31
  0, // CallLeaf: 32
  0, // CallLeafNoFP: 33
  0, // CallRuntime: 34
  0, // CallStaticJava: 35
  0, // CastII: 36
  0, // CastX2P: 37
  0, // CastP2X: 38
  0, // CastPP: 39
  0, // Catch: 40
  0, // CatchProj: 41
  0, // CheckCastPP: 42
  0, // ClearArray: 43
  0, // ConstraintCast: 44
  0, // CMoveD: 45
  0, // CMoveF: 46
  0, // CMoveI: 47
  0, // CMoveL: 48
  0, // CMoveP: 49
  0, // CMoveN: 50
  1, // CmpN: 51
  1, // CmpD: 52
  0, // CmpD3: 53
  1, // CmpF: 54
  0, // CmpF3: 55
  1, // CmpI: 56
  1, // CmpL: 57
  0, // CmpL3: 58
  0, // CmpLTMask: 59
  1, // CmpP: 60
  1, // CmpU: 61
  0, // CompareAndSwapI: 62
  0, // CompareAndSwapL: 63
  0, // CompareAndSwapP: 64
  0, // CompareAndSwapN: 65
  0, // Con: 66
  0, // ConN: 67
  0, // ConD: 68
  0, // ConF: 69
  0, // ConI: 70
  0, // ConL: 71
  0, // ConP: 72
  0, // Conv2B: 73
  0, // ConvD2F: 74
  0, // ConvD2I: 75
  0, // ConvD2L: 76
  0, // ConvF2D: 77
  0, // ConvF2I: 78
  0, // ConvF2L: 79
  0, // ConvI2D: 80
  0, // ConvI2F: 81
  0, // ConvI2L: 82
  0, // ConvL2D: 83
  0, // ConvL2F: 84
  0, // ConvL2I: 85
  0, // CosD: 86
  0, // CountedLoop: 87
  0, // CountedLoopEnd: 88
  0, // CreateEx: 89
  0, // DecodeN: 90
  0, // DivD: 91
  0, // DivF: 92
  0, // DivI: 93
  0, // DivL: 94
  0, // DivMod: 95
  0, // DivModI: 96
  0, // DivModL: 97
  0, // EncodeP: 98
  0, // ExpD: 99
  1, // FastLock: 100
  1, // FastUnlock: 101
  0, // Goto: 102
  0, // Halt: 103
  0, // If: 104
  0, // IfFalse: 105
  0, // IfTrue: 106
  0, // Initialize: 107
  0, // JProj: 108
  0, // Jump: 109
  0, // JumpProj: 110
  0, // LShiftI: 111
  0, // LShiftL: 112
  0, // LoadB: 113
  0, // LoadC: 114
  0, // LoadD: 115
  0, // LoadD_unaligned: 116
  0, // LoadF: 117
  0, // LoadI: 118
  0, // LoadKlass: 119
  0, // LoadNKlass: 120
  0, // LoadL: 121
  0, // LoadL_unaligned: 122
  0, // LoadPLocked: 123
  0, // LoadLLocked: 124
  0, // LoadP: 125
  0, // LoadN: 126
  0, // LoadRange: 127
  0, // LoadS: 128
  0, // Lock: 129
  0, // LogD: 130
  0, // Log10D: 131
  0, // Loop: 132
  0, // Mach: 133
  0, // MachProj: 134
  0, // MaxI: 135
  0, // MemBarAcquire: 136
  0, // MemBarCPUOrder: 137
  0, // MemBarRelease: 138
  0, // MemBarVolatile: 139
  0, // MergeMem: 140
  0, // MinI: 141
  0, // ModD: 142
  0, // ModF: 143
  0, // ModI: 144
  0, // ModL: 145
  0, // MoveI2F: 146
  0, // MoveF2I: 147
  0, // MoveL2D: 148
  0, // MoveD2L: 149
  0, // MulD: 150
  0, // MulF: 151
  0, // MulHiL: 152
  0, // MulI: 153
  0, // MulL: 154
  0, // Multi: 155
  0, // NegD: 156
  0, // NegF: 157
  0, // NeverBranch: 158
  0, // Opaque1: 159
  0, // Opaque2: 160
  0, // OrI: 161
  0, // OrL: 162
  0, // PCTable: 163
  0, // Parm: 164
  0, // PartialSubtypeCheck: 165
  0, // Phi: 166
  0, // PowD: 167
  0, // PrefetchRead: 168
  0, // PrefetchWrite: 169
  0, // Proj: 170
  0, // RShiftI: 171
  0, // RShiftL: 172
  0, // Region: 173
  0, // Rethrow: 174
  0, // Return: 175
  0, // Root: 176
  0, // RoundDouble: 177
  0, // RoundFloat: 178
  0, // SafePoint: 179
  0, // SafePointScalarObject: 180
  0, // SCMemProj: 181
  0, // SinD: 182
  0, // SqrtD: 183
  0, // Start: 184
  0, // StartOSR: 185
  0, // StoreB: 186
  0, // StoreC: 187
  0, // StoreCM: 188
  0, // StorePConditional: 189
  0, // StoreIConditional: 190
  0, // StoreLConditional: 191
  0, // StoreD: 192
  0, // StoreF: 193
  0, // StoreI: 194
  0, // StoreL: 195
  0, // StoreP: 196
  0, // StoreN: 197
  0, // StrComp: 198
  0, // SubD: 199
  0, // SubF: 200
  0, // SubI: 201
  0, // SubL: 202
  0, // TailCall: 203
  0, // TailJump: 204
  0, // TanD: 205
  0, // ThreadLocal: 206
  0, // Unlock: 207
  0, // URShiftI: 208
  0, // URShiftL: 209
  0, // XorI: 210
  0, // XorL: 211
  0, // Vector: 212
  0, // AddVB: 213
  0, // AddVC: 214
  0, // AddVS: 215
  0, // AddVI: 216
  0, // AddVL: 217
  0, // AddVF: 218
  0, // AddVD: 219
  0, // SubVB: 220
  0, // SubVC: 221
  0, // SubVS: 222
  0, // SubVI: 223
  0, // SubVL: 224
  0, // SubVF: 225
  0, // SubVD: 226
  0, // MulVF: 227
  0, // MulVD: 228
  0, // DivVF: 229
  0, // DivVD: 230
  0, // LShiftVB: 231
  0, // LShiftVC: 232
  0, // LShiftVS: 233
  0, // LShiftVI: 234
  0, // URShiftVB: 235
  0, // URShiftVC: 236
  0, // URShiftVS: 237
  0, // URShiftVI: 238
  0, // AndV: 239
  0, // OrV: 240
  0, // XorV: 241
  0, // VectorLoad: 242
  0, // Load16B: 243
  0, // Load8B: 244
  0, // Load4B: 245
  0, // Load8C: 246
  0, // Load4C: 247
  0, // Load2C: 248
  0, // Load8S: 249
  0, // Load4S: 250
  0, // Load2S: 251
  0, // Load4I: 252
  0, // Load2I: 253
  0, // Load2L: 254
  0, // Load4F: 255
  0, // Load2F: 256
  0, // Load2D: 257
  0, // VectorStore: 258
  0, // Store16B: 259
  0, // Store8B: 260
  0, // Store4B: 261
  0, // Store8C: 262
  0, // Store4C: 263
  0, // Store2C: 264
  0, // Store4I: 265
  0, // Store2I: 266
  0, // Store2L: 267
  0, // Store4F: 268
  0, // Store2F: 269
  0, // Store2D: 270
  0, // Pack: 271
  0, // PackB: 272
  0, // PackS: 273
  0, // PackC: 274
  0, // PackI: 275
  0, // PackL: 276
  0, // PackF: 277
  0, // PackD: 278
  0, // Pack2x1B: 279
  0, // Pack2x2B: 280
  0, // Replicate16B: 281
  0, // Replicate8B: 282
  0, // Replicate4B: 283
  0, // Replicate8S: 284
  0, // Replicate4S: 285
  0, // Replicate2S: 286
  0, // Replicate8C: 287
  0, // Replicate4C: 288
  0, // Replicate2C: 289
  0, // Replicate4I: 290
  0, // Replicate2I: 291
  0, // Replicate2L: 292
  0, // Replicate4F: 293
  0, // Replicate2F: 294
  0, // Replicate2D: 295
  0, // Extract: 296
  0, // ExtractB: 297
  0, // ExtractS: 298
  0, // ExtractC: 299
  0, // ExtractI: 300
  0, // ExtractL: 301
  0, // ExtractF: 302
  0 // no trailing comma // ExtractD: 303
};
//  The following instructions can cisc-spill



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R_L0",
  "R_L0H",
  "R_L1",
  "R_L1H",
  "R_L2",
  "R_L2H",
  "R_L3",
  "R_L3H",
  "R_L4",
  "R_L4H",
  "R_L5",
  "R_L5H",
  "R_L6",
  "R_L6H",
  "R_L7",
  "R_L7H",
  "R_G0",
  "R_G0H",
  "R_G1",
  "R_G1H",
  "R_G2",
  "R_G2H",
  "R_G3",
  "R_G3H",
  "R_G4",
  "R_G4H",
  "R_G5",
  "R_G5H",
  "R_G6",
  "R_G6H",
  "R_G7",
  "R_G7H",
  "R_O7",
  "R_O7H",
  "R_SP",
  "R_SPH",
  "R_O0",
  "R_O0H",
  "R_O1",
  "R_O1H",
  "R_O2",
  "R_O2H",
  "R_O3",
  "R_O3H",
  "R_O4",
  "R_O4H",
  "R_O5",
  "R_O5H",
  "R_I0",
  "R_I0H",
  "R_I1",
  "R_I1H",
  "R_I2",
  "R_I2H",
  "R_I3",
  "R_I3H",
  "R_I4",
  "R_I4H",
  "R_I5",
  "R_I5H",
  "R_FP",
  "R_FPH",
  "R_I7",
  "R_I7H",
  "R_F8",
  "R_F9",
  "R_F10",
  "R_F11",
  "R_F12",
  "R_F13",
  "R_F14",
  "R_F15",
  "R_F16",
  "R_F17",
  "R_F18",
  "R_F19",
  "R_F20",
  "R_F21",
  "R_F22",
  "R_F23",
  "R_F24",
  "R_F25",
  "R_F26",
  "R_F27",
  "R_F28",
  "R_F29",
  "R_F30",
  "R_F31",
  "R_F0",
  "R_F1",
  "R_F2",
  "R_F3",
  "R_F4",
  "R_F5",
  "R_F6",
  "R_F7",
  "R_D32",
  "R_D32x",
  "R_D34",
  "R_D34x",
  "R_D36",
  "R_D36x",
  "R_D38",
  "R_D38x",
  "R_D40",
  "R_D40x",
  "R_D42",
  "R_D42x",
  "R_D44",
  "R_D44x",
  "R_D46",
  "R_D46x",
  "R_D48",
  "R_D48x",
  "R_D50",
  "R_D50x",
  "R_D52",
  "R_D52x",
  "R_D54",
  "R_D54x",
  "R_D56",
  "R_D56x",
  "R_D58",
  "R_D58x",
  "R_D60",
  "R_D60x",
  "R_D62",
  "R_D62x",
  "CCR",
  "FCC0",
  "FCC1",
  "FCC2",
  "FCC3" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	L0->as_VMReg(),
	L0->as_VMReg()->next(),
	L1->as_VMReg(),
	L1->as_VMReg()->next(),
	L2->as_VMReg(),
	L2->as_VMReg()->next(),
	L3->as_VMReg(),
	L3->as_VMReg()->next(),
	L4->as_VMReg(),
	L4->as_VMReg()->next(),
	L5->as_VMReg(),
	L5->as_VMReg()->next(),
	L6->as_VMReg(),
	L6->as_VMReg()->next(),
	L7->as_VMReg(),
	L7->as_VMReg()->next(),
	G0->as_VMReg(),
	G0->as_VMReg()->next(),
	G1->as_VMReg(),
	G1->as_VMReg()->next(),
	G2->as_VMReg(),
	G2->as_VMReg()->next(),
	G3->as_VMReg(),
	G3->as_VMReg()->next(),
	G4->as_VMReg(),
	G4->as_VMReg()->next(),
	G5->as_VMReg(),
	G5->as_VMReg()->next(),
	G6->as_VMReg(),
	G6->as_VMReg()->next(),
	G7->as_VMReg(),
	G7->as_VMReg()->next(),
	O7->as_VMReg(),
	O7->as_VMReg()->next(),
	SP->as_VMReg(),
	SP->as_VMReg()->next(),
	O0->as_VMReg(),
	O0->as_VMReg()->next(),
	O1->as_VMReg(),
	O1->as_VMReg()->next(),
	O2->as_VMReg(),
	O2->as_VMReg()->next(),
	O3->as_VMReg(),
	O3->as_VMReg()->next(),
	O4->as_VMReg(),
	O4->as_VMReg()->next(),
	O5->as_VMReg(),
	O5->as_VMReg()->next(),
	I0->as_VMReg(),
	I0->as_VMReg()->next(),
	I1->as_VMReg(),
	I1->as_VMReg()->next(),
	I2->as_VMReg(),
	I2->as_VMReg()->next(),
	I3->as_VMReg(),
	I3->as_VMReg()->next(),
	I4->as_VMReg(),
	I4->as_VMReg()->next(),
	I5->as_VMReg(),
	I5->as_VMReg()->next(),
	FP->as_VMReg(),
	FP->as_VMReg()->next(),
	I7->as_VMReg(),
	I7->as_VMReg()->next(),
	F8->as_VMReg(),
	F9->as_VMReg(),
	F10->as_VMReg(),
	F11->as_VMReg(),
	F12->as_VMReg(),
	F13->as_VMReg(),
	F14->as_VMReg(),
	F15->as_VMReg(),
	F16->as_VMReg(),
	F17->as_VMReg(),
	F18->as_VMReg(),
	F19->as_VMReg(),
	F20->as_VMReg(),
	F21->as_VMReg(),
	F22->as_VMReg(),
	F23->as_VMReg(),
	F24->as_VMReg(),
	F25->as_VMReg(),
	F26->as_VMReg(),
	F27->as_VMReg(),
	F28->as_VMReg(),
	F29->as_VMReg(),
	F30->as_VMReg(),
	F31->as_VMReg(),
	F0->as_VMReg(),
	F1->as_VMReg(),
	F2->as_VMReg(),
	F3->as_VMReg(),
	F4->as_VMReg(),
	F5->as_VMReg(),
	F6->as_VMReg(),
	F7->as_VMReg(),
	F32->as_VMReg()->next(),
	F32->as_VMReg(),
	F34->as_VMReg()->next(),
	F34->as_VMReg(),
	F36->as_VMReg()->next(),
	F36->as_VMReg(),
	F38->as_VMReg()->next(),
	F38->as_VMReg(),
	F40->as_VMReg()->next(),
	F40->as_VMReg(),
	F42->as_VMReg()->next(),
	F42->as_VMReg(),
	F44->as_VMReg()->next(),
	F44->as_VMReg(),
	F46->as_VMReg()->next(),
	F46->as_VMReg(),
	F48->as_VMReg()->next(),
	F48->as_VMReg(),
	F50->as_VMReg()->next(),
	F50->as_VMReg(),
	F52->as_VMReg()->next(),
	F52->as_VMReg(),
	F54->as_VMReg()->next(),
	F54->as_VMReg(),
	F56->as_VMReg()->next(),
	F56->as_VMReg(),
	F58->as_VMReg()->next(),
	F58->as_VMReg(),
	F60->as_VMReg()->next(),
	F60->as_VMReg(),
	F62->as_VMReg()->next(),
	F62->as_VMReg(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\x10',  // R_L0
  (unsigned char)'\x90',  // R_L0H
  (unsigned char)'\x11',  // R_L1
  (unsigned char)'\x91',  // R_L1H
  (unsigned char)'\x12',  // R_L2
  (unsigned char)'\x92',  // R_L2H
  (unsigned char)'\x13',  // R_L3
  (unsigned char)'\x93',  // R_L3H
  (unsigned char)'\x14',  // R_L4
  (unsigned char)'\x94',  // R_L4H
  (unsigned char)'\x15',  // R_L5
  (unsigned char)'\x95',  // R_L5H
  (unsigned char)'\x16',  // R_L6
  (unsigned char)'\x96',  // R_L6H
  (unsigned char)'\x17',  // R_L7
  (unsigned char)'\x97',  // R_L7H
  (unsigned char)'\x0',  // R_G0
  (unsigned char)'\x80',  // R_G0H
  (unsigned char)'\x1',  // R_G1
  (unsigned char)'\x81',  // R_G1H
  (unsigned char)'\x2',  // R_G2
  (unsigned char)'\x82',  // R_G2H
  (unsigned char)'\x3',  // R_G3
  (unsigned char)'\x83',  // R_G3H
  (unsigned char)'\x4',  // R_G4
  (unsigned char)'\x84',  // R_G4H
  (unsigned char)'\x5',  // R_G5
  (unsigned char)'\x85',  // R_G5H
  (unsigned char)'\x6',  // R_G6
  (unsigned char)'\x86',  // R_G6H
  (unsigned char)'\x7',  // R_G7
  (unsigned char)'\x87',  // R_G7H
  (unsigned char)'\xF',  // R_O7
  (unsigned char)'\x8F',  // R_O7H
  (unsigned char)'\xE',  // R_SP
  (unsigned char)'\x8E',  // R_SPH
  (unsigned char)'\x8',  // R_O0
  (unsigned char)'\x88',  // R_O0H
  (unsigned char)'\x9',  // R_O1
  (unsigned char)'\x89',  // R_O1H
  (unsigned char)'\xA',  // R_O2
  (unsigned char)'\x8A',  // R_O2H
  (unsigned char)'\xB',  // R_O3
  (unsigned char)'\x8B',  // R_O3H
  (unsigned char)'\xC',  // R_O4
  (unsigned char)'\x8C',  // R_O4H
  (unsigned char)'\xD',  // R_O5
  (unsigned char)'\x8D',  // R_O5H
  (unsigned char)'\x18',  // R_I0
  (unsigned char)'\x98',  // R_I0H
  (unsigned char)'\x19',  // R_I1
  (unsigned char)'\x99',  // R_I1H
  (unsigned char)'\x1A',  // R_I2
  (unsigned char)'\x9A',  // R_I2H
  (unsigned char)'\x1B',  // R_I3
  (unsigned char)'\x9B',  // R_I3H
  (unsigned char)'\x1C',  // R_I4
  (unsigned char)'\x9C',  // R_I4H
  (unsigned char)'\x1D',  // R_I5
  (unsigned char)'\x9D',  // R_I5H
  (unsigned char)'\x1E',  // R_FP
  (unsigned char)'\x9E',  // R_FPH
  (unsigned char)'\x1F',  // R_I7
  (unsigned char)'\x9F',  // R_I7H
  (unsigned char)'\x8',  // R_F8
  (unsigned char)'\x9',  // R_F9
  (unsigned char)'\xA',  // R_F10
  (unsigned char)'\xB',  // R_F11
  (unsigned char)'\xC',  // R_F12
  (unsigned char)'\xD',  // R_F13
  (unsigned char)'\xE',  // R_F14
  (unsigned char)'\xF',  // R_F15
  (unsigned char)'\x10',  // R_F16
  (unsigned char)'\x11',  // R_F17
  (unsigned char)'\x12',  // R_F18
  (unsigned char)'\x13',  // R_F19
  (unsigned char)'\x14',  // R_F20
  (unsigned char)'\x15',  // R_F21
  (unsigned char)'\x16',  // R_F22
  (unsigned char)'\x17',  // R_F23
  (unsigned char)'\x18',  // R_F24
  (unsigned char)'\x19',  // R_F25
  (unsigned char)'\x1A',  // R_F26
  (unsigned char)'\x1B',  // R_F27
  (unsigned char)'\x1C',  // R_F28
  (unsigned char)'\x1D',  // R_F29
  (unsigned char)'\x1E',  // R_F30
  (unsigned char)'\x1F',  // R_F31
  (unsigned char)'\x0',  // R_F0
  (unsigned char)'\x1',  // R_F1
  (unsigned char)'\x2',  // R_F2
  (unsigned char)'\x3',  // R_F3
  (unsigned char)'\x4',  // R_F4
  (unsigned char)'\x5',  // R_F5
  (unsigned char)'\x6',  // R_F6
  (unsigned char)'\x7',  // R_F7
  (unsigned char)'\x1',  // R_D32
  (unsigned char)'\xFF',  // R_D32x
  (unsigned char)'\x3',  // R_D34
  (unsigned char)'\xFF',  // R_D34x
  (unsigned char)'\x5',  // R_D36
  (unsigned char)'\xFF',  // R_D36x
  (unsigned char)'\x7',  // R_D38
  (unsigned char)'\xFF',  // R_D38x
  (unsigned char)'\x9',  // R_D40
  (unsigned char)'\xFF',  // R_D40x
  (unsigned char)'\xB',  // R_D42
  (unsigned char)'\xFF',  // R_D42x
  (unsigned char)'\xD',  // R_D44
  (unsigned char)'\xFF',  // R_D44x
  (unsigned char)'\xF',  // R_D46
  (unsigned char)'\xFF',  // R_D46x
  (unsigned char)'\x11',  // R_D48
  (unsigned char)'\xFF',  // R_D48x
  (unsigned char)'\x13',  // R_D50
  (unsigned char)'\xFF',  // R_D50x
  (unsigned char)'\x15',  // R_D52
  (unsigned char)'\xFF',  // R_D52x
  (unsigned char)'\x17',  // R_D54
  (unsigned char)'\xFF',  // R_D54x
  (unsigned char)'\x19',  // R_D56
  (unsigned char)'\xFF',  // R_D56x
  (unsigned char)'\x1B',  // R_D58
  (unsigned char)'\xFF',  // R_D58x
  (unsigned char)'\x1D',  // R_D60
  (unsigned char)'\xFF',  // R_D60x
  (unsigned char)'\x1F',  // R_D62
  (unsigned char)'\xFF',  // R_D62x
  (unsigned char)'\x0',  // CCR
  (unsigned char)'\x0',  // FCC0
  (unsigned char)'\x1',  // FCC1
  (unsigned char)'\x2',  // FCC2
  (unsigned char)'\x3' // no trailing comma  // FCC3
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone(Compile* C) const {
  return  new (C) labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone(Compile* C) const {
  return  new (C) methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *iRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *notemp_iRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NOTEMP_INT_REG_mask;
}

const RegMask *o0RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &O0_REGI_mask;
}

const RegMask *iRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask;
}

const RegMask *sp_ptr_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &SP_PTR_REG_mask;
}

const RegMask *lock_ptr_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LOCK_PTR_REG_mask;
}

const RegMask *g1RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G1_REGP_mask;
}

const RegMask *g2RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G2_REGP_mask;
}

const RegMask *g3RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G3_REGP_mask;
}

const RegMask *g1RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G1_REGI_mask;
}

const RegMask *g3RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G3_REGI_mask;
}

const RegMask *g4RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G4_REGI_mask;
}

const RegMask *g4RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G4_REGP_mask;
}

const RegMask *i0RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &I0_REGP_mask;
}

const RegMask *o0RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &O0_REGP_mask;
}

const RegMask *o1RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &O1_REGP_mask;
}

const RegMask *o2RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &O2_REGP_mask;
}

const RegMask *o7RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &O7_REGP_mask;
}

const RegMask *l7RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &L7_REGP_mask;
}

const RegMask *o7RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &O7_REGI_mask;
}

const RegMask *iRegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *iRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask;
}

const RegMask *o2RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &O2_REGL_mask;
}

const RegMask *o7RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &O7_REGL_mask;
}

const RegMask *g1RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G1_REGL_mask;
}

const RegMask *g3RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G3_REGL_mask;
}

const RegMask *iRegIsafeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask;
}

const RegMask *flagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_FLAGS_mask;
}

const RegMask *flagsRegF0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_FLAG0_mask;
}

const RegMask *flagsReg_long_LTGEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsReg_long_EQNEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsReg_long_LEGTOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DFLT_REG_mask;
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &SFLT_REG_mask;
}

const RegMask *regD_lowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DFLT_LOW_REG_mask;
}

const RegMask *inline_cache_regPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G5_REGP_mask;
}

const RegMask *interpreter_method_oop_regPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &G5_REGP_mask;
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &SP_PTR_REG_mask;
}

const RegMask *indOffset13Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &SP_PTR_REG_mask;
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

//------------------Define members for classes derived from MachNode----------

// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode( MachNode* node, Compile* C) const {
  // This architecture does not have cisc or short branch instructions
  ShouldNotCallThis();
}

void branchNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchConNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchCon_regINode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchCon_regPNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchCon_regLNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchConUNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchConPNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchConFNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchLoopEndNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchLoopEndUNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void branchCon_longNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int  loadBNode::reloc()   const {
  return  1;
}

int  loadUBNode::reloc()   const {
  return  1;
}

int  loadUBLNode::reloc()   const {
  return  1;
}

int  loadUCLNode::reloc()   const {
  return  1;
}

int  loadCNode::reloc()   const {
  return  1;
}

int  loadINode::reloc()   const {
  return  1;
}

int  loadLNode::reloc()   const {
  return  1;
}

int  loadL_unalignedNode::reloc()   const {
  return  1;
}

int  loadA8BNode::reloc()   const {
  return  1;
}

int  loadA4CNode::reloc()   const {
  return  1;
}

int  loadA4SNode::reloc()   const {
  return  1;
}

int  loadA2INode::reloc()   const {
  return  1;
}

int  loadRangeNode::reloc()   const {
  return  1;
}

int  loadI_fregNode::reloc()   const {
  return  1;
}

int  loadPNode::reloc()   const {
  return  1;
}

int  loadNNode::reloc()   const {
  return  1;
}

int  loadKlassNode::reloc()   const {
  return  1;
}

int  loadNKlassNode::reloc()   const {
  return  1;
}

int  loadSNode::reloc()   const {
  return  1;
}

int  loadDNode::reloc()   const {
  return  1;
}

int  loadD_unalignedNode::reloc()   const {
  return  1;
}

int  loadFNode::reloc()   const {
  return  1;
}

int  loadConPNode::reloc()   const {
  return  2;
}

int  loadConP0Node::reloc()   const {
  return  2;
}

int  loadConP_pollNode::reloc()   const {
  return  2;
}

int  loadConFNode::reloc()   const {
  return  7;
}

int  loadConDNode::reloc()   const {
  return  7;
}

int  prefetchrNode::reloc()   const {
  return  1;
}

int  prefetchwNode::reloc()   const {
  return  1;
}

int  storeBNode::reloc()   const {
  return  2;
}

int  storeB0Node::reloc()   const {
  return  2;
}

int  storeCM0Node::reloc()   const {
  return  2;
}

int  storeCNode::reloc()   const {
  return  2;
}

int  storeC0Node::reloc()   const {
  return  2;
}

int  storeINode::reloc()   const {
  return  2;
}

int  storeLNode::reloc()   const {
  return  2;
}

int  storeI0Node::reloc()   const {
  return  2;
}

int  storeL0Node::reloc()   const {
  return  2;
}

int  storeI_FregNode::reloc()   const {
  return  2;
}

int  storePNode::reloc()   const {
  return  2;
}

int  storeP0Node::reloc()   const {
  return  4;
}

int  storeNNode::reloc()   const {
  return  2;
}

int  storeN0Node::reloc()   const {
  return  2;
}

int  storeDNode::reloc()   const {
  return  2;
}

int  storeD0Node::reloc()   const {
  return  3;
}

int  storeFNode::reloc()   const {
  return  2;
}

int  storeF0Node::reloc()   const {
  return  3;
}

int  storeA8BNode::reloc()   const {
  return  2;
}

int  storeA8B0Node::reloc()   const {
  return  2;
}

int  storeA4CNode::reloc()   const {
  return  2;
}

int  storeA4C0Node::reloc()   const {
  return  2;
}

int  storeA2INode::reloc()   const {
  return  2;
}

int  storeA2I0Node::reloc()   const {
  return  2;
}

int  cmovPP_immNode::reloc()   const {
  return  2;
}

int  cmovPI_immNode::reloc()   const {
  return  2;
}

int  cmovPF_immNode::reloc()   const {
  return  2;
}

int  loadPLockedNode::reloc()   const {
  return  1;
}

int  loadLLockedNode::reloc()   const {
  return  1;
}

int  convI2D_memNode::reloc()   const {
  return  1;
}

int  convI2F_memNode::reloc()   const {
  return  1;
}

int  compP_iRegP_imm13Node::reloc()   const {
  return  2;
}

int  branchCon_regPNode::reloc()   const {
  return  2;
}

int  cmovPL_immNode::reloc()   const {
  return  2;
}

int  safePoint_pollNode::reloc()   const {
  return  1;
}

int  CallStaticJavaDirectNode::reloc()   const {
  return  1;
}

int  CallDynamicJavaDirectNode::reloc()   const {
  return  1;
}

int  CallRuntimeDirectNode::reloc()   const {
  return  1;
}

int  CallLeafDirectNode::reloc()   const {
  return  1;
}

int  CallLeafNoFPDirectNode::reloc()   const {
  return  1;
}

int  TailCalljmpIndNode::reloc()   const {
  return  1;
}

int  RetNode::reloc()   const {
  return  1;
}

int  tailjmpIndNode::reloc()   const {
  return  1;
}

int  RethrowExceptionNode::reloc()   const {
  return  1;
}

int  partialSubtypeCheck_vs_zeroNode::reloc()   const {
  return  2;
}

int  loadI_reversedNode::reloc()   const {
  return  1;
}

int  loadL_reversedNode::reloc()   const {
  return  1;
}

int  storeI_reversedNode::reloc()   const {
  return  2;
}

int  storeL_reversedNode::reloc()   const {
  return  2;
}


void  stkI_to_regFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  stkI_to_regFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  stkL_to_regDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lddf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  stkL_to_regDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  regF_to_stkINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stf_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  regF_to_stkINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  regD_to_stkLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stdf_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  regD_to_stkLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  regI_to_stkLHiNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stw_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    guarantee(opnd_array(0)->disp(ra_,this,0), "cannot offset a reg-reg operand by 4");
    emit_form3_mem_reg(cbuf, this, (Assembler::stw_op3 /*primary()*/), -1, opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0) + 4, opnd_array(0)->index(ra_,this,idx0), (R_G0_enc));
  }
}
uint  regI_to_stkLHiNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  regL_to_stkDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  regL_to_stkDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  stkI_to_regINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lduw_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  stkI_to_regINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  regI_to_stkINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stw_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  regI_to_stkINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  stkL_to_regLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  stkL_to_regLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  regL_to_stkLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  regL_to_stkLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  stkP_to_regPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  stkP_to_regPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  regP_to_stkPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  regP_to_stkPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  Nop_A0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_nop(cbuf);
  }
}
void  Nop_A1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_nop(cbuf);
  }
}
void  Nop_MSNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Generates the instruction LDUXA [o6,g0],#0x82,g0
    unsigned int *code = (unsigned int*)cbuf.code_end();
    *code = (unsigned int)0xc0839040;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  Nop_FANode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Generates the instruction FMOVS f31,f31
    unsigned int *code = (unsigned int*)cbuf.code_end();
    *code = (unsigned int)0xbfa0003f;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  Nop_BRNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Generates the instruction BPN,PN .
    unsigned int *code = (unsigned int*)cbuf.code_end();
    *code = (unsigned int)0x00400000;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  loadBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldsb_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadBNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadUBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldub_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadUBNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadUBLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldub_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadUBLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadUCLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lduh_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadUCLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadCNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lduh_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadCNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lduw_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadL_unalignedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    assert( Assembler::is_simm13(opnd_array(1)->disp(ra_,this,idx1)  ), "need disp and disp+4" );
    assert( Assembler::is_simm13(opnd_array(1)->disp(ra_,this,idx1)+4), "need disp and disp+4" );
    guarantee(opnd_array(1)->index(ra_,this,idx1) == R_G0_enc, "double index?");
    emit_form3_mem_reg(cbuf, this, (Assembler::lduw_op3 /*primary()*/), -1, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1)+4, R_G0_enc, R_O7_enc );
    emit_form3_mem_reg(cbuf, this, (Assembler::lduw_op3 /*primary()*/), -1, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1),   R_G0_enc, opnd_array(0)->reg(ra_,this) );
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::sllx_op3, opnd_array(0)->reg(ra_,this), 0x1020 );
    emit3( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, opnd_array(0)->reg(ra_,this), 0, R_O7_enc );
  }
}
uint  loadL_unalignedNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 16, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 16);
}
void  loadA8BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lddf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadA8BNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadA4CNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lddf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadA4CNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadA4SNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lddf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadA4SNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadA2INode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lddf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadA2INode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadRangeNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lduw_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadRangeNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadI_fregNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadI_fregNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldx_op3 /*primary()*/), (REGP_OP /*tertiary()*/),
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadNNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

     Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
     Register index = as_Register(opnd_array(1)->index(ra_,this,idx1));
     Register dst = opnd_array(0)->as_Register(ra_,this);
     if (index != G0) {
       __ lduw(base, index, dst);
     } else {
       __ lduw(base, opnd_array(1)->disp(ra_,this,idx1), dst);
     }
   }
}
uint  loadNNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadKlassNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldx_op3 /*primary()*/), (REGP_OP /*tertiary()*/),
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadKlassNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadNKlassNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

     Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
     Register index = as_Register(opnd_array(1)->index(ra_,this,idx1));
     Register dst = opnd_array(0)->as_Register(ra_,this);
     if (index != G0) {
       __ lduw(base, index, dst);
     } else {
       __ lduw(base, opnd_array(1)->disp(ra_,this,idx1), dst);
     }
  }
}
uint  loadNKlassNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadSNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldsh_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadSNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lddf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadD_unalignedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    assert( Assembler::is_simm13(opnd_array(1)->disp(ra_,this,idx1)  ), "need disp and disp+4" );
    assert( Assembler::is_simm13(opnd_array(1)->disp(ra_,this,idx1)+4), "need disp and disp+4" );
    guarantee(opnd_array(1)->index(ra_,this,idx1) == R_G0_enc, "double index?");
    // Load long with 2 instructions
    emit_form3_mem_reg(cbuf, this, (Assembler::ldf_op3 /*primary()*/), -1, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1),   R_G0_enc, opnd_array(0)->reg(ra_,this)+0 );
    emit_form3_mem_reg(cbuf, this, (Assembler::ldf_op3 /*primary()*/), -1, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1)+4, R_G0_enc, opnd_array(0)->reg(ra_,this)+1 );
  }
}
uint  loadD_unalignedNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  loadFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadConINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    __ set(opnd_array(1)->constant(), reg_to_register_object(opnd_array(0)->reg(ra_,this)));
  }
}
void  loadConI13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, opnd_array(1)->constant() );
  }
}
uint  loadConI13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadConPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    Register dest = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    // [RGV] This next line should be generated from ADLC
    if ( _opnds[1]->constant_is_oop() ) {
      intptr_t val = opnd_array(1)->constant();
      MacroAssembler _masm(&cbuf);
      __ set_oop_constant((jobject)val, dest);
    } else {          // non-oop pointers, e.g. card mark base, heap top
      emit_ptr(cbuf, opnd_array(1)->constant(), dest, /*ForceRelocatable=*/ false);
    }
  }
}
void  loadConP0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, 0 );
  }
}
uint  loadConP0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadConP_pollNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Address polling_page(reg_to_register_object(opnd_array(0)->reg(ra_,this)), (address)os::get_polling_page());
    __ sethi(polling_page, false );
  }
}
void  loadConN0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, 0 );
  }
}
uint  loadConN0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadConNNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Register dst = opnd_array(0)->as_Register(ra_,this);
    __ set_narrow_oop((jobject)opnd_array(1)->constant(), dst);
  }
}
void  loadConLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    Register dest = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    Register temp = reg_to_register_object(R_O7_enc);
    __ set64( opnd_array(1)->constantL(), dest, temp );
  }
}
void  loadConL0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, opnd_array(1)->constantL() );
  }
}
uint  loadConL0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadConL13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, opnd_array(1)->constantL() );
  }
}
uint  loadConL13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadConFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    address float_address = MacroAssembler(&cbuf).float_constant(opnd_array(1)->constantF());
    RelocationHolder rspec = internal_word_Relocation::spec(float_address);
#ifdef _LP64
    Register   tmp_reg = reg_to_register_object(R_O7_enc);
    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit_ptr(cbuf, (intptr_t)float_address, tmp_reg, /*ForceRelocatable=*/ true);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::ldf_op3, R_O7_enc, 0 );
#else  // _LP64
    uint *code;
    int tmp_reg = R_O7_enc;

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit2_22( cbuf, Assembler::branch_op, tmp_reg, Assembler::sethi_op2, (intptr_t) float_address );

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::ldf_op3, tmp_reg, (intptr_t) float_address );
#endif // _LP64
  }
}
uint  loadConFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 36, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 36);
}
void  loadConDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    address double_address = MacroAssembler(&cbuf).double_constant(opnd_array(1)->constantD());
    RelocationHolder rspec = internal_word_Relocation::spec(double_address);
#ifdef _LP64
    Register   tmp_reg = reg_to_register_object(R_O7_enc);
    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit_ptr(cbuf, (intptr_t)double_address, tmp_reg, /*ForceRelocatable=*/ true);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, R_O7_enc, 0 );
#else // _LP64
    uint *code;
    int tmp_reg = R_O7_enc;

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit2_22( cbuf, Assembler::branch_op, tmp_reg, Assembler::sethi_op2, (intptr_t) double_address );

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, tmp_reg, (intptr_t) double_address );
#endif // _LP64
  }
}
uint  loadConDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 36, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 36);
}
void  prefetchrNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::prefetch_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), 0/*prefetch function many-reads*/);
  }
}
void  prefetchwNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::prefetch_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), 2/*prefetch function many-writes*/);
  }
}
void  storeBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stb_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeBNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeB0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stb_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeB0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeCM0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stb_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeCM0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeCNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::sth_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeCNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeC0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::sth_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeC0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stw_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stw_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeI0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeL0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeL0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeI_FregNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeI_FregNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storePNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), (REGP_OP /*tertiary()*/),
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storePNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeP0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), (REGP_OP /*tertiary()*/),
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeP0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeNNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

     Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
     Register index = as_Register(opnd_array(1)->index(ra_,this,idx1));
     Register src = opnd_array(2)->as_Register(ra_,this,idx2);
     if (index != G0) {
       __ stw(src, base, index);
     } else {
       __ stw(src, base, opnd_array(1)->disp(ra_,this,idx1));
     }
   }
}
uint  storeNNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeN0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

     Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
     Register index = as_Register(opnd_array(1)->index(ra_,this,idx1));
     if (index != G0) {
       __ stw(0, base, index);
     } else {
       __ stw(0, base, opnd_array(1)->disp(ra_,this,idx1));
     }
   }
}
uint  storeN0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stdf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeD0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeD0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeF0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stw_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeF0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeA8BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stdf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeA8BNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  encodeHeapOopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ encode_heap_oop(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(0)->as_Register(ra_,this));
  }
}
void  encodeHeapOop_not_nullNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ encode_heap_oop_not_null(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(0)->as_Register(ra_,this));
  }
}
void  decodeHeapOopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ decode_heap_oop(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(0)->as_Register(ra_,this));
  }
}
void  decodeHeapOop_not_nullNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ decode_heap_oop_not_null(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(0)->as_Register(ra_,this));
  }
}
void  storeA8B0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeA8B0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeA4CNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stdf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeA4CNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeA4C0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeA4C0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeA2INode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stdf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  storeA2INode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storeA2I0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), (R_G0_enc));
  }
}
uint  storeA2I0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  membar_acquireNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    __ membar( Assembler::Membar_mask_bits(Assembler::LoadStore | Assembler::LoadLoad) );
  }
}
uint  membar_acquireNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_acquire_lockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_releaseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    __ membar( Assembler::Membar_mask_bits(Assembler::LoadStore | Assembler::StoreStore) );
  }
}
uint  membar_releaseNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_release_lockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    __ membar( Assembler::Membar_mask_bits(Assembler::StoreLoad) );
  }
}
uint  membar_volatileNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  unnecessary_membar_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  unnecessary_membar_volatileNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  roundDouble_nopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  roundFloat_nopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  castX2PNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Encode a reg-reg copy.  If it is useless, then empty encoding.
    if( opnd_array(1)->reg(ra_,this,idx1) != opnd_array(0)->reg(ra_,this) )
      emit3( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, 0, opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  castP2XNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Encode a reg-reg copy.  If it is useless, then empty encoding.
    if( opnd_array(1)->reg(ra_,this,idx1) != opnd_array(0)->reg(ra_,this) )
      emit3( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, 0, opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  stfSSDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stdf_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  ldfSSDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lddf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
void  stfSSFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stf_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  cmovIP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::ptr_cc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovIP_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::ptr_cc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovII_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::icc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovII_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovII_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::icc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovII_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovII_U_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::icc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovII_U_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovII_U_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::icc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovII_U_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovIF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (0 << 18) |                    // cc2 bit for 'fccX'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (opnd_array(2)->reg(ra_,this,idx2) << 11) |            // cc1, cc0 bits for fcc0-fcc3
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovIF_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovIF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (0 << 18) |                    // cc2 bit for 'fccX'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (opnd_array(2)->reg(ra_,this,idx2) << 11) |            // cc1, cc0 bits for fcc0-fcc3
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovIF_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovNP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::ptr_cc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovNI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::icc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovNI_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovNF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (0 << 18) |                    // cc2 bit for 'fccX'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (opnd_array(2)->reg(ra_,this,idx2) << 11) |            // cc1, cc0 bits for fcc0-fcc3
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovNF_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovPP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::ptr_cc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovPP_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::ptr_cc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovPI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::icc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovPI_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovPI_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::icc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovPI_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovPF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (0 << 18) |                    // cc2 bit for 'fccX'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (opnd_array(2)->reg(ra_,this,idx2) << 11) |            // cc1, cc0 bits for fcc0-fcc3
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovPF_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovPF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (0 << 18) |                    // cc2 bit for 'fccX'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (opnd_array(2)->reg(ra_,this,idx2) << 11) |            // cc1, cc0 bits for fcc0-fcc3
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovPF_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovFP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::fpop2_op3 << 19) |
             (0 << 18) |
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select register move
             (((Assembler::ptr_cc)) << 11) |       // cc1-cc0 bits for 'icc' or 'xcc'
             ((0x101 /*primary()*/) << 5) |              // select single, double or quad
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovFI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::fpop2_op3 << 19) |
             (0 << 18) |
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select register move
             (((Assembler::icc)) << 11) |       // cc1-cc0 bits for 'icc' or 'xcc'
             ((0x101 /*primary()*/) << 5) |              // select single, double or quad
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovFI_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovFF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::fpop2_op3 << 19) |
             (0 << 18) |
             (opnd_array(1)->ccode() << 14) |
             (opnd_array(2)->reg(ra_,this,idx2) << 11) |            // cc2-cc0 bits for 'fccX'
             ((0x1 /*primary()*/) << 5) |              // select single, double or quad
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovFF_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovDP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::fpop2_op3 << 19) |
             (0 << 18) |
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select register move
             (((Assembler::ptr_cc)) << 11) |       // cc1-cc0 bits for 'icc' or 'xcc'
             ((0x102 /*primary()*/) << 5) |              // select single, double or quad
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovDP_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovDI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::fpop2_op3 << 19) |
             (0 << 18) |
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select register move
             (((Assembler::icc)) << 11) |       // cc1-cc0 bits for 'icc' or 'xcc'
             ((0x102 /*primary()*/) << 5) |              // select single, double or quad
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovDI_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovDF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::fpop2_op3 << 19) |
             (0 << 18) |
             (opnd_array(1)->ccode() << 14) |
             (opnd_array(2)->reg(ra_,this,idx2) << 11) |            // cc2-cc0 bits for 'fccX'
             ((0x2 /*primary()*/) << 5) |              // select single, double or quad
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovDF_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovLP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::ptr_cc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovLP_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::ptr_cc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovLI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::icc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovLI_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovLF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (0 << 18) |                    // cc2 bit for 'fccX'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (opnd_array(2)->reg(ra_,this,idx2) << 11) |            // cc1, cc0 bits for fcc0-fcc3
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovLF_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  tlsLoadPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  checkCastPPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  checkCastPPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  castPPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  castIINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  addI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ add(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(2)->as_Register(ra_,this,idx2), opnd_array(0)->as_Register(ra_,this));
  }
}
uint  addI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addI_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::add_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  addI_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addP_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::add_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  addP_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addP_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::add_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  addP_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::add_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  addL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addL_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::add_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  addL_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadPLockedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldx_op3 /*primary()*/), (REGP_OP /*tertiary()*/),
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
void  loadLLockedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  loadLLockedNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  storePConditionalNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register Rmem = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rold = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rnew = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    // casx_under_lock picks 1 of 3 encodings:
    // For 32-bit pointers you get a 32-bit CAS
    // For 64-bit pointers you get a 64-bit CASX
    __ casn(Rmem, Rold, Rnew); // Swap(*Rmem,Rnew) if *Rmem == Rold
    __ cmp( Rold, Rnew );
  }
}
void  storeIConditionalNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register Rmem = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rold = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rnew = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    // casx_under_lock picks 1 of 3 encodings:
    // For 32-bit pointers you get a 32-bit CAS
    // For 64-bit pointers you get a 64-bit CASX
    __ casn(Rmem, Rold, Rnew); // Swap(*Rmem,Rnew) if *Rmem == Rold
    __ cmp( Rold, Rnew );
  }
}
void  storeLConditionalNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register Rmem = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rold = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rnew = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    // casx_under_lock picks 1 of 3 encodings:
    // For 32-bit pointers you get a 32-bit CAS
    // For 64-bit pointers you get a 64-bit CASX
    __ casn(Rmem, Rold, Rnew); // Swap(*Rmem,Rnew) if *Rmem == Rold
    __ cmp( Rold, Rnew );
  }
}
void  compareAndSwapL_boolNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Register Rmem = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rold = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rnew = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    MacroAssembler _masm(&cbuf);
    __ mov(Rnew, O7);
    __ casx(Rmem, Rold, O7);
    __ cmp( Rold, O7 );
  }
  {
    Register Rres = reg_to_register_object(opnd_array(0)->reg(ra_,this));

    MacroAssembler _masm(&cbuf);
    __ mov(1, Rres);
    __ movcc( Assembler::notEqual, false, Assembler::xcc, G0, Rres );
  }
}
void  compareAndSwapI_boolNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Register Rmem = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rold = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rnew = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    MacroAssembler _masm(&cbuf);
    __ mov(Rnew, O7);
    __ cas(Rmem, Rold, O7);
    __ cmp( Rold, O7 );
  }
  {
    Register Rres = reg_to_register_object(opnd_array(0)->reg(ra_,this));

    MacroAssembler _masm(&cbuf);
    __ mov(1, Rres);
    __ movcc( Assembler::notEqual, false, Assembler::icc, G0, Rres );
  }
}
void  compareAndSwapP_boolNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Register Rmem = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rold = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rnew = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    MacroAssembler _masm(&cbuf);
    __ mov(Rnew, O7);
    __ casx(Rmem, Rold, O7);
    __ cmp( Rold, O7 );
  }
  {
    Register Rres = reg_to_register_object(opnd_array(0)->reg(ra_,this));

    MacroAssembler _masm(&cbuf);
    __ mov(1, Rres);
    __ movcc( Assembler::notEqual, false, Assembler::xcc, G0, Rres );
  }
}
void  compareAndSwapN_boolNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Register Rmem = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rold = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rnew = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    MacroAssembler _masm(&cbuf);
    __ mov(Rnew, O7);
    __ cas(Rmem, Rold, O7);
    __ cmp( Rold, O7 );
  }
  {
    Register Rres = reg_to_register_object(opnd_array(0)->reg(ra_,this));

    MacroAssembler _masm(&cbuf);
    __ mov(1, Rres);
    __ movcc( Assembler::notEqual, false, Assembler::icc, G0, Rres );
  }
}
void  subI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sub_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  subI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subI_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sub_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  subI_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subI_zero_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sub_op3 /*primary()*/), (R_G0_enc), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  subI_zero_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sub_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  subL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subL_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sub_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  subL_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  negL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sub_op3 /*primary()*/), (R_G0_enc), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  negL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::mulx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  mulI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulI_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::mulx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  mulI_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::mulx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  mulL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulL_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::mulx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  mulL_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  divI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register Rdividend = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rdivisor = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rresult = reg_to_register_object(opnd_array(0)->reg(ra_,this));

    __ sra(Rdivisor, 0, Rdivisor);
    __ sra(Rdividend, 0, Rdividend);
    __ sdivx(Rdividend, Rdivisor, Rresult);
}
}
void  divI_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

    Register Rdividend = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    int divisor = opnd_array(2)->constant();
    Register Rresult = reg_to_register_object(opnd_array(0)->reg(ra_,this));

    __ sra(Rdividend, 0, Rdividend);
    __ sdivx(Rdividend, divisor, Rresult);
}
}
void  mul_hiNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this));

    __ sra( Rsrc1, 0, Rsrc1 );
    __ sra( Rsrc2, 0, Rsrc2 );
    __ mulx( Rsrc1, Rsrc2, Rdst );
    __ srlx( Rdst, 32, Rdst );
}
}
void  loadConI_x66666667Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    __ set((0x66666667), reg_to_register_object(opnd_array(0)->reg(ra_,this)));
  }
}
uint  loadConI_x66666667Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  sra_31Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::sra_op3, opnd_array(1)->reg(ra_,this,idx1), 31 );
  }
}
void  sra_reg_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sra_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (0x2) );
  }
}
void  divL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sdivx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  divL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  divL_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sdivx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  divL_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  modI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register Rdividend = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rdivisor = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rresult = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    Register Rscratch = reg_to_register_object(R_O7_enc);

    assert(Rdividend != Rscratch, "");
    assert(Rdivisor  != Rscratch, "");

    __ sra(Rdividend, 0, Rdividend);
    __ sra(Rdivisor, 0, Rdivisor);
    __ sdivx(Rdividend, Rdivisor, Rscratch);
    __ mulx(Rscratch, Rdivisor, Rscratch);
    __ sub(Rdividend, Rscratch, Rresult);
}
}
void  modI_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

    Register Rdividend = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    int divisor = opnd_array(2)->constant();
    Register Rresult = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    Register Rscratch = reg_to_register_object(R_O7_enc);

    assert(Rdividend != Rscratch, "");

    __ sra(Rdividend, 0, Rdividend);
    __ sdivx(Rdividend, divisor, Rscratch);
    __ mulx(Rscratch, divisor, Rscratch);
    __ sub(Rdividend, Rscratch, Rresult);
}
}
void  divL_reg_reg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sdivx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  divL_reg_reg_1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  divL_reg_imm13_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sdivx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  divL_reg_imm13_1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulL_reg_reg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::mulx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  mulL_reg_reg_1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulL_reg_imm13_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::mulx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  mulL_reg_imm13_1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subL_reg_reg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sub_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  subL_reg_reg_1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subL_reg_reg_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sub_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  subL_reg_reg_2Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shlI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sll_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  shlI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shlI_reg_imm5Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sll_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() & 0x1F );
  }
}
uint  shlI_reg_imm5Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shlL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sllx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0x80, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  shlL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shlL_reg_imm6Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sllx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (opnd_array(2)->constant() & 0x3F) | 0x1000 );
  }
}
uint  shlL_reg_imm6Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  sarI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sra_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  sarI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  sarI_reg_imm5Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sra_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() & 0x1F );
  }
}
uint  sarI_reg_imm5Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  sarL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srax_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0x80, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  sarL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  sarL_reg_imm6Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srax_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (opnd_array(2)->constant() & 0x3F) | 0x1000 );
  }
}
uint  sarL_reg_imm6Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shrI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srl_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  shrI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shrI_reg_imm5Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srl_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() & 0x1F );
  }
}
uint  shrI_reg_imm5Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shrL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srlx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0x80, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  shrL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shrL_reg_imm6Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srlx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (opnd_array(2)->constant() & 0x3F) | 0x1000 );
  }
}
uint  shrL_reg_imm6Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shrP_reg_imm6Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srlx_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (opnd_array(2)->constant() & 0x3F) | 0x1000 );
  }
}
uint  shrP_reg_imm6Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addF_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fadds_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  addF_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addD_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::faddd_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  addD_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subF_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fsubs_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  subF_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subD_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fsubd_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  subD_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulF_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fmuls_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  mulF_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulD_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fmuld_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  mulD_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  divF_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fdivs_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  divF_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  divD_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fdivd_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  divD_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  absD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    FloatRegister Fdst = reg_to_DoubleFloatRegister_object(opnd_array(0)->reg(ra_,this));
    FloatRegister Fsrc = reg_to_DoubleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1));

    __ fabs(FloatRegisterImpl::D, Fsrc, Fdst);
}
}
void  absF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    FloatRegister Fdst = reg_to_SingleFloatRegister_object(opnd_array(0)->reg(ra_,this));
    FloatRegister Fsrc = reg_to_SingleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1));

    __ fabs(FloatRegisterImpl::S, Fsrc, Fdst);
}
}
void  negF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fnegs_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
}
uint  negF_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  negD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    FloatRegister Fdst = reg_to_DoubleFloatRegister_object(opnd_array(0)->reg(ra_,this));
    FloatRegister Fsrc = reg_to_DoubleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1));

    __ fneg(FloatRegisterImpl::D, Fsrc, Fdst);
}
}
void  sqrtF_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    FloatRegister Fdst = reg_to_SingleFloatRegister_object(opnd_array(0)->reg(ra_,this));
    FloatRegister Fsrc = reg_to_SingleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1));

    __ fsqrt(FloatRegisterImpl::S, Fsrc, Fdst);
}
}
uint  sqrtF_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  sqrtD_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    FloatRegister Fdst = reg_to_DoubleFloatRegister_object(opnd_array(0)->reg(ra_,this));
    FloatRegister Fsrc = reg_to_DoubleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1));

    __ fsqrt(FloatRegisterImpl::D, Fsrc, Fdst);
}
}
uint  sqrtD_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  andI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::and_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  andI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  andI_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::and_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  andI_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  andL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::and_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  andL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  andL_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::and_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  andL_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  orI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::or_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  orI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  orI_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::or_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  orI_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  orL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::or_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  orL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  orL_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::or_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  orL_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  orL_reg_castP2XNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::or_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  orL_reg_castP2XNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  orL_reg_castP2X_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::or_op3 /*primary()*/), opnd_array(2)->reg(ra_,this,idx2), 0, opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  orL_reg_castP2X_0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  xorI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::xor_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  xorI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  xorI_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::xor_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  xorI_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  xorL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::xor_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  xorL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  xorL_reg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::xor_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  xorL_reg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  convI2BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3       ( cbuf, Assembler::arith_op,         0, Assembler::subcc_op3, 0, 0, opnd_array(1)->reg(ra_,this,idx1) );
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::addc_op3 , 0, 0 );
  }
}
void  convP2BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Encode a reg-reg copy.  If it is useless, then empty encoding.
    if( opnd_array(1)->reg(ra_,this,idx1) != opnd_array(0)->reg(ra_,this) )
      emit3( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, 0, opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    MacroAssembler _masm(&cbuf);

    Register   src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register   dst_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    __ movr(Assembler::rc_nz, src_reg, 1, dst_reg);
  }
}
void  cmpLTMask_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3       ( cbuf, Assembler::arith_op,         0, Assembler::subcc_op3, opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
    // clear if nothing else is happening
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0,  0 );
    // blt,a,pn done
    emit2_19    ( cbuf, Assembler::branch_op, 1/*annul*/, Assembler::less, Assembler::bp_op2, Assembler::icc, 0/*predict not taken*/, 2 );
    // mov dst,-1 in delay slot
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, -1 );
  }
}
void  cadd_cmpLTMaskNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // (Set p (AddI (AndI (CmpLTMask p q) y) (SubI p q)))
    MacroAssembler _masm(&cbuf);

    Register   p_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register   q_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   y_reg = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register tmp_reg = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4));

    __ subcc( p_reg, q_reg,   p_reg );
    __ add  ( p_reg, y_reg, tmp_reg );
    __ movcc( Assembler::less, false, Assembler::icc, tmp_reg, p_reg );
  }
}
void  cadd_cmpLTMask_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // (Set p (AddI (AndI (CmpLTMask p q) y) (SubI p q)))
    MacroAssembler _masm(&cbuf);

    Register   p_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register   q_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   y_reg = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register tmp_reg = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4));

    __ subcc( p_reg, q_reg,   p_reg );
    __ add  ( p_reg, y_reg, tmp_reg );
    __ movcc( Assembler::less, false, Assembler::icc, tmp_reg, p_reg );
  }
}
void  cadd_cmpLTMask_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // (Set p (AddI (AndI (CmpLTMask p q) y) (SubI p q)))
    MacroAssembler _masm(&cbuf);

    Register   p_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   q_reg = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register   y_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register tmp_reg = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4));

    __ subcc( p_reg, q_reg,   p_reg );
    __ add  ( p_reg, y_reg, tmp_reg );
    __ movcc( Assembler::less, false, Assembler::icc, tmp_reg, p_reg );
  }
}
void  cadd_cmpLTMask_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // (Set p (AddI (AndI (CmpLTMask p q) y) (SubI p q)))
    MacroAssembler _masm(&cbuf);

    Register   p_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register   q_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   y_reg = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register tmp_reg = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4));

    __ subcc( p_reg, q_reg,   p_reg );
    __ add  ( p_reg, y_reg, tmp_reg );
    __ movcc( Assembler::less, false, Assembler::icc, tmp_reg, p_reg );
  }
}
void  cadd_cmpLTMask2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // (Set p (AddI (AndI (CmpLTMask p q) y) (SubI p q)))
    MacroAssembler _masm(&cbuf);

    Register   p_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register   q_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   y_reg = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register tmp_reg = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4));

    __ subcc( p_reg, q_reg,   p_reg );
    __ add  ( p_reg, y_reg, tmp_reg );
    __ movcc( Assembler::less, false, Assembler::icc, tmp_reg, p_reg );
  }
}
void  cadd_cmpLTMask2_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // (Set p (AddI (AndI (CmpLTMask p q) y) (SubI p q)))
    MacroAssembler _masm(&cbuf);

    Register   p_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register   q_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   y_reg = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register tmp_reg = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4));

    __ subcc( p_reg, q_reg,   p_reg );
    __ add  ( p_reg, y_reg, tmp_reg );
    __ movcc( Assembler::less, false, Assembler::icc, tmp_reg, p_reg );
  }
}
void  cadd_cmpLTMask2_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // (Set p (AddI (AndI (CmpLTMask p q) y) (SubI p q)))
    MacroAssembler _masm(&cbuf);

    Register   p_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register   q_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   y_reg = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register tmp_reg = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4));

    __ subcc( p_reg, q_reg,   p_reg );
    __ add  ( p_reg, y_reg, tmp_reg );
    __ movcc( Assembler::less, false, Assembler::icc, tmp_reg, p_reg );
  }
}
void  cadd_cmpLTMask2_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // (Set p (AddI (AndI (CmpLTMask p q) y) (SubI p q)))
    MacroAssembler _masm(&cbuf);

    Register   p_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   q_reg = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register   y_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register tmp_reg = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4));

    __ subcc( p_reg, q_reg,   p_reg );
    __ add  ( p_reg, y_reg, tmp_reg );
    __ movcc( Assembler::less, false, Assembler::icc, tmp_reg, p_reg );
  }
}
void  convD2F_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fdtos_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
}
uint  convD2F_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  convD2I_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // fcmp %fcc0,$src,$src
    emit3( cbuf, Assembler::arith_op , Assembler::fcc0, Assembler::fpop2_op3, opnd_array(1)->reg(ra_,this,idx1), Assembler::fcmpd_opf, opnd_array(1)->reg(ra_,this,idx1) );
    // branch %fcc0 not-nan, predict taken
    emit2_19( cbuf, Assembler::branch_op, 0/*annul*/, Assembler::f_ordered, Assembler::fbp_op2, Assembler::fcc0, 1/*predict taken*/, 4 );
    // fdtoi $src,$dst
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3,         0, Assembler::fdtoi_opf, opnd_array(1)->reg(ra_,this,idx1) );
    // fitos $dst,$dst (if nan)
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3,         0, Assembler::fitos_opf, opnd_array(0)->reg(ra_,this) );
    // clear $dst (if nan)
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3, opnd_array(0)->reg(ra_,this), Assembler::fsubs_opf, opnd_array(0)->reg(ra_,this) );
    // carry on here...
  }
}
void  convD2L_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // fcmp %fcc0,$src,$src  check for NAN
    emit3( cbuf, Assembler::arith_op , Assembler::fcc0, Assembler::fpop2_op3, opnd_array(1)->reg(ra_,this,idx1), Assembler::fcmpd_opf, opnd_array(1)->reg(ra_,this,idx1) );
    // branch %fcc0 not-nan, predict taken
    emit2_19( cbuf, Assembler::branch_op, 0/*annul*/, Assembler::f_ordered, Assembler::fbp_op2, Assembler::fcc0, 1/*predict taken*/, 4 );
    // fdtox $src,$dst   convert in delay slot
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3,         0, Assembler::fdtox_opf, opnd_array(1)->reg(ra_,this,idx1) );
    // fxtod $dst,$dst  (if nan)
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3,         0, Assembler::fxtod_opf, opnd_array(0)->reg(ra_,this) );
    // clear $dst (if nan)
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3, opnd_array(0)->reg(ra_,this), Assembler::fsubd_opf, opnd_array(0)->reg(ra_,this) );
    // carry on here...
  }
}
void  convF2D_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fstod_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
}
void  convF2I_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // fcmps %fcc0,$src,$src
    emit3( cbuf, Assembler::arith_op , Assembler::fcc0, Assembler::fpop2_op3, opnd_array(1)->reg(ra_,this,idx1), Assembler::fcmps_opf, opnd_array(1)->reg(ra_,this,idx1) );
    // branch %fcc0 not-nan, predict taken
    emit2_19( cbuf, Assembler::branch_op, 0/*annul*/, Assembler::f_ordered, Assembler::fbp_op2, Assembler::fcc0, 1/*predict taken*/, 4 );
    // fstoi $src,$dst
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3,         0, Assembler::fstoi_opf, opnd_array(1)->reg(ra_,this,idx1) );
    // fitos $dst,$dst (if nan)
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3,         0, Assembler::fitos_opf, opnd_array(0)->reg(ra_,this) );
    // clear $dst (if nan)
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3, opnd_array(0)->reg(ra_,this), Assembler::fsubs_opf, opnd_array(0)->reg(ra_,this) );
    // carry on here...
  }
}
void  convF2L_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // fcmps %fcc0,$src,$src
    emit3( cbuf, Assembler::arith_op , Assembler::fcc0, Assembler::fpop2_op3, opnd_array(1)->reg(ra_,this,idx1), Assembler::fcmps_opf, opnd_array(1)->reg(ra_,this,idx1) );
    // branch %fcc0 not-nan, predict taken
    emit2_19( cbuf, Assembler::branch_op, 0/*annul*/, Assembler::f_ordered, Assembler::fbp_op2, Assembler::fcc0, 1/*predict taken*/, 4 );
    // fstox $src,$dst
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3,         0, Assembler::fstox_opf, opnd_array(1)->reg(ra_,this,idx1) );
    // fxtod $dst,$dst (if nan)
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3,         0, Assembler::fxtod_opf, opnd_array(0)->reg(ra_,this) );
    // clear $dst (if nan)
    emit3( cbuf, Assembler::arith_op , opnd_array(0)->reg(ra_,this), Assembler::fpop1_op3, opnd_array(0)->reg(ra_,this), Assembler::fsubd_opf, opnd_array(0)->reg(ra_,this) );
    // carry on here...
  }
}
void  convI2D_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fitod_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
}
void  convI2D_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
  {
    emit3(cbuf,Assembler::arith_op,opnd_array(0)->reg(ra_,this),Assembler::fpop1_op3,0,(Assembler::fitod_opf /*secondary()*/),opnd_array(0)->reg(ra_,this));
  }
}
uint  convI2D_memNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  convI2F_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fitos_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
}
void  convI2F_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
  {
    emit3(cbuf,Assembler::arith_op,opnd_array(0)->reg(ra_,this),Assembler::fpop1_op3,0,(Assembler::fitos_opf /*secondary()*/),opnd_array(0)->reg(ra_,this));
  }
}
uint  convI2F_memNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  convI2L_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::sra_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, (R_G0_enc) );
  }
}
uint  convI2L_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  convI2L_reg_zexNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srl_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, (R_G0_enc) );
  }
}
uint  convI2L_reg_zexNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  zerox_longNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srl_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, (R_G0_enc) );
  }
}
uint  zerox_longNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  MoveF2I_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lduw_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  MoveF2I_stack_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  MoveI2F_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  MoveI2F_stack_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  MoveD2L_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::ldx_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  MoveD2L_stack_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  MoveL2D_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::lddf_op3 /*primary()*/), -1,
                       opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
uint  MoveL2D_stack_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  MoveF2I_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stf_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  MoveF2I_reg_stackNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  MoveI2F_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stw_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  MoveI2F_reg_stackNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  MoveD2L_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stdf_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  MoveD2L_reg_stackNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  MoveL2D_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg(cbuf, this, (Assembler::stx_op3 /*primary()*/), -1,
                       opnd_array(0)->base(ra_,this,idx0), opnd_array(0)->disp(ra_,this,0), opnd_array(0)->index(ra_,this,idx0), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  MoveL2D_reg_stackNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadConI_x43300000Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit2_22( cbuf, Assembler::branch_op, opnd_array(0)->reg(ra_,this), Assembler::sethi_op2, (0x43300000) );
  }
}
uint  loadConI_x43300000Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  loadConI_x41f00000Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit2_22( cbuf, Assembler::branch_op, opnd_array(0)->reg(ra_,this), Assembler::sethi_op2, (0x41f00000) );
  }
}
uint  loadConI_x41f00000Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  regDHi_regDLo_to_regDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fmovs_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this)+1,(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fmovs_opf /*tertiary()*/),opnd_array(2)->reg(ra_,this,idx2)+1); }
}
uint  regDHi_regDLo_to_regDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  convI2D_regDHi_regDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fitod_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
}
uint  convI2D_regDHi_regDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addD_regD_regDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::faddd_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  addD_regD_regDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  subD_regD_regDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fsubd_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  subD_regD_regDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  mulD_regD_regDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop1_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fmuld_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  mulD_regD_regDNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  convL2D_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fxtod_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
}
uint  convL2D_helperNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  convL2F_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {emit3(cbuf,(Assembler::arith_op /*secondary()*/),opnd_array(0)->reg(ra_,this),(Assembler::fpop1_op3 /*primary()*/),0,(Assembler::fxtos_opf /*tertiary()*/),opnd_array(1)->reg(ra_,this,idx1)); }
}
uint  convL2F_helperNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  convL2I_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Sign extend low half
    emit3( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::sra_op3, opnd_array(1)->reg(ra_,this,idx1), 0, 0 );
  }
}
uint  convL2I_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  shrL_reg_imm6_L2INode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::srax_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (opnd_array(2)->constant() & 0x3F) | 0x1000 );
  }
}
uint  shrL_reg_imm6_L2INode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  Repl8B_reg_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register dst_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    __ sllx(src_reg, 56, dst_reg);
    __ srlx(dst_reg,  8, O7);
    __ or3 (dst_reg, O7, dst_reg);
    __ srlx(dst_reg, 16, O7);
    __ or3 (dst_reg, O7, dst_reg);
    __ srlx(dst_reg, 32, O7);
    __ or3 (dst_reg, O7, dst_reg);
  }
}
void  Repl8B_immINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Load a constant replicated "count" times with width "width"
    int bit_width = ((1)) * 8;
    jlong elt_val = opnd_array(1)->constant();
    elt_val  &= (((jlong)1) << bit_width) - 1; // mask off sign bits
    jlong val = elt_val;
    for (int i = 0; i < ((8)) - 1; i++) {
        val <<= bit_width;
        val |= elt_val;
    }
    jdouble dval = *(jdouble*)&val; // coerce to double type
    address double_address = MacroAssembler(&cbuf).double_constant(dval);
    RelocationHolder rspec = internal_word_Relocation::spec(double_address);
#ifdef _LP64
    Register   tmp_reg = reg_to_register_object(R_O7_enc);
    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit_ptr(cbuf, (intptr_t)double_address, tmp_reg, /*ForceRelocatable=*/ true);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, R_O7_enc, 0 );
#else // _LP64
    uint *code;
    int tmp_reg = R_O7_enc;

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit2_22( cbuf, Assembler::branch_op, tmp_reg, Assembler::sethi_op2, (intptr_t) double_address );

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, tmp_reg, (intptr_t) double_address );
#endif // _LP64
  }
}
uint  Repl8B_immINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 36, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 36);
}
void  Repl4C_reg_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register dst_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    __ sllx(src_reg, 48, dst_reg);
    __ srlx(dst_reg, 16, O7);
    __ or3 (dst_reg, O7, dst_reg);
    __ srlx(dst_reg, 32, O7);
    __ or3 (dst_reg, O7, dst_reg);
  }
}
void  Repl4C_immINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Load a constant replicated "count" times with width "width"
    int bit_width = ((2)) * 8;
    jlong elt_val = opnd_array(1)->constant();
    elt_val  &= (((jlong)1) << bit_width) - 1; // mask off sign bits
    jlong val = elt_val;
    for (int i = 0; i < ((4)) - 1; i++) {
        val <<= bit_width;
        val |= elt_val;
    }
    jdouble dval = *(jdouble*)&val; // coerce to double type
    address double_address = MacroAssembler(&cbuf).double_constant(dval);
    RelocationHolder rspec = internal_word_Relocation::spec(double_address);
#ifdef _LP64
    Register   tmp_reg = reg_to_register_object(R_O7_enc);
    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit_ptr(cbuf, (intptr_t)double_address, tmp_reg, /*ForceRelocatable=*/ true);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, R_O7_enc, 0 );
#else // _LP64
    uint *code;
    int tmp_reg = R_O7_enc;

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit2_22( cbuf, Assembler::branch_op, tmp_reg, Assembler::sethi_op2, (intptr_t) double_address );

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, tmp_reg, (intptr_t) double_address );
#endif // _LP64
  }
}
uint  Repl4C_immINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 36, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 36);
}
void  Repl4S_reg_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register dst_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    __ sllx(src_reg, 48, dst_reg);
    __ srlx(dst_reg, 16, O7);
    __ or3 (dst_reg, O7, dst_reg);
    __ srlx(dst_reg, 32, O7);
    __ or3 (dst_reg, O7, dst_reg);
  }
}
void  Repl4S_immINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Load a constant replicated "count" times with width "width"
    int bit_width = ((2)) * 8;
    jlong elt_val = opnd_array(1)->constant();
    elt_val  &= (((jlong)1) << bit_width) - 1; // mask off sign bits
    jlong val = elt_val;
    for (int i = 0; i < ((4)) - 1; i++) {
        val <<= bit_width;
        val |= elt_val;
    }
    jdouble dval = *(jdouble*)&val; // coerce to double type
    address double_address = MacroAssembler(&cbuf).double_constant(dval);
    RelocationHolder rspec = internal_word_Relocation::spec(double_address);
#ifdef _LP64
    Register   tmp_reg = reg_to_register_object(R_O7_enc);
    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit_ptr(cbuf, (intptr_t)double_address, tmp_reg, /*ForceRelocatable=*/ true);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, R_O7_enc, 0 );
#else // _LP64
    uint *code;
    int tmp_reg = R_O7_enc;

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit2_22( cbuf, Assembler::branch_op, tmp_reg, Assembler::sethi_op2, (intptr_t) double_address );

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, tmp_reg, (intptr_t) double_address );
#endif // _LP64
  }
}
uint  Repl4S_immINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 36, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 36);
}
void  Repl2I_reg_helperNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register dst_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    __ sllx(src_reg, 32, dst_reg);
    __ srlx(dst_reg, 32, O7);
    __ or3 (dst_reg, O7, dst_reg);
  }
}
void  Repl2I_immINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Load a constant replicated "count" times with width "width"
    int bit_width = ((4)) * 8;
    jlong elt_val = opnd_array(1)->constant();
    elt_val  &= (((jlong)1) << bit_width) - 1; // mask off sign bits
    jlong val = elt_val;
    for (int i = 0; i < ((2)) - 1; i++) {
        val <<= bit_width;
        val |= elt_val;
    }
    jdouble dval = *(jdouble*)&val; // coerce to double type
    address double_address = MacroAssembler(&cbuf).double_constant(dval);
    RelocationHolder rspec = internal_word_Relocation::spec(double_address);
#ifdef _LP64
    Register   tmp_reg = reg_to_register_object(R_O7_enc);
    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit_ptr(cbuf, (intptr_t)double_address, tmp_reg, /*ForceRelocatable=*/ true);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, R_O7_enc, 0 );
#else // _LP64
    uint *code;
    int tmp_reg = R_O7_enc;

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit2_22( cbuf, Assembler::branch_op, tmp_reg, Assembler::sethi_op2, (intptr_t) double_address );

    cbuf.relocate(cbuf.code_end(), rspec, 0);
    emit3_simm10( cbuf, Assembler::ldst_op, opnd_array(0)->reg(ra_,this), Assembler::lddf_op3, tmp_reg, (intptr_t) double_address );
#endif // _LP64
  }
}
uint  Repl2I_immINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 36, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 36);
}
void  compI_iRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  compI_iRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compU_iRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  compU_iRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compI_iReg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  compI_iReg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  testI_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::andcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  testI_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  testI_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::andcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  testI_reg_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  compL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compL_reg_conNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  compL_reg_conNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  testL_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::andcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  testL_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  testL_reg_conNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::andcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constantL() );
  }
}
uint  testL_reg_conNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compU_iReg_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  compU_iReg_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compP_iRegPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  compP_iRegPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compP_iRegP_imm13Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  compP_iRegP_imm13Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compN_iRegNNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  compN_iRegNNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  compN_iRegN_immN0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3_simm13( cbuf, (Assembler::arith_op /*secondary()*/), (R_G0_enc), (Assembler::subcc_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->constant() );
  }
}
uint  compN_iRegN_immN0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovI_reg_ltNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(1)->reg(ra_,this,idx1) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             ((Assembler::less /*primary()*/) << 14) |
             (0 << 13) |                    // select register move
             (0 << 11) |                    // cc1, cc0 bits for 'icc'
             (opnd_array(2)->reg(ra_,this,idx2) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
uint  cmovI_reg_ltNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmovI_reg_gtNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(1)->reg(ra_,this,idx1) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             ((Assembler::greater /*primary()*/) << 14) |
             (0 << 13) |                    // select register move
             (0 << 11) |                    // cc1, cc0 bits for 'icc'
             (opnd_array(2)->reg(ra_,this,idx2) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmpF_ccNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop2_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fcmps_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  cmpF_ccNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmpD_ccNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit3( cbuf, (Assembler::arith_op /*secondary()*/), opnd_array(0)->reg(ra_,this), (Assembler::fpop2_op3 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), (Assembler::fcmpd_opf /*tertiary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  cmpD_ccNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmpF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    FloatRegister Fsrc1 = (true /*primary()*/) ? reg_to_SingleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1))
                                     : reg_to_DoubleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1));
    FloatRegister Fsrc2 = (true /*primary()*/) ? reg_to_SingleFloatRegister_object(opnd_array(2)->reg(ra_,this,idx2))
                                     : reg_to_DoubleFloatRegister_object(opnd_array(2)->reg(ra_,this,idx2));

    // Convert condition code fcc0 into -1,0,1; unordered reports less-than (-1)
    __ float_cmp( (true /*primary()*/), -1, Fsrc1, Fsrc2, Rdst);
  }
}
void  cmpD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this));
    FloatRegister Fsrc1 = (false /*primary()*/) ? reg_to_SingleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1))
                                     : reg_to_DoubleFloatRegister_object(opnd_array(1)->reg(ra_,this,idx1));
    FloatRegister Fsrc2 = (false /*primary()*/) ? reg_to_SingleFloatRegister_object(opnd_array(2)->reg(ra_,this,idx2))
                                     : reg_to_DoubleFloatRegister_object(opnd_array(2)->reg(ra_,this,idx2));

    // Convert condition code fcc0 into -1,0,1; unordered reports less-than (-1)
    __ float_cmp( (false /*primary()*/), -1, Fsrc1, Fsrc2, Rdst);
  }
}
void  jumpXtndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    Register switch_reg       = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register table_reg        = O7;

    address table_base = __ address_table_constant(_index2label);
    RelocationHolder rspec = internal_word_Relocation::spec(table_base);

    // Load table address
    Address the_pc(table_reg, table_base, rspec);
    __ load_address(the_pc);

    // Jump to base address + switch value
    __ ld_ptr(table_reg, switch_reg, table_reg);
    __ jmp(table_reg, G0);
    __ delayed()->nop();

  }
}
void  branchNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *(opnd_array(1)->label());
    __ ba(false, L);
    __ delayed()->nop();
  }
}
uint  branchNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchConNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *(opnd_array(3)->label());
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bp( (Assembler::Condition)(opnd_array(1)->ccode()), false, Assembler::icc, predict_taken, L);
    __ delayed()->nop();
  }
}
uint  branchConNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchCon_regINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *opnd_array(4)->label();
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bpr( (Assembler::RCondition)(opnd_array(1)->ccode()), false, predict_taken, as_Register(opnd_array(2)->reg(ra_,this,idx2)), L);
    __ delayed()->nop();
  }
}
uint  branchCon_regINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchCon_regPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *opnd_array(4)->label();
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bpr( (Assembler::RCondition)(opnd_array(1)->ccode()), false, predict_taken, as_Register(opnd_array(2)->reg(ra_,this,idx2)), L);
    __ delayed()->nop();
  }
}
uint  branchCon_regPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchCon_regLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *opnd_array(4)->label();
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bpr( (Assembler::RCondition)(opnd_array(1)->ccode()), false, predict_taken, as_Register(opnd_array(2)->reg(ra_,this,idx2)), L);
    __ delayed()->nop();
  }
}
uint  branchCon_regLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchConUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *(opnd_array(3)->label());
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bp( (Assembler::Condition)(opnd_array(1)->ccode()), false, Assembler::icc, predict_taken, L);
    __ delayed()->nop();
  }
}
void  branchConPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *(opnd_array(3)->label());
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bp( (Assembler::Condition)(opnd_array(1)->ccode()), false, Assembler::ptr_cc, predict_taken, L);
    __ delayed()->nop();
  }
}
uint  branchConPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchConFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *(opnd_array(3)->label());
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ fbp( (Assembler::Condition)(opnd_array(1)->ccode()), false, (Assembler::CC)(opnd_array(2)->reg(ra_,this,idx2)), predict_taken, L);
    __ delayed()->nop();
  }
}
uint  branchConFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchLoopEndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *(opnd_array(3)->label());
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bp( (Assembler::Condition)(opnd_array(1)->ccode()), false, Assembler::icc, predict_taken, L);
    __ delayed()->nop();
  }
}
uint  branchLoopEndNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchLoopEndUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *(opnd_array(3)->label());
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bp( (Assembler::Condition)(opnd_array(1)->ccode()), false, Assembler::icc, predict_taken, L);
    __ delayed()->nop();
  }
}
uint  branchLoopEndUNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  branchCon_longNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Label &L = *(opnd_array(3)->label());
    Assembler::Predict predict_taken =
      cbuf.is_backward_branch(L) ? Assembler::pt : Assembler::pn;

    __ bp( (Assembler::Condition)(opnd_array(1)->ccode()), false, Assembler::xcc, predict_taken, L);
    __ delayed()->nop();
  }
}
uint  branchCon_longNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  cmpL3_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // CMP $src1,$src2
    emit3( cbuf, Assembler::arith_op, 0, Assembler::subcc_op3, opnd_array(1)->reg(ra_,this,idx1), 0, opnd_array(2)->reg(ra_,this,idx2) );
    // blt,a,pn done
    emit2_19( cbuf, Assembler::branch_op, 1/*annul*/, Assembler::less   , Assembler::bp_op2, Assembler::xcc, 0/*predict not taken*/, 5 );
    // mov dst,-1 in delay slot
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0, -1 );
    // bgt,a,pn done
    emit2_19( cbuf, Assembler::branch_op, 1/*annul*/, Assembler::greater, Assembler::bp_op2, Assembler::xcc, 0/*predict not taken*/, 3 );
    // mov dst,1 in delay slot
    emit3_simm13( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3, 0,  1 );
    // CLR    $dst
    emit3( cbuf, Assembler::arith_op, opnd_array(0)->reg(ra_,this), Assembler::or_op3 , 0, 0, 0 );
  }
}
uint  cmpL3_reg_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 24, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 24);
}
void  cmovLL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::xcc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovLL_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constantL() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::xcc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovIL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::xcc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovIL_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::xcc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovNL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::xcc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovPL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (0 << 13) |                    // select register move
             (((Assembler::xcc)) << 11) |       // cc1, cc0 bits for 'icc' or 'xcc'
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovPL_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int simm11 = opnd_array(4)->constant() & ((1<<11)-1); // Mask to 11 bits
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::movcc_op3 << 19) |
             (1 << 18) |                    // cc2 bit for 'icc'
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select immediate move
             (((Assembler::xcc)) << 11) |       // cc1, cc0 bits for 'icc'
             (simm11 << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovFL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::fpop2_op3 << 19) |
             (0 << 18) |
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select register move
             (((Assembler::xcc)) << 11) |       // cc1-cc0 bits for 'icc' or 'xcc'
             ((0x101 /*primary()*/) << 5) |              // select single, double or quad
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  cmovDL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = (Assembler::arith_op << 30) |
             (opnd_array(3)->reg(ra_,this,idx3) << 25) |
             (Assembler::fpop2_op3 << 19) |
             (0 << 18) |
             (opnd_array(1)->ccode() << 14) |
             (1 << 13) |                    // select register move
             (((Assembler::xcc)) << 11) |       // cc1-cc0 bits for 'icc' or 'xcc'
             ((0x102 /*primary()*/) << 5) |              // select single, double or quad
             (opnd_array(4)->reg(ra_,this,idx4) << 0);
    *((int*)(cbuf.code_end())) = op;
    cbuf.set_code_end(cbuf.code_end() + BytesPerInstWord);
  }
}
void  safePoint_pollNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ relocate(relocInfo::poll_type);
    __ ld_ptr(opnd_array(1)->as_Register(ra_,this,idx1), 0, G0);
  }
}
uint  safePoint_pollNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  CallStaticJavaDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    if ( !_method ) {
      emit_call_reloc(cbuf, opnd_array(1)->method(), relocInfo::runtime_call_type);
    } else if (_optimized_virtual) {
      emit_call_reloc(cbuf, opnd_array(1)->method(), relocInfo::opt_virtual_call_type);
    } else {
      emit_call_reloc(cbuf, opnd_array(1)->method(), relocInfo::static_call_type);
    }
    if( _method ) {  // Emit stub for static call
      emit_java_to_interp(cbuf);
    }
  }
  {
    if( VerifyStackAtCalls ) {
      MacroAssembler _masm(&cbuf);
      int framesize = ra_->C->frame_slots() << LogBytesPerInt;
      Register temp_reg = G3;
      __ add(SP, framesize, temp_reg);
      __ cmp(temp_reg, FP);
      __ breakpoint_trap(Assembler::notEqual, Assembler::ptr_cc);
    }
  }
}
uint  CallStaticJavaDirectNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  CallDynamicJavaDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);
    __ set_inst_mark();
    int vtable_index = this->_vtable_index;
    // MachCallDynamicJavaNode::ret_addr_offset uses this same test
    if (vtable_index < 0) {
      // must be invalid_vtable_index, not nonvirtual_vtable_index
      assert(vtable_index == methodOopDesc::invalid_vtable_index, "correct sentinel value");
      Register G5_ic_reg = reg_to_register_object(Matcher::inline_cache_reg_encode());
      assert(G5_ic_reg == G5_inline_cache_reg, "G5_inline_cache_reg used in assemble_ic_buffer_code()");
      assert(G5_ic_reg == G5_megamorphic_method, "G5_megamorphic_method used in megamorphic call stub");
      // !!!!!
      // Generate  "set 0x01, R_G5", placeholder instruction to load oop-info
      // emit_call_dynamic_prologue( cbuf );
      __ set_oop((jobject)Universe::non_oop_word(), G5_ic_reg);

      address  virtual_call_oop_addr = __ inst_mark();
      // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
      // who we intended to call.
      __ relocate(virtual_call_Relocation::spec(virtual_call_oop_addr));
      emit_call_reloc(cbuf, opnd_array(1)->method(), relocInfo::none);
    } else {
      assert(!UseInlineCaches, "expect vtable calls only if not using ICs");
      // Just go thru the vtable
      // get receiver klass (receiver already checked for non-null)
      // If we end up going thru a c2i adapter interpreter expects method in G5
      int off = __ offset();
      __ load_klass(O0, G3_scratch);
      int klass_load_size;
      if (UseCompressedOops) {
        klass_load_size = 3*BytesPerInstWord;
      } else {
        klass_load_size = 1*BytesPerInstWord;
      }
      int entry_offset = instanceKlass::vtable_start_offset() + vtable_index*vtableEntry::size();
      int v_off = entry_offset*wordSize + vtableEntry::method_offset_in_bytes();
      if( __ is_simm13(v_off) ) {
        __ ld_ptr(G3, v_off, G5_method);
      } else {
        // Generate 2 instructions
        __ Assembler::sethi(v_off & ~0x3ff, G5_method);
        __ or3(G5_method, v_off & 0x3ff, G5_method);
        // ld_ptr, set_hi, set
        assert(__ offset() - off == klass_load_size + 2*BytesPerInstWord,
               "Unexpected instruction size(s)");
        __ ld_ptr(G3, G5_method, G5_method);
      }
      // NOTE: for vtable dispatches, the vtable entry will never be null.
      // However it may very well end up in handle_wrong_method if the
      // method is abstract for the particular class.
      __ ld_ptr(G5_method, in_bytes(methodOopDesc::from_compiled_offset()), G3_scratch);
      // jump to target (either compiled code or c2iadapter)
      __ jmpl(G3_scratch, G0, O7);
      __ delayed()->nop();
    }
  }
  {
    if( VerifyStackAtCalls ) {
      MacroAssembler _masm(&cbuf);
      int framesize = ra_->C->frame_slots() << LogBytesPerInt;
      Register temp_reg = G3;
      __ add(SP, framesize, temp_reg);
      __ cmp(temp_reg, FP);
      __ breakpoint_trap(Assembler::notEqual, Assembler::ptr_cc);
    }
  }
}
void  CallRuntimeDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // CALL directly to the runtime
    // The user of this is responsible for ensuring that R_L7 is empty (killed).
    emit_call_reloc(cbuf, opnd_array(1)->method(), relocInfo::runtime_call_type,
                    /*preserve_g2=*/true, /*force far call*/true);
  }
  {
    if( VerifyStackAtCalls ) {
      MacroAssembler _masm(&cbuf);
      int framesize = ra_->C->frame_slots() << LogBytesPerInt;
      Register temp_reg = G3;
      __ add(SP, framesize, temp_reg);
      __ cmp(temp_reg, FP);
      __ breakpoint_trap(Assembler::notEqual, Assembler::ptr_cc);
    }
  }
  {
#ifndef _LP64
    if (returns_long()) {
      //    sllx  O0,32,O0
      emit3_simm13( cbuf, Assembler::arith_op, R_O0_enc, Assembler::sllx_op3, R_O0_enc, 0x1020 );
      //    srl   O1,0,O1
      emit3_simm13( cbuf, Assembler::arith_op, R_O1_enc, Assembler::srl_op3, R_O1_enc, 0x0000 );
      //    or    O0,O1,G1
      emit3       ( cbuf, Assembler::arith_op, R_G1_enc, Assembler:: or_op3, R_O0_enc, 0, R_O1_enc );
    }
#endif
  }
}
void  CallLeafDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // CALL directly to the runtime
    // The user of this is responsible for ensuring that R_L7 is empty (killed).
    emit_call_reloc(cbuf, opnd_array(1)->method(), relocInfo::runtime_call_type,
                    /*preserve_g2=*/true, /*force far call*/true);
  }
  {
    if( VerifyStackAtCalls ) {
      MacroAssembler _masm(&cbuf);
      int framesize = ra_->C->frame_slots() << LogBytesPerInt;
      Register temp_reg = G3;
      __ add(SP, framesize, temp_reg);
      __ cmp(temp_reg, FP);
      __ breakpoint_trap(Assembler::notEqual, Assembler::ptr_cc);
    }
  }
  {
#ifndef _LP64
    if (returns_long()) {
      //    sllx  O0,32,O0
      emit3_simm13( cbuf, Assembler::arith_op, R_O0_enc, Assembler::sllx_op3, R_O0_enc, 0x1020 );
      //    srl   O1,0,O1
      emit3_simm13( cbuf, Assembler::arith_op, R_O1_enc, Assembler::srl_op3, R_O1_enc, 0x0000 );
      //    or    O0,O1,G1
      emit3       ( cbuf, Assembler::arith_op, R_G1_enc, Assembler:: or_op3, R_O0_enc, 0, R_O1_enc );
    }
#endif
  }
}
void  CallLeafNoFPDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // CALL directly to the runtime
    // The user of this is responsible for ensuring that R_L7 is empty (killed).
    emit_call_reloc(cbuf, opnd_array(1)->method(), relocInfo::runtime_call_type,
                    /*preserve_g2=*/true, /*force far call*/true);
  }
  {
    if( VerifyStackAtCalls ) {
      MacroAssembler _masm(&cbuf);
      int framesize = ra_->C->frame_slots() << LogBytesPerInt;
      Register temp_reg = G3;
      __ add(SP, framesize, temp_reg);
      __ cmp(temp_reg, FP);
      __ breakpoint_trap(Assembler::notEqual, Assembler::ptr_cc);
    }
  }
  {
#ifndef _LP64
    if (returns_long()) {
      //    sllx  O0,32,O0
      emit3_simm13( cbuf, Assembler::arith_op, R_O0_enc, Assembler::sllx_op3, R_O0_enc, 0x1020 );
      //    srl   O1,0,O1
      emit3_simm13( cbuf, Assembler::arith_op, R_O1_enc, Assembler::srl_op3, R_O1_enc, 0x0000 );
      //    or    O0,O1,G1
      emit3       ( cbuf, Assembler::arith_op, R_G1_enc, Assembler:: or_op3, R_O0_enc, 0, R_O1_enc );
    }
#endif
  }
}
void  TailCalljmpIndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_jmpl(cbuf, opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  RetNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  // User did not define which encode class to use.
}
uint  RetNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  tailjmpIndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_jmpl_set_exception_pc(cbuf, opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  CreateExceptionNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  RethrowExceptionNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    cbuf.set_inst_mark();
    Register temp_reg = G3;
    Address rethrow_stub(temp_reg, OptoRuntime::rethrow_stub());
    assert(temp_reg != reg_to_register_object(R_I0_num), "temp must not break oop_reg");
    MacroAssembler _masm(&cbuf);
#ifdef ASSERT
    __ save_frame(0);
    Address last_rethrow_addr(L1, (address)&last_rethrow);
    __ sethi(last_rethrow_addr);
    __ get_pc(L2);
    __ inc(L2, 3 * BytesPerInstWord);  // skip this & 2 more insns to point at jump_to
    __ st_ptr(L2, last_rethrow_addr);
    __ restore();
#endif
    __ JUMP(rethrow_stub, 0); // sethi;jmp
    __ delayed()->nop();
  }
}
void  ShouldNotReachHereNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    emit_illtrap(cbuf);
  }
}
uint  ShouldNotReachHereNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  partialSubtypeCheckNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    __ call(StubRoutines::Sparc::partial_subtype_check(), relocInfo::runtime_call_type);
    __ delayed()->nop();
  }
}
void  partialSubtypeCheck_vs_zeroNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    __ call(StubRoutines::Sparc::partial_subtype_check(), relocInfo::runtime_call_type);
    __ delayed()->nop();
  }
}
void  cmpFastLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

    Register Roop  = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rbox  = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rscratch = reg_to_register_object(R_O7_enc);
    Register Rmark =    reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    assert(Roop  != Rscratch, "");
    assert(Roop  != Rmark, "");
    assert(Rbox  != Rscratch, "");
    assert(Rbox  != Rmark, "");

    __ compiler_lock_object(Roop, Rmark, Rbox, Rscratch, _counters, UseBiasedLocking && !UseOptoBiasInlining);
}
}
uint  cmpFastLockNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4*112, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4*112);
}
void  cmpFastUnlockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

    Register Roop  = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register Rbox  = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register Rscratch = reg_to_register_object(R_O7_enc);
    Register Rmark =    reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));

    assert(Roop  != Rscratch, "");
    assert(Roop  != Rmark, "");
    assert(Rbox  != Rscratch, "");
    assert(Rbox  != Rmark, "");

    __ compiler_unlock_object(Roop, Rmark, Rbox, Rscratch, UseBiasedLocking && !UseOptoBiasInlining);
  }
}
uint  cmpFastUnlockNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4*120, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4*120);
}
void  clear_arrayNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Register    nof_bytes_arg   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register    nof_bytes_tmp    = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3));
    Register    base_pointer_arg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));

    Label loop;
    __ mov(nof_bytes_arg, nof_bytes_tmp);

    // Loop and clear, walking backwards through the array.
    // nof_bytes_tmp (if >0) is always the number of bytes to zero
    __ bind(loop);
    __ deccc(nof_bytes_tmp, 8);
    __ br(Assembler::greaterEqual, true, Assembler::pt, loop);
    __ delayed()-> stx(G0, base_pointer_arg, nof_bytes_tmp);
    // %%%% this mini-loop must not cross a cache boundary!
  }
}
void  string_compareNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    Label Ldone, Lloop;
    MacroAssembler _masm(&cbuf);

    Register   str1_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1));
    Register   str2_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2));
    Register   tmp1_reg = reg_to_register_object(R_G3_enc);
    Register   tmp2_reg = reg_to_register_object(R_G4_enc);
    Register result_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this));

    // Get the first character position in both strings
    //         [8] char array, [12] offset, [16] count
    int  value_offset = java_lang_String:: value_offset_in_bytes();
    int offset_offset = java_lang_String::offset_offset_in_bytes();
    int  count_offset = java_lang_String:: count_offset_in_bytes();

    // load str1 (jchar*) base address into tmp1_reg
    __ load_heap_oop(Address(str1_reg, 0,  value_offset), tmp1_reg);
    __ ld(Address(str1_reg, 0, offset_offset), result_reg);
    __ add(tmp1_reg, arrayOopDesc::base_offset_in_bytes(T_CHAR), tmp1_reg);
    __    ld(Address(str1_reg, 0, count_offset), str1_reg); // hoisted
    __ sll(result_reg, exact_log2(sizeof(jchar)), result_reg);
    __    load_heap_oop(Address(str2_reg, 0,  value_offset), tmp2_reg); // hoisted
    __ add(result_reg, tmp1_reg, tmp1_reg);

    // load str2 (jchar*) base address into tmp2_reg
    // __ ld_ptr(Address(str2_reg, 0,  value_offset), tmp2_reg); // hoisted
    __ ld(Address(str2_reg, 0, offset_offset), result_reg);
    __ add(tmp2_reg, arrayOopDesc::base_offset_in_bytes(T_CHAR), tmp2_reg);
    __    ld(Address(str2_reg, 0, count_offset), str2_reg); // hoisted
    __ sll(result_reg, exact_log2(sizeof(jchar)), result_reg);
    __   subcc(str1_reg, str2_reg, O7); // hoisted
    __ add(result_reg, tmp2_reg, tmp2_reg);

    // Compute the minimum of the string lengths(str1_reg) and the
    // difference of the string lengths (stack)

    // discard string base pointers, after loading up the lengths
    // __ ld(Address(str1_reg, 0, count_offset), str1_reg); // hoisted
    // __ ld(Address(str2_reg, 0, count_offset), str2_reg); // hoisted

    // See if the lengths are different, and calculate min in str1_reg.
    // Stash diff in O7 in case we need it for a tie-breaker.
    Label Lskip;
    // __ subcc(str1_reg, str2_reg, O7); // hoisted
    __ sll(str1_reg, exact_log2(sizeof(jchar)), str1_reg); // scale the limit
    __ br(Assembler::greater, true, Assembler::pt, Lskip);
    // str2 is shorter, so use its count:
    __ delayed()->sll(str2_reg, exact_log2(sizeof(jchar)), str1_reg); // scale the limit
    __ bind(Lskip);

    // reallocate str1_reg, str2_reg, result_reg
    // Note:  limit_reg holds the string length pre-scaled by 2
    Register limit_reg =   str1_reg;
    Register  chr2_reg =   str2_reg;
    Register  chr1_reg = result_reg;
    // tmp{12} are the base pointers

    // Is the minimum length zero?
    __ cmp(limit_reg, (int)(0 * sizeof(jchar))); // use cast to resolve overloading ambiguity
    __ br(Assembler::equal, true, Assembler::pn, Ldone);
    __ delayed()->mov(O7, result_reg);  // result is difference in lengths

    // Load first characters
    __ lduh(tmp1_reg, 0, chr1_reg);
    __ lduh(tmp2_reg, 0, chr2_reg);

    // Compare first characters
    __ subcc(chr1_reg, chr2_reg, chr1_reg);
    __ br(Assembler::notZero, false, Assembler::pt,  Ldone);
    assert(chr1_reg == result_reg, "result must be pre-placed");
    __ delayed()->nop();

    {
      // Check after comparing first character to see if strings are equivalent
      Label LSkip2;
      // Check if the strings start at same location
      __ cmp(tmp1_reg, tmp2_reg);
      __ brx(Assembler::notEqual, true, Assembler::pt, LSkip2);
      __ delayed()->nop();

      // Check if the length difference is zero (in O7)
      __ cmp(G0, O7);
      __ br(Assembler::equal, true, Assembler::pn, Ldone);
      __ delayed()->mov(G0, result_reg);  // result is zero

      // Strings might not be equal
      __ bind(LSkip2);
    }

    __ subcc(limit_reg, 1 * sizeof(jchar), chr1_reg);
    __ br(Assembler::equal, true, Assembler::pn, Ldone);
    __ delayed()->mov(O7, result_reg);  // result is difference in lengths

    // Shift tmp1_reg and tmp2_reg to the end of the arrays, negate limit
    __ add(tmp1_reg, limit_reg, tmp1_reg);
    __ add(tmp2_reg, limit_reg, tmp2_reg);
    __ neg(chr1_reg, limit_reg);  // limit = -(limit-2)

    // Compare the rest of the characters
    __ lduh(tmp1_reg, limit_reg, chr1_reg);
    __ bind(Lloop);
    // __ lduh(tmp1_reg, limit_reg, chr1_reg); // hoisted
    __ lduh(tmp2_reg, limit_reg, chr2_reg);
    __ subcc(chr1_reg, chr2_reg, chr1_reg);
    __ br(Assembler::notZero, false, Assembler::pt, Ldone);
    assert(chr1_reg == result_reg, "result must be pre-placed");
    __ delayed()->inccc(limit_reg, sizeof(jchar));
    // annul LDUH if branch is not taken to prevent access past end of string
    __ br(Assembler::notZero, true, Assembler::pt, Lloop);
    __ delayed()->lduh(tmp1_reg, limit_reg, chr1_reg); // hoisted

    // If strings are equal up to min length, return the length difference.
    __ mov(O7, result_reg);

    // Otherwise, return the difference between the first mismatched chars.
    __ bind(Ldone);
  }
}
void  bytes_reverse_intNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg_asi(cbuf, this, (Assembler::lduwa_op3 /*primary()*/), -1,
                     opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this), Assembler::ASI_PRIMARY_LITTLE);
  }
}
uint  bytes_reverse_intNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  bytes_reverse_longNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_form3_mem_reg_asi(cbuf, this, (Assembler::ldxa_op3 /*primary()*/), -1,
                     opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this), Assembler::ASI_PRIMARY_LITTLE);
  }
}
uint  bytes_reverse_longNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  loadI_reversedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg_asi(cbuf, this, (Assembler::lduwa_op3 /*primary()*/), -1,
                     opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this), Assembler::ASI_PRIMARY_LITTLE);
  }
}
uint  loadI_reversedNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  loadL_reversedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_form3_mem_reg_asi(cbuf, this, (Assembler::ldxa_op3 /*primary()*/), -1,
                     opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(0)->reg(ra_,this), Assembler::ASI_PRIMARY_LITTLE);
  }
}
uint  loadL_reversedNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  storeI_reversedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg_asi(cbuf, this, (Assembler::stwa_op3 /*primary()*/), -1,
                     opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2), Assembler::ASI_PRIMARY_LITTLE);
  }
}
uint  storeI_reversedNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
void  storeL_reversedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_form3_mem_reg_asi(cbuf, this, (Assembler::stxa_op3 /*primary()*/), -1,
                     opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2), Assembler::ASI_PRIMARY_LITTLE);
  }
}
uint  storeL_reversedNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 8);
}
const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUCLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadCNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_unalignedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA4CNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA4SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA2INode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI_fregNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_unalignedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeB0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeCM0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeC0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeI0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeL0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeI_FregNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeP0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeN0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA8B0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeA4CNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA4C0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeA2INode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA2I0Node::memory_operand() const { return _opnds[1]; }
const TypePtr *membar_acquireNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquireNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_acquire_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquire_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_releaseNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_releaseNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_release_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_release_lockNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2D_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2F_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI_reversedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_reversedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeI_reversedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL_reversedNode::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    false,  // AbsI
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // AryEq
    false,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    false,  // CProj
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    true ,  // CMoveN
    true ,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    true ,  // CompareAndSwapN
    false,  // Con
    true ,  // ConN
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    false,  // CosD
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    true ,  // CreateEx
    true ,  // DecodeN
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    false,  // DivModI
    false,  // DivModL
    true ,  // EncodeP
    false,  // ExpD
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // Goto
    true ,  // Halt
    true ,  // If
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    true ,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadC
    true ,  // LoadD
    true ,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    true ,  // LoadNKlass
    true ,  // LoadL
    true ,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadLLocked
    true ,  // LoadP
    true ,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    false,  // LogD
    false,  // Log10D
    false,  // Loop
    false,  // Mach
    false,  // MachProj
    true ,  // MaxI
    true ,  // MemBarAcquire
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // MemBarVolatile
    false,  // MergeMem
    true ,  // MinI
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    false,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // Opaque1
    false,  // Opaque2
    true ,  // OrI
    true ,  // OrL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // Phi
    false,  // PowD
    true ,  // PrefetchRead
    true ,  // PrefetchWrite
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    true ,  // RoundDouble
    true ,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    false,  // SCMemProj
    false,  // SinD
    true ,  // SqrtD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StoreN
    true ,  // StrComp
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    false,  // TanD
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    false,  // AddVB
    false,  // AddVC
    false,  // AddVS
    false,  // AddVI
    false,  // AddVL
    false,  // AddVF
    false,  // AddVD
    false,  // SubVB
    false,  // SubVC
    false,  // SubVS
    false,  // SubVI
    false,  // SubVL
    false,  // SubVF
    false,  // SubVD
    false,  // MulVF
    false,  // MulVD
    false,  // DivVF
    false,  // DivVD
    false,  // LShiftVB
    false,  // LShiftVC
    false,  // LShiftVS
    false,  // LShiftVI
    false,  // URShiftVB
    false,  // URShiftVC
    false,  // URShiftVS
    false,  // URShiftVI
    false,  // AndV
    false,  // OrV
    false,  // XorV
    false,  // VectorLoad
    false,  // Load16B
    true ,  // Load8B
    false,  // Load4B
    false,  // Load8C
    true ,  // Load4C
    false,  // Load2C
    false,  // Load8S
    true ,  // Load4S
    false,  // Load2S
    false,  // Load4I
    true ,  // Load2I
    false,  // Load2L
    false,  // Load4F
    false,  // Load2F
    false,  // Load2D
    false,  // VectorStore
    false,  // Store16B
    true ,  // Store8B
    false,  // Store4B
    false,  // Store8C
    true ,  // Store4C
    false,  // Store2C
    false,  // Store4I
    true ,  // Store2I
    false,  // Store2L
    false,  // Store4F
    false,  // Store2F
    false,  // Store2D
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackC
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2x1B
    false,  // Pack2x2B
    false,  // Replicate16B
    true ,  // Replicate8B
    false,  // Replicate4B
    false,  // Replicate8S
    true ,  // Replicate4S
    false,  // Replicate2S
    false,  // Replicate8C
    true ,  // Replicate4C
    false,  // Replicate2C
    false,  // Replicate4I
    true ,  // Replicate2I
    false,  // Replicate2L
    false,  // Replicate4F
    false,  // Replicate2F
    false,  // Replicate2D
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractS
    false,  // ExtractC
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false   // ExtractD
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 2; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::Name(R_I7_num); }

uint Compile::in_preserve_stack_slots() { return 0; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return 12; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

    (void) SharedRuntime::java_calling_convention(sig_bt, regs, length, is_outgoing);

  
}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, length);
  
}

OptoRegPair Matcher::return_value(int ideal_reg, bool is_outgoing) {

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
#ifdef     _LP64
    static int lo_out[Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, R_O0_num,     R_O0_num,     R_O0_num,     R_F0_num,     R_F0_num, R_O0_num };
    static int hi_out[Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, R_O0H_num,    OptoReg::Bad, R_F1_num, R_O0H_num};
    static int lo_in [Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, R_I0_num,     R_I0_num,     R_I0_num,     R_F0_num,     R_F0_num, R_I0_num };
    static int hi_in [Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, R_I0H_num,    OptoReg::Bad, R_F1_num, R_I0H_num};
#else  // !_LP64
    static int lo_out[Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, R_O0_num,     R_O0_num,     R_O0_num,     R_F0_num,     R_F0_num, R_G1_num };
    static int hi_out[Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, R_F1_num, R_G1H_num};
    static int lo_in [Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, R_I0_num,     R_I0_num,     R_I0_num,     R_F0_num,     R_F0_num, R_G1_num };
    static int hi_in [Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, R_F1_num, R_G1H_num};
#endif
    return OptoRegPair( (is_outgoing?hi_out:hi_in)[ideal_reg],
                        (is_outgoing?lo_out:lo_in)[ideal_reg] );
  
}

OptoRegPair Matcher::c_return_value(int ideal_reg, bool is_outgoing) {

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
#ifdef     _LP64
    static int lo_out[Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, R_O0_num,     R_O0_num,     R_O0_num,     R_F0_num,     R_F0_num, R_O0_num };
    static int hi_out[Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, R_O0H_num,    OptoReg::Bad, R_F1_num, R_O0H_num};
    static int lo_in [Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, R_I0_num,     R_I0_num,     R_I0_num,     R_F0_num,     R_F0_num, R_I0_num };
    static int hi_in [Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, R_I0H_num,    OptoReg::Bad, R_F1_num, R_I0H_num};
#else  // !_LP64
    static int lo_out[Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, R_O0_num,     R_O0_num,     R_O0_num,     R_F0_num,     R_F0_num, R_G1_num };
    static int hi_out[Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, R_F1_num, R_G1H_num };
    static int lo_in [Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, R_I0_num,     R_I0_num,     R_I0_num,     R_F0_num,     R_F0_num, R_G1_num };
    static int hi_in [Op_RegL+1] = { OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, R_F1_num, R_G1H_num };
#endif
    return OptoRegPair( (is_outgoing?hi_out:hi_in)[ideal_reg],
                        (is_outgoing?lo_out:lo_in)[ideal_reg] );
  
}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(R_G5_num); }

const RegMask &Matcher::inline_cache_reg_mask() { return INLINE_CACHE_REG_mask; }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(R_G5_num); }

const RegMask &Matcher::interpreter_method_oop_reg_mask() { return INTERPRETER_METHOD_OOP_REG_mask; }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Bad; }

const RegMask &Matcher::interpreter_frame_pointer_reg_mask() { static RegMask dummy; return dummy; }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(R_SP_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DSPARC=1
#ifndef SPARC
#  error "SPARC must be defined"
#endif // SPARC
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
